/***********************************
   Alumna: Cristina Iriarte Piqué
   Base de dades: Oracle 11g
   TFC: Control consum energètic
***********************************/

------------------------------
-- SECCIÓ CREACIÓ DE TAULES --
------------------------------

-- Taula PAIS
---------------------------------------
CREATE TABLE PAIS(
    ID_PAIS VARCHAR2(2 CHAR) CONSTRAINT NN_PAIS_ID NOT NULL,
    DESCRIPCIÓ VARCHAR2(20 CHAR),
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_ID_PAIS PRIMARY KEY (ID_PAIS)
);
/
-- Taula MODEL_COMPTADORS
---------------------------------------
CREATE TABLE MODEL_COMPTADORS(
    ID_MODEL_COMPTADOR VARCHAR2(15 CHAR) CONSTRAINT NN_MOD_COMPT_ID NOT NULL, 
    MARCA VARCHAR2(20 CHAR),
    DATA_FABRICACIO DATE,     
    EMPRESA_FABRICACIO VARCHAR2(30 CHAR),
    TELF_EMPRESA VARCHAR2(20 CHAR),
    DOMICILI_EMPRESA VARCHAR2(75 CHAR),
    CODI_POSTAL VARCHAR2(15 CHAR),
    LOCALITAT VARCHAR2(30 CHAR),
    PROVINCIA VARCHAR2(30 CHAR),
    ID_PAIS VARCHAR2(2 CHAR),
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_MOD_COMPT_ID PRIMARY KEY (ID_MODEL_COMPTADOR),
    CONSTRAINT FK_MOD_COMPT_PAIS FOREIGN KEY (ID_PAIS) REFERENCES PAIS (ID_PAIS)
);     
/     
-- Taula COMPTADORS
---------------------------------------
CREATE TABLE COMPTADORS(
    ID_COMPTADOR NUMBER CONSTRAINT NN_COMPT_ID NOT NULL,
    ID_MODEL_COMPTADOR VARCHAR2(15 CHAR),
    NUM_SERIE VARCHAR2(20 CHAR),
    TIPUS_SUMINISTRE VARCHAR2(20 CHAR),
    TIPUS_LECTURA VARCHAR2(20) CONSTRAINT NN_TIPUS NOT NULL,
    KHW_CONSUMITS NUMBER,
    DOMICILI VARCHAR2(75 CHAR),
    CODI_POSTAL VARCHAR2(15 CHAR),
    LOCALITAT VARCHAR2(30 CHAR),
    PROVINCIA VARCHAR2(30 CHAR),
    ID_PAIS VARCHAR2(2 CHAR),
    DATA_INSTAL DATE,
    EMPRESA_INSTAL VARCHAR2(30 CHAR),
    TELF_EMPRESA VARCHAR2(15 CHAR),
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_COMPT_ID PRIMARY KEY (ID_COMPTADOR),
    CONSTRAINT FK_COMPT_MODEL FOREIGN KEY (ID_MODEL_COMPTADOR) REFERENCES MODEL_COMPTADORS (ID_MODEL_COMPTADOR),
    CONSTRAINT FK_COMPT_PAIS FOREIGN KEY (ID_PAIS) REFERENCES PAIS (ID_PAIS),
    CONSTRAINT CH_COMPT_TIPUS_SUM CHECK(TIPUS_SUMINISTRE IN ('MONOFÀSIC','TRIFÀSIC')),
    CONSTRAINT CH_COMPT_TIPUS_LECTURA CHECK(TIPUS_LECTURA IN ('TELEMÀTICA','PRESENCIAL'))
);
/
-- Taula CONTROL_LECTURES
---------------------------------------
CREATE TABLE CONTROL_LECTURES(
    ID_COMPTADOR NUMBER CONSTRAINT NN_CONTROL_LECT_ID NOT NULL,
    DATA_LECTURA DATE CONSTRAINT NN_CONTROL_LECT_DATA NOT NULL,
    CONSUM NUMBER CONSTRAINT NN_CONSUM_CL NOT NULL,
    TIPUS_LECTURA VARCHAR2(20) CONSTRAINT NN_TIPUS_CL NOT NULL,
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_CONTROL_LECT_ID PRIMARY KEY (ID_COMPTADOR,DATA_LECTURA,CONSUM),
    CONSTRAINT FK_CONTROL_LECT_COMP FOREIGN KEY (ID_COMPTADOR) REFERENCES COMPTADORS (ID_COMPTADOR),     
    CONSTRAINT CH_CONTROL_LECT_TIPUS_CL CHECK(TIPUS_LECTURA IN ('TELEMÀTICA','PRESENCIAL'))
);
/
-- Taula HIST_COMPTADORS
---------------------------------------
CREATE TABLE HIST_COMPTADORS(
    ID_COMPTADOR NUMBER CONSTRAINT NN_HIST_COMPT_ID NOT NULL,
    DATA_MENSUAL DATE CONSTRAINT NN_HIST_COMPT_DATA NOT NULL,
    CONSUM NUMBER CONSTRAINT NN_HIST_COM_CONSUM NOT NULL,
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_HIST_COMP_ID_DATA PRIMARY KEY (ID_COMPTADOR, DATA_MENSUAL),
    CONSTRAINT FK_HIST_COMP_COMP FOREIGN KEY (ID_COMPTADOR) REFERENCES COMPTADORS (ID_COMPTADOR)
);
/
-- Taula CLIENTS
---------------------------------------
CREATE TABLE CLIENTS(
    ID_CLIENT NUMBER CONSTRAINT NN_CLIENTS_ID NOT NULL,
    NOM VARCHAR2(50 CHAR),
    PRIMER_COGNOM VARCHAR2(50 CHAR),
    SEGON_COGNOM VARCHAR2(50 CHAR),
    DATA_NAIXEMENT DATE,
    NIF VARCHAR2(10 CHAR) CONSTRAINT U_NIF UNIQUE,
    TIPUS_CLIENT VARCHAR2(20 CHAR),
    PROFESSIO VARCHAR2(30 CHAR),
    TELEFON_1 VARCHAR2(15 CHAR),
    TELEFON_2 VARCHAR2(15 CHAR),
    EMAIL VARCHAR2(50 CHAR),
    DOMICILI VARCHAR2(75 CHAR),
    CODI_POSTAL VARCHAR2(15 CHAR),
    LOCALITAT VARCHAR2(30 CHAR),
    PROVINCIA VARCHAR2(30 CHAR),
    ID_PAIS VARCHAR2(2 CHAR),
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_CLIENTS_ID PRIMARY KEY (ID_CLIENT),
    CONSTRAINT FK_CLIENTS_PAIS FOREIGN KEY (ID_PAIS) REFERENCES PAIS (ID_PAIS)
);
/
-- Taula CONTRACTES
---------------------------------------
CREATE TABLE CONTRACTES(
    ID_CONTRACTE NUMBER CONSTRAINT NN_CONTRACT_ID NOT NULL,
    ID_CLIENT NUMBER CONSTRAINT NN_CONTRACT_CLIENT NOT NULL,
    ID_COMPTADOR NUMBER CONSTRAINT NN_CONTRACT_COMPT NOT NULL,
    CUPS VARCHAR2(25 CHAR),
    POTENCIA_CONTRACTADA NUMBER,
    TARIFA_ACCES VARCHAR2(10 CHAR),
    DATA_FI_CONTRACTE DATE, 
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_CONTRACT_ID PRIMARY KEY (ID_CONTRACTE),
    CONSTRAINT FK_CONTRACT_CLIENT FOREIGN KEY (ID_CLIENT) REFERENCES CLIENTS (ID_CLIENT),
    CONSTRAINT FK_CONTRACT_COMPT FOREIGN KEY (ID_COMPTADOR) REFERENCES COMPTADORS (ID_COMPTADOR),
    CONSTRAINT FK_CONTRACT_COMPT_UN UNIQUE (ID_COMPTADOR)
);
/
-- Taula CENTRALS_PRODUCCIO
---------------------------------------
CREATE TABLE CENTRALS_PRODUCCIO(
    ID_CENT_PROD NUMBER CONSTRAINT NN_CENT_PROD_ID NOT NULL,
    TIPUS_CENTRAL VARCHAR2(50 CHAR),
    ENERGIA_MIN NUMBER,
    REBUIG NUMBER, 
    EMISSIONS NUMBER,
    MOLINS NUMBER,
    PANELLS NUMBER,
    ENERGIA_MAX_PROD NUMBER,
    DATA_ULT_INSP_TECNICA DATE,
    TELEFON VARCHAR2(15 CHAR),
    DOMICILI VARCHAR2(75 CHAR),
    CODI_POSTAL VARCHAR2(15 CHAR),
    LOCALITAT VARCHAR2(30 CHAR),
    PROVINCIA VARCHAR2(30 CHAR),
    ID_PAIS VARCHAR2(2 CHAR),
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_CENT_PROD_ID PRIMARY KEY (ID_CENT_PROD),
    CONSTRAINT FK_CENT_PROD_PAIS FOREIGN KEY (ID_PAIS) REFERENCES PAIS (ID_PAIS),
    CONSTRAINT CH_CENT_PROD_TIPUS CHECK(TIPUS_CENTRAL IN ('NUCLEAR','TÈRMICA','CARBÓ','EÒLICA','SOLAR'))
);
/
-- Taula HIST_CENTRALS_PRODUCCIO
---------------------------------------
CREATE TABLE HIST_CENTRALS_PRODUCCIO(
    ID_CENT_PROD NUMBER CONSTRAINT NN_HIST_CENT_PROD_ID NOT NULL,
    DATA_MENSUAL DATE CONSTRAINT NN_HIST_CENT_PROD_DATA NOT NULL,
    ENERGIA_PROD NUMBER CONSTRAINT NN_HIST_CENT_PROD_ENERG NOT NULL,
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_HIST_CENT_PROD_ID_DATA PRIMARY KEY (ID_CENT_PROD, DATA_MENSUAL),
    CONSTRAINT FK_HIST_CENT_PROD_ID FOREIGN KEY (ID_CENT_PROD) REFERENCES CENTRALS_PRODUCCIO (ID_CENT_PROD)
);
/
-- Taula CENTRALETES_DISTRIBUCIO
---------------------------------------
CREATE TABLE CENTRALETES_DISTRIBUCIO(
    ID_CENTL_DIST VARCHAR2(100 CHAR),
    ENERGIA_MAX_SUB NUMBER,
    DATA_ULT_INSP_TECNICA DATE,
    DOMICILI VARCHAR2(75 CHAR),
    CODI_POSTAL VARCHAR2(15 CHAR) CONSTRAINT NN_CENTL_DIST_CP NOT NULL,
    LOCALITAT VARCHAR2(30 CHAR),
    PROVINCIA VARCHAR2(30 CHAR),
    ID_PAIS VARCHAR2(2 CHAR),
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_CENTL_DIST_ID PRIMARY KEY (ID_CENTL_DIST),
    CONSTRAINT FK_CENTL_DIST_PAIS FOREIGN KEY (ID_PAIS) REFERENCES PAIS (ID_PAIS)
);
/

-- Taula CON_DISTRIBUCIO_COMPTADOR
---------------------------------------
CREATE TABLE CON_DISTRIBUCIO_COMPTADOR(
    ID_COMPTADOR NUMBER CONSTRAINT NN_CON_DIST_COMP_ID_COMT NOT NULL,
    ID_CENTL_DIST VARCHAR2(100 CHAR) CONSTRAINT NN_CON_DIST_COMP_ID_DIST NOT NULL,
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_CON_DIST_COMP_ID PRIMARY KEY (ID_COMPTADOR),
    CONSTRAINT FK_CON_DIST_COMP_COMP FOREIGN KEY (ID_COMPTADOR) REFERENCES COMPTADORS (ID_COMPTADOR),     
    CONSTRAINT FK_CON_DIST_COMP_DIST FOREIGN KEY (ID_CENTL_DIST) REFERENCES CENTRALETES_DISTRIBUCIO (ID_CENTL_DIST)
);
/
-- Taula LINIES_COMUNICACIO
---------------------------------------
CREATE TABLE LINIES_COMUNICACIO(
    ID_LINIA_COM NUMBER CONSTRAINT NN_LIN_COM_ID NOT NULL,
    TRANSPORT_MAX NUMBER CONSTRAINT NN_LIN_COM_TRANS_MAX NOT NULL,
    PROVINCIA VARCHAR2(30 CHAR),
    ID_PAIS VARCHAR2(2 CHAR),
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_LIN_COM_ID PRIMARY KEY (ID_LINIA_COM),
    CONSTRAINT FK_LIN_COM_PAIS FOREIGN KEY (ID_PAIS) REFERENCES PAIS (ID_PAIS)
);
/
-- Taula CON_PRODUCCIO_DISTRIBUCIO
---------------------------------------
CREATE TABLE CON_PRODUCCIO_DISTRIBUCIO(
    ID_LINIA_COM NUMBER CONSTRAINT NN_CON_PROD_DIST_ID NOT NULL,
    ID_CENT_PROD NUMBER CONSTRAINT NN_CON_PROD_DIST_PROD NOT NULL,
    ID_CENTL_DIST VARCHAR2(100 CHAR) CONSTRAINT NN_CON_PROD_DIST_DIST NOT NULL,
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_CON_PROD_DIST_ID PRIMARY KEY (ID_LINIA_COM, ID_CENT_PROD),
    CONSTRAINT FK_CON_PROD_DIST_LIN FOREIGN KEY (ID_LINIA_COM) REFERENCES LINIES_COMUNICACIO (ID_LINIA_COM),     
    CONSTRAINT FK_CON_PROD_DIST_PROD FOREIGN KEY (ID_CENT_PROD) REFERENCES CENTRALS_PRODUCCIO (ID_CENT_PROD),     
    CONSTRAINT FK_CON_PROD_DIST_DIST FOREIGN KEY (ID_CENTL_DIST) REFERENCES CENTRALETES_DISTRIBUCIO (ID_CENTL_DIST)
);
/
-- Taula MSG_ERRORS
--------------------------------------
CREATE TABLE MSG_ERRORS(
    ID_ERROR NUMBER CONSTRAINT NN_ERROR_ID NOT NULL,
    DESCRIPCIO VARCHAR2(256 CHAR) CONSTRAINT NN_ERROR_DESC NOT NULL,
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    USUARI_MODIFICACIO VARCHAR2(20 CHAR),
    DATA_MODIFICACIO TIMESTAMP,
    CONSTRAINT PK_ERRORS_ID PRIMARY KEY (ID_ERROR)
);
/
-- Taula LOG
--------------------------------------
CREATE TABLE LOGS(
    ID_LOG NUMBER,
    PROCEDIMENT VARCHAR2(50 CHAR),
    PARAM_ENTRADA VARCHAR2(500 CHAR),
    PARAM_SORTIDA VARCHAR2(500 CHAR),
    USUARI_CREACIO VARCHAR2(20 CHAR),
    DATA_CREACIO TIMESTAMP,
    RSP VARCHAR2(500),
    CONSTRAINT PK_LOG_ID PRIMARY KEY (ID_LOG)
);
/    
-- Taula ESTADISTICA_01
---------------------------------------
CREATE TABLE ESTADISTICA_01(
    ID_CENT_PROD NUMBER CONSTRAINT NN_EST01_ID_CENT_PROD NOT NULL,
    CONSUM_COMPT NUMBER,
    USUARI_ACTUALITZACIO VARCHAR2(20 CHAR),
    DATA_ACTUALITZACIO TIMESTAMP,
    CONSTRAINT PK_EST01_ID PRIMARY KEY (ID_CENT_PROD),
    CONSTRAINT FK_EST01_PROD FOREIGN KEY (ID_CENT_PROD) REFERENCES CENTRALS_PRODUCCIO (ID_CENT_PROD)     
);
/
-- Taula ESTADISTICA_02
---------------------------------------
CREATE TABLE ESTADISTICA_02(
    ID_LINIA_COM NUMBER CONSTRAINT NN_EST02_ID_LIN_COM NOT NULL,
    PERIODE_ANUAL NUMBER CONSTRAINT NN_EST02_PERIODE NOT NULL,
    CONSUM_MITJA NUMBER CONSTRAINT NN_EST02_CONSUM NOT NULL,
    USUARI_ACTUALITZACIO VARCHAR2(20 CHAR),
    DATA_ACTUALITZACIO TIMESTAMP,
    CONSTRAINT PK_EST02_ID PRIMARY KEY (ID_LINIA_COM,PERIODE_ANUAL),
    CONSTRAINT FK_EST02_LIN FOREIGN KEY (ID_LINIA_COM) REFERENCES LINIES_COMUNICACIO (ID_LINIA_COM)
    
);
/
-- Taula ESTADISTICA_03
---------------------------------------
CREATE TABLE ESTADISTICA_03(
    ID_LINIA_COM NUMBER CONSTRAINT NN_EST03_ID_LIN_COM NOT NULL,
    TRANSPORT_MAX NUMBER CONSTRAINT NN_EST03_TRANS NOT NULL,
    ENERGIA_CONSUMIDA NUMBER CONSTRAINT NN_EST03_ENERG_CON NOT NULL,
    USUARI_ACTUALITZACIO VARCHAR2(20 CHAR),
    DATA_ACTUALITZACIO TIMESTAMP,
    CONSTRAINT PK_EST03_ID PRIMARY KEY (ID_LINIA_COM),
    CONSTRAINT FK_EST03_LIN FOREIGN KEY (ID_LINIA_COM) REFERENCES LINIES_COMUNICACIO (ID_LINIA_COM)
);
/
-- Taula ESTADISTICA_04
---------------------------------------
CREATE TABLE ESTADISTICA_04(
    PERIODE_ANUAL NUMBER CONSTRAINT NN_EST04_PERIODE NOT NULL,
    PERCENTATGE_LINIES NUMBER,
    USUARI_ACTUALITZACIO VARCHAR2(20 CHAR),
    DATA_ACTUALITZACIO TIMESTAMP,
    CONSTRAINT PK_EST04_ID PRIMARY KEY (PERIODE_ANUAL)
);
/
-- Taula ESTADISTICA_05
---------------------------------------
CREATE TABLE ESTADISTICA_05(
    PERIODE_ANUAL NUMBER CONSTRAINT NN_EST05_PERIODE NOT NULL,
    TOTAL_CENT_PROD NUMBER CONSTRAINT NN_EST05_TOTAL NOT NULL,
    USUARI_ACTUALITZACIO VARCHAR2(20 CHAR),
    DATA_ACTUALITZACIO TIMESTAMP,
    CONSTRAINT PK_EST05_ID PRIMARY KEY (PERIODE_ANUAL)
);
/
-- Taula ESTADISTICA_06
---------------------------------------
CREATE TABLE ESTADISTICA_06(
    ID_COMPTADOR NUMBER CONSTRAINT NN_EST06_ID_COMPT NOT NULL,
    CONSUM NUMBER CONSTRAINT NN_EST06_CONSUM NOT NULL,
    USUARI_ACTUALITZACIO VARCHAR2(20 CHAR),
    DATA_ACTUALITZACIO TIMESTAMP,
    CONSTRAINT PK_EST06_ID PRIMARY KEY (ID_COMPTADOR),
    CONSTRAINT FK_EST06_COMPT FOREIGN KEY (ID_COMPTADOR) REFERENCES COMPTADORS (ID_COMPTADOR)     
);
/
-- Taula ESTADISTICA_07
---------------------------------------
CREATE TABLE ESTADISTICA_07(
    ID_CLIENT NUMBER CONSTRAINT NN_EST07_ID_CLIENT NOT NULL,
    CONSUM_MITJA NUMBER CONSTRAINT NN_EST07_CONSUM NOT NULL,
    USUARI_ACTUALITZACIO VARCHAR2(20 CHAR),
    DATA_ACTUALITZACIO TIMESTAMP,
    CONSTRAINT PK_EST07_ID PRIMARY KEY (ID_CLIENT),
    CONSTRAINT FK_EST07_CLIENT FOREIGN KEY (ID_CLIENT) REFERENCES CLIENTS (ID_CLIENT)     
);
/
-------------------------------
-- SECCIÓ CREACIÓ COMENTARIS --
-------------------------------
COMMENT ON TABLE PAIS IS 'PAÏSOS QUE FORMEN LA UNIÓ EUROPEA';
COMMENT ON TABLE MODEL_COMPTADORS  IS 'DADES RELATIVES A MODELS DE COMPTADORS';
COMMENT ON TABLE COMPTADORS  IS 'DADES RELATIVES A COMPTADORS';
COMMENT ON TABLE CONTROL_LECTURES  IS 'DADES RELATIVES A LES LECTURES EFECTUADES DE COMPTADORS. DATA, VALOR I TIPUS DE LECTURA';
COMMENT ON TABLE HIST_COMPTADORS  IS 'HISTÒRIC DELS CONSUMS DE COMPTADORS';
COMMENT ON TABLE CLIENTS  IS 'DADES RELATIVES A CLIENTS';
COMMENT ON TABLE CONTRACTES IS 'DADES RELATIVES A CONTRACTES';
COMMENT ON TABLE CENTRALS_PRODUCCIO IS 'DADES RELATIVES A CENTRALS DE PRODUCCIÓ';
COMMENT ON TABLE HIST_CENTRALS_PRODUCCIO IS 'HISTÒRIC DE L''ENERGIA PRODUÏDA PER CENTRALS DE PRODUCCIÓ'; 
COMMENT ON TABLE CENTRALETES_DISTRIBUCIO IS 'DADES RELATIVES A CENTRALETES DE DISTRIBUCIÓ';
COMMENT ON TABLE CON_DISTRIBUCIO_COMPTADOR IS 'DADES RELATIVES A LES CONNEXIÓNS ENTRE COMPTADORS I CENTRALETES DE DISTRIBUCIÓ';
COMMENT ON TABLE LINIES_COMUNICACIO IS 'DADES RELATIVES A LES LÍNIES DE COMUNICACIÓ';
COMMENT ON TABLE CON_PRODUCCIO_DISTRIBUCIO IS 'DADES RELATIVES A LES CONNEXIONS ENTRE LÍNIES DE COMUNICACIÓ, CENTRALS DE PRODUCCIÓ I CENTRALS DE DISTRIBUCIÓ';
COMMENT ON TABLE MSG_ERRORS IS 'DECLARACIÓ MISSATGES D''ERROR';
COMMENT ON TABLE ESTADISTICA_01 IS 'CONSUMS DE COMPTADORS DEPENDENTS D''UNA CENTRAL DE PRODUCCIÓ';
COMMENT ON TABLE ESTADISTICA_02 IS 'VALOR MITJÀ DE L''ENERGIA CONSUMIDA D''UNA LÍNIA DE COMUNICACIÓ';
COMMENT ON TABLE ESTADISTICA_03 IS 'LÍNIA DE COMUNICACIÓ MÉS CARREGADA A NIVELL D''ENERGIA CONSUMIDA';
COMMENT ON TABLE ESTADISTICA_04 IS 'PERCENTATGE DE LÍNIES DE COMUNICACIÓ QUE SUPEREN EL 50% D''ENERGIA CONSUMIDA';
COMMENT ON TABLE ESTADISTICA_05 IS 'NOMBRE DE CENTRALS DE PRODUCCIÓ QUE GENEREN MENYS DEL 30% DE PRODUCCIÓ';
COMMENT ON TABLE ESTADISTICA_06 IS 'TOP-10 DE COMPTADORS QUE HISTÒRICAMENT HAN TINGUT MÉS CONSUM';
COMMENT ON TABLE ESTADISTICA_07 IS 'CONSUM MITJÀ DE CLIENTS';
/
--------------------------------
-- SECCIÓ CREACIÓ SEQÜÈNCIES --
--------------------------------

-- Seqüència taula LOG
-------------------------------------
CREATE SEQUENCE SEQ_ID_LOG 
    INCREMENT BY 1 START WITH 1;
/

---------------------------------------------
-- SECCIÓ CREACIÓ DE DISPARADORS (TRIGGERS) --
----------------------------------------------
-- Trigger taula PAIS
---------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_PAIS
    BEFORE INSERT ON PAIS
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_PAIS;
/
CREATE OR REPLACE TRIGGER TRG_UPD_PAIS
    BEFORE UPDATE ON PAIS
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_PAIS;
/

-- Trigger taula MODEL_COMPTADORS
---------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_MODEL_COMPT
    BEFORE INSERT ON MODEL_COMPTADORS
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_MODEL_COMPT;
/
CREATE OR REPLACE TRIGGER TRG_UPD_MODEL_COMPT
    BEFORE UPDATE ON MODEL_COMPTADORS
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_MODEL_COMPT;
/

-- Trigger taula COMPTADORS
---------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_COMPTADORS
    BEFORE INSERT ON COMPTADORS
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_COMPTADORS;
/
CREATE OR REPLACE TRIGGER TRG_UPD_COMPTADORS
    BEFORE UPDATE ON COMPTADORS
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_COMPTADORS;
/

-- Trigger taula CONTROL_LECTURES
------------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_CONT_LECT
    BEFORE INSERT ON CONTROL_LECTURES
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_CONT_LECT;
/
CREATE OR REPLACE TRIGGER TRG_UPD_CONT_LECT
    BEFORE UPDATE ON CONTROL_LECTURES
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_CONT_LECT;
/

-- Trigger taula HIST_COMPTADORS
---------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_HIST_COMPT
    BEFORE INSERT ON HIST_COMPTADORS
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_HIST_COMPT;
/
CREATE OR REPLACE TRIGGER TRG_UPD_HIST_COMPT
    BEFORE UPDATE ON HIST_COMPTADORS
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_HIST_COMPT;
/

-- Trigger taula CLIENTS
---------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_CLIENTS
    BEFORE INSERT ON CLIENTS
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_CLIENTS;
/
CREATE OR REPLACE TRIGGER TRG_UPD_CLIENTS
    BEFORE UPDATE ON CLIENTS
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_CLIENTS;
/
CREATE OR REPLACE TRIGGER TRG_TIPUS_CLIENTS
  BEFORE INSERT OR UPDATE OF NIF ON CLIENTS
  FOR EACH ROW
    BEGIN
      IF :NEW.NIF IS NOT NULL OR
         :OLD.NIF IS NOT NULL OR
         :NEW.NIF <> :OLD.NIF 
     	THEN
           -- PERSONA JURÍDICA
           IF (SUBSTR(:new.nif,1,1) IN ('A','B','C','D','E','F','G','H','J','P','Q','R','S','U','V','N','W'))
           THEN
               :NEW.TIPUS_CLIENT := 'PERSONA JURÍDICA';
           ELSE
             -- PERSONA_FISÍCA
               :NEW.TIPUS_CLIENT := 'PERSONA FÍSICA';
           END IF;
      END IF;
      
      IF :NEW.NIF IS NULL AND :OLD.TIPUS_CLIENT IS NOT NULL
      THEN
         :NEW.TIPUS_CLIENT := '';
      END IF;
END TRG_TIPUS_CLIENTS;
/

-- Trigger taula CONTRACTES
---------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_CONTRACTES
    BEFORE INSERT ON CONTRACTES
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_CONTRACTES;
/
CREATE OR REPLACE TRIGGER TRG_UPD_CONTRACTES
    BEFORE UPDATE ON CONTRACTES
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_CONTRACTES;
/

-- Trigger taula CENTRALS_PRODUCCIO
---------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_CENT_PROD
    BEFORE INSERT ON CENTRALS_PRODUCCIO
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_CENT_PROD;
/
CREATE OR REPLACE TRIGGER TRG_UPD_CENT_PROD
    BEFORE UPDATE ON CENTRALS_PRODUCCIO
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_CENT_PROD;
/

-- Trigger taula HIST_CENTRALS_PRODUCCIO
---------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_HIST_CENT_PROD
    BEFORE INSERT ON HIST_CENTRALS_PRODUCCIO
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_HIST_CENT_PROD;
/
CREATE OR REPLACE TRIGGER TRG_UPD_HIST_CENT_PROD
    BEFORE UPDATE ON HIST_CENTRALS_PRODUCCIO
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_HIST_CENT_PROD;
/

-- Trigger taula CENTRALETES_DISTRIBUCIO
---------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_CENTL_DIST
    BEFORE INSERT ON CENTRALETES_DISTRIBUCIO
    FOR EACH ROW
  	BEGIN
           -- Genera la informació de l'ID
     :NEW.ID_CENTL_DIST := :NEW.ID_PAIS||'-'||:NEW.PROVINCIA||'-'||:NEW.LOCALITAT||'-'||:NEW.CODI_POSTAL||' - '||:NEW.DOMICILI;
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_CENTL_DIST;
/
CREATE OR REPLACE TRIGGER TRG_UPD_CENTL_DIST
    BEFORE UPDATE ON CENTRALETES_DISTRIBUCIO
    FOR EACH ROW
  	BEGIN
           IF (:NEW.DOMICILI <> :OLD.DOMICILI OR :NEW.CODI_POSTAL <> :OLD.CODI_POSTAL OR 
               :NEW.LOCALITAT <> :OLD.LOCALITAT OR :NEW.ID_PAIS <> :OLD.ID_PAIS)
           THEN
               -- Modifica la informació de l'ID
               :NEW.ID_CENTL_DIST := :NEW.ID_PAIS||'-'||:NEW.PROVINCIA||'-'||:NEW.LOCALITAT||'-'||:NEW.CODI_POSTAL||' - '||:NEW.DOMICILI;
           END IF;
           
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_CENTL_DIST;
/

-- Trigger taula CON_DISTRIBUCIO_COMPTADOR
------------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_CON_DIST_COMPT
    BEFORE INSERT ON CON_DISTRIBUCIO_COMPTADOR
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_CON_DIST_COMPT;
/
CREATE OR REPLACE TRIGGER TRG_UPD_CON_DIST_COMPT
    BEFORE UPDATE ON CON_DISTRIBUCIO_COMPTADOR
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_CON_DIST_COMPT;
/


-- Trigger taula LINIES_COMUNICACIO
------------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_LIN_COM
    BEFORE INSERT ON LINIES_COMUNICACIO
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_LIN_COM;
/
CREATE OR REPLACE TRIGGER TRG_UPD_LIN_COM
    BEFORE UPDATE ON LINIES_COMUNICACIO
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_LIN_COM;
/

-- Trigger taula CON_PRODUCCIO_DISTRIBUCIO
------------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_CON_PROD_DIST
    BEFORE INSERT ON CON_PRODUCCIO_DISTRIBUCIO
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_CON_PROD_DIST;
/
CREATE OR REPLACE TRIGGER TRG_UPD_CON_PROD_DIST
    BEFORE UPDATE ON CON_PRODUCCIO_DISTRIBUCIO
    FOR EACH ROW
  	BEGIN
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_CON_PROD_DIST;
/

-- Trigger taula LOGS
------------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_LOGS
    BEFORE INSERT ON LOGS
    FOR EACH ROW
    BEGIN
	   SELECT SEQ_ID_LOG.NEXTVAL INTO :NEW.ID_LOG
	   FROM DUAL;
     
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_LOG;
/

-- Trigger taula MSG__ERRORS
------------------------------------------
CREATE OR REPLACE TRIGGER TRG_INS_ME
BEFORE INSERT ON MSG_ERRORS
    FOR EACH ROW
    BEGIN
	   
	   :NEW.USUARI_CREACIO := USER;
 	   :NEW.DATA_CREACIO := SYSDATE;
END TRG_INS_ME;
/
CREATE OR REPLACE TRIGGER TRG_UPD_ME
BEFORE UPDATE ON MSG_ERRORS
    FOR EACH ROW
    BEGIN
	   
	   :NEW.USUARI_MODIFICACIO := USER;
 	   :NEW.DATA_MODIFICACIO := SYSDATE;
END TRG_UPD_ME;
/

-- Trigger taula ESTADISTICA_01
------------------------------------------
CREATE OR REPLACE TRIGGER TRG_EST01
    BEFORE INSERT OR UPDATE ON ESTADISTICA_01
    FOR EACH ROW
    BEGIN
	   :NEW.USUARI_ACTUALITZACIO := USER;
 	   :NEW.DATA_ACTUALITZACIO := SYSDATE;
END TRG_EST01;
/

-- Trigger taula ESTADISTICA_02
------------------------------------------
CREATE OR REPLACE TRIGGER TRG_EST02
    BEFORE INSERT OR UPDATE ON ESTADISTICA_02
    FOR EACH ROW
    BEGIN
	   :NEW.USUARI_ACTUALITZACIO := USER;
 	   :NEW.DATA_ACTUALITZACIO := SYSDATE;
END TRG_EST02;
/

-- Trigger taula ESTADISTICA_03
------------------------------------------
CREATE OR REPLACE TRIGGER TRG_EST03
    BEFORE INSERT OR UPDATE ON ESTADISTICA_03
    FOR EACH ROW
    BEGIN
	   :NEW.USUARI_ACTUALITZACIO := USER;
 	   :NEW.DATA_ACTUALITZACIO := SYSDATE;
END TRG_EST03;
/

-- Trigger taula ESTADISTICA_04
------------------------------------------
CREATE OR REPLACE TRIGGER TRG_EST04
    BEFORE INSERT OR UPDATE ON ESTADISTICA_04
    FOR EACH ROW
    BEGIN
	   :NEW.USUARI_ACTUALITZACIO := USER;
 	   :NEW.DATA_ACTUALITZACIO := SYSDATE;
END TRG_EST04;
/

-- Trigger taula ESTADISTICA_05
------------------------------------------
CREATE OR REPLACE TRIGGER TRG_EST05
    BEFORE INSERT OR UPDATE ON ESTADISTICA_05
    FOR EACH ROW
    BEGIN
	   :NEW.USUARI_ACTUALITZACIO := USER;
 	   :NEW.DATA_ACTUALITZACIO := SYSDATE;
END TRG_EST05;
/

-- Trigger taula ESTADISTICA_06
------------------------------------------
CREATE OR REPLACE TRIGGER TRG_EST06
    BEFORE INSERT OR UPDATE ON ESTADISTICA_06
    FOR EACH ROW
    BEGIN
	   :NEW.USUARI_ACTUALITZACIO := USER;
 	   :NEW.DATA_ACTUALITZACIO := SYSDATE;
END TRG_EST06;
/

-- Trigger taula ESTADISTICA_07
------------------------------------------
CREATE OR REPLACE TRIGGER TRG_EST07
    BEFORE INSERT OR UPDATE ON ESTADISTICA_07
    FOR EACH ROW
    BEGIN
	   :NEW.USUARI_ACTUALITZACIO := USER;
 	   :NEW.DATA_ACTUALITZACIO := SYSDATE;
END TRG_EST07;
/

COMMIT;
/
