CREATE OR REPLACE PACKAGE PKG_MODUL_ESTADISTIC AS
    /*========================================================================================================================================
    NOM: MODUL_ESTADISTIC_01
    DESCRIPCIÓ: PROCEDIMENT QUE GENERA LA INFORMACIÓ REFERENT A: 
              - DONADA UNA CENTRAL DE PRODUCIÓ, EL CONSUM DELS COMPTADORS QUE DEPENEN DE LA CENTRAL
               A LA TAULA ESTADISTICA_01
    PARAMETERS:
          INPUT: --
          OUTPUT: RSP
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: PKG_MODUL_ESTADISTIC.MODUL_ESTADISTIC_01(RESP);
    ========================================================================================================================================*/
    PROCEDURE MODUL_ESTADISTIC_01 (E_RSP OUT NOCOPY LOGS.RSP%TYPE);
    
    /*========================================================================================================================================
    NOM: MODUL_ESTADISTIC_02
    DESCRIPCIÓ: PROCEDIMENT QUE GENERA LA INFORMACIÓ REFERENT A: 
		           - SEGONS UNA LÍNIA DE COMUNICACIÓ I UN ANY EN CONCRET, EL VALOR MITJÀ DE L'ENERGIA CONSUMIDA, TENINT EN COMPTE QUE AQUEST 
                  CONSUM DEPÈN DELS COMPTADORS QUE S'ALIMENTEN MITJANÇANT AQUESTA LÍNIA, INCLOENT EL CONSUM DE LA SUMA TOTAL DE LES 
                  LÍNIES QUE UTILITZI EL COMPTADOR
               A LA TAULA ESTADISTICA_02
    PARAMETERS:
          INPUT: --
          OUTPUT: RSP
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: PKG_MODUL_ESTADISTIC.MODUL_ESTADISTIC_02(RESP);
    ========================================================================================================================================*/
    PROCEDURE MODUL_ESTADISTIC_02 (E_RSP OUT NOCOPY LOGS.RSP%TYPE);
       
    /*========================================================================================================================================
    NOM: MODUL_ESTADISTIC_03
    DESCRIPCIÓ: PROCEDIMENT QUE GENERA LA INFORMACIÓ REFERENT A: 
            	- LÍNIA QUE HA ESTAT MÉS CARREGADA A NIVELL D'ENERGIA CONSUMIDA
               A LA TAULA ESTADISTICA_03
    PARAMETERS:
          INPUT: --
          OUTPUT: RSP
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: PKG_MODUL_ESTADISTIC.MODUL_ESTADISTIC_03(RESP);
    ========================================================================================================================================*/
    PROCEDURE MODUL_ESTADISTIC_03 (E_RSP OUT NOCOPY LOGS.RSP%TYPE);
    
    /*========================================================================================================================================
    NOM: MODUL_ESTADISTIC_04
    DESCRIPCIÓ: PROCEDIMENT QUE GENERA LA INFORMACIÓ REFERENT A: 
		           - DONAT UN ANY CONCRET, PERCENTATGE DE LÍNIES QUE SUPEREN EL 50% D'ENERGIA CONSUMIDA
               A LA TAULA ESTADISTICA_04
    PARAMETERS: 
          INPUT: --
          OUTPUT: RSP
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: PKG_MODUL_ESTADISTIC.MODUL_ESTADISTIC_04(RESP);
    ========================================================================================================================================*/
    PROCEDURE MODUL_ESTADISTIC_04 (E_RSP OUT NOCOPY LOGS.RSP%TYPE);
    
    /*========================================================================================================================================
    NOM: MODUL_ESTADISTIC_05
    DESCRIPCIÓ: PROCEDIMENT QUE GENERA LA INFORMACIÓ REFERENT A: 
              -  DONAT UN ANY CONCRET: EL NOMBRE DE CENTRALS DE PRODUCCIÓ QUE GENEREN MENYS DEL 30% DE PRODUCCIÓ
               A LA TAULA ESTADISTICA_05
    PARAMETERS: 
          INPUT: ANY
          OUTPUT: RSP
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: PKG_MODUL_ESTADISTIC.MODUL_ESTADISTIC_05(RESP);
    ========================================================================================================================================*/
    PROCEDURE MODUL_ESTADISTIC_05 (E_RSP OUT NOCOPY LOGS.RSP%TYPE);
    
    /*========================================================================================================================================
    NOM: MODUL_ESTADISTIC_06
    DESCRIPCIÓ: PROCEDIMENT QUE GENERA LA INFORMACIÓ REFERENT A: 
		           -  TOP-10 DE COMPTADORS QUE HISTÒRICAMENT HAN TINGUT MÉS CONSUM
               A LA TAULA ESTADISTICA_06
    PARAMETERS: 
          INPUT: --
          OUTPUT: RSP
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: PKG_MODUL_ESTADISTIC.MODUL_ESTADISTIC_06(RESP);
    ========================================================================================================================================*/
    PROCEDURE MODUL_ESTADISTIC_06 (E_RSP OUT NOCOPY LOGS.RSP%TYPE);
    
    /*========================================================================================================================================
    NOM: MODUL_ESTADISTIC_07
    DESCRIPCIÓ: PROCEDIMENT QUE GENERA LA INFORMACIÓ REFERENT A: 
              	-  CONSUM MIG DE TOTS ELS CLIENTS
               A LA TAULA ESTADISTICA_07
    PARAMETERS: 
          INPUT: --
          OUTPUT: RSP
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: PKG_MODUL_ESTADISTIC.MODUL_ESTADISTIC_07(RESP);
    ========================================================================================================================================*/
    PROCEDURE MODUL_ESTADISTIC_07 (E_RSP OUT NOCOPY LOGS.RSP%TYPE);
    
    /*========================================================================================================================================
    NOM: ESTADISTIQUES
    DESCRIPCIÓ: PROCEDIMENT QUE CRIDA A TOTES ALS 7 MÒDULS D'ESTADÍDTIQUES
    PARAMETERS: 
          INPUT: --
          OUTPUT: RSP
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: PKG_MODUL_ESTADISTIC.ESTADISTIQUES(RESP);
    ========================================================================================================================================*/
    PROCEDURE ESTADISTIQUES (E_RSP OUT NOCOPY LOGS.RSP%TYPE);

END PKG_MODUL_ESTADISTIC;
/

CREATE OR REPLACE PACKAGE BODY PKG_MODUL_ESTADISTIC AS
    
    /* VARIABLES COMUNS */
    V_PROCEDIMENT LOGS.PROCEDIMENT%TYPE;
    V_PARAM_SORTIDA LOGS.PARAM_SORTIDA%TYPE; 
    V_ERROR MSG_ERRORS.ID_ERROR%TYPE;
    V_DESC  MSG_ERRORS.DESCRIPCIO%TYPE;
    V_ID_LINIA_COM LINIES_COMUNICACIO.ID_LINIA_COM%TYPE;
    V_ID_CENT_PROD CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE;
    V_ID_COMPTADOR COMPTADORS.ID_COMPTADOR%TYPE;
    V_CONSUM HIST_COMPTADORS.CONSUM%TYPE;
    V_CONSUM_MITJA HIST_COMPTADORS.CONSUM%TYPE;
    V_COUNT NUMBER;
    V_PERIODE NUMBER;  
    

/* ACTUALITZA DADES ESTADISTICA 01 */
    PROCEDURE MODUL_ESTADISTIC_01 (
          E_RSP OUT NOCOPY LOGS.RSP%TYPE) IS

          CURSOR C01 IS 
               SELECT  PD.ID_CENT_PROD, SUM(HC.CONSUM) 
               FROM    HIST_COMPTADORS HC, CON_DISTRIBUCIO_COMPTADOR DC, CON_PRODUCCIO_DISTRIBUCIO PD
               WHERE   PD.ID_CENTL_DIST = DC.ID_CENTL_DIST AND DC.ID_COMPTADOR = HC.ID_COMPTADOR
               GROUP BY PD.ID_CENT_PROD;
   	  
        BEGIN
        V_PROCEDIMENT := 'MODUL_ESTADISTIC_01';	
        E_RSP := 'OK'; 
    
         -- GENEREM LA INFORMACIÓ A INSERTAR A LA TAULA ESTADISTICA_01
        OPEN C01;
        LOOP
              FETCH C01 INTO V_ID_CENT_PROD, V_CONSUM;
              
              /* EL CURSOR NO RETORNA CAP REGISTRE, NO S'ACTUALITZA LA TAULA */
               IF (C01%NOTFOUND = TRUE AND C01%ROWCOUNT = 0)
               THEN
                     V_PARAM_SORTIDA := 'NO HI HAN DADES ACTUALITZABLES';
                     INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                     COMMIT;
               END IF;
                            
              EXIT WHEN C01%NOTFOUND;
              -- SI LA CENTRAL DE PRODUCCIÓ I EL COMPTADOR EXISTEIXEN A LA TAULA,
              -- REALITZAREM UPDATE DEL CONSUM. EN CAS CONTRARI, REALITZAREM INSERT 
              -- D'UN NOU REGISTRE
              
              SELECT COUNT(1)
              INTO   V_COUNT
              FROM   ESTADISTICA_01
              WHERE  ID_CENT_PROD = V_ID_CENT_PROD;
              
              IF (V_COUNT = 0)
              THEN
                  -- INSERTEM LES DADES OBTINGUDES A LA TAULA ESTADISTICA_01
                  INSERT INTO ESTADISTICA_01 
                  VALUES(V_ID_CENT_PROD, V_CONSUM, NULL, NULL);
              ELSE
                  -- MODIFIQUEM EL REGISTRE A LA TAULA ESTADISTICA_01
                  UPDATE ESTADISTICA_01
                  SET CONSUM_COMPT = V_CONSUM
                  WHERE ID_CENT_PROD = V_ID_CENT_PROD;
              END IF;
              
              -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
              V_PARAM_SORTIDA := 'CENT_PROD: '||V_ID_CENT_PROD||' / CONSUM: '||V_CONSUM;
              INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
        END LOOP;
        CLOSE C01;
        COMMIT;
    
     EXCEPTION 
          WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               E_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS 
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);   
          WHEN DUP_VAL_ON_INDEX THEN
               V_ERROR := -01003;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               E_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               E_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);
    
    END MODUL_ESTADISTIC_01;

/* ACTUALITZA DADES ESTADISTICA 02 */
    PROCEDURE MODUL_ESTADISTIC_02 (
          E_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
        
        CURSOR C02 IS
              SELECT PD.ID_LINIA_COM, EXTRACT(YEAR FROM HC.DATA_MENSUAL),AVG(HC.CONSUM)
              FROM   HIST_COMPTADORS HC, CON_DISTRIBUCIO_COMPTADOR DC, CON_PRODUCCIO_DISTRIBUCIO PD
              WHERE  PD.ID_CENTL_DIST = DC.ID_CENTL_DIST AND DC.ID_COMPTADOR = HC.ID_COMPTADOR
              GROUP BY PD.ID_LINIA_COM, EXTRACT(YEAR FROM HC.DATA_MENSUAL);

        BEGIN
        V_PROCEDIMENT := 'MODUL_ESTADISTIC_02';	
        E_RSP := 'OK'; 
    
        -- GENEREM LA INFORMACIÓ A INSERTAR A LA TAULA ESTADISTICA_02
        OPEN C02;
        LOOP
              FETCH C02 INTO V_ID_LINIA_COM, V_PERIODE, V_CONSUM_MITJA;
              
              /* EL CURSOR NO RETORNA CAP REGISTRE, NO S'ACTUALITZA LA TAULA */
              IF (C02%NOTFOUND = TRUE AND C02%ROWCOUNT = 0)
              THEN
                   V_PARAM_SORTIDA := 'NO HI HAN DADES ACTUALITZABLES';
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                   COMMIT;
              END IF;
              
              EXIT WHEN C02%NOTFOUND;
              -- SI LA LÍNIA I EL PERÍODE EXISTEIXEN, ACTUALITZEM LES DADES, EN CAS CONTRARI, 
              -- INSERTEM UN NOU REGISTRE A LA TAULA
                  
              SELECT COUNT(ID_LINIA_COM)
              INTO   V_COUNT
              FROM   ESTADISTICA_02
              WHERE  ID_LINIA_COM = V_ID_LINIA_COM AND PERIODE_ANUAL=V_PERIODE;
              
              IF (V_COUNT = 0)
              THEN
                   -- INSERTEM LES DADES OBTINGUDES A LA TAULA ESTADISTICA_02
                    INSERT INTO ESTADISTICA_02
                    VALUES(V_ID_LINIA_COM, V_PERIODE, V_CONSUM_MITJA, NULL, NULL);
              ELSE
                  -- ACTUALITZEM EL REGISTRE
                  UPDATE ESTADISTICA_02
                  SET    CONSUM_MITJA = V_CONSUM_MITJA
                  WHERE  ID_LINIA_COM = V_ID_LINIA_COM AND 
                         PERIODE_ANUAL = V_PERIODE;
              END IF;
         
              -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
              V_PARAM_SORTIDA := 'LINIA_COM: '||V_ID_LINIA_COM||' / PERIODE_ANUAL: '||V_PERIODE||
                            ' /CONSUM_MITJA: '||V_CONSUM_MITJA;
              INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
              COMMIT;
        END LOOP;
        CLOSE C02;
        
    
      EXCEPTION 
          WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               E_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS 
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);   
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               E_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);
    END MODUL_ESTADISTIC_02;

/* ACTUALITZA DADES ESTADISTICA_03 */
    PROCEDURE MODUL_ESTADISTIC_03 (
          E_RSP OUT NOCOPY LOGS.RSP%TYPE) IS

        V_ID_LINIA_COM_OLD LINIES_COMUNICACIO.ID_LINIA_COM%TYPE;
        V_ID_LINIA_COM_NEW LINIES_COMUNICACIO.ID_LINIA_COM%TYPE;
        V_TRANSPORT_MAX_OLD LINIES_COMUNICACIO.TRANSPORT_MAX%TYPE;
        V_TRANSPORT_MAX_NEW LINIES_COMUNICACIO.TRANSPORT_MAX%TYPE;
        V_CONSUM_OLD CONTROL_LECTURES.CONSUM%TYPE;
        V_CONSUM_NEW CONTROL_LECTURES.CONSUM%TYPE;
        V_DIFERENCIA_OLD NUMBER;
        V_DIFERENCIA_NEW NUMBER;
        
    
        CURSOR C03 IS 
               SELECT LC.ID_LINIA_COM, LC.TRANSPORT_MAX, SUM(HC.CONSUM)
               FROM   HIST_COMPTADORS HC, CON_DISTRIBUCIO_COMPTADOR DC,CON_PRODUCCIO_DISTRIBUCIO PD, LINIES_COMUNICACIO LC
               WHERE LC.ID_LINIA_COM = PD.ID_LINIA_COM AND PD.ID_CENTL_DIST = DC.ID_CENTL_DIST AND DC.ID_COMPTADOR = HC.ID_COMPTADOR
               GROUP BY LC.ID_LINIA_COM, LC.TRANSPORT_MAX;
           
        BEGIN
        V_PROCEDIMENT := 'MODUL_ESTADISTIC_03';	
        E_RSP := 'OK'; 
    
          -- GENEREM LA INFORMACIÓ A INSERTAR A LA TAULA ESTADISTICA_03
        OPEN C03;
        FETCH C03 INTO V_ID_LINIA_COM_NEW, V_TRANSPORT_MAX_NEW, V_CONSUM_NEW;
        V_DIFERENCIA_NEW := (V_TRANSPORT_MAX_NEW - V_CONSUM_NEW);
        LOOP
              FETCH C03 INTO V_ID_LINIA_COM_OLD, V_TRANSPORT_MAX_OLD, V_CONSUM_OLD;
              
               /* EL CURSOR NO RETORNA CAP REGISTRE, NO S'ACTUALITZA LA TAULA */
              IF (C03%NOTFOUND = TRUE AND C03%ROWCOUNT = 0)
              THEN
                   V_PARAM_SORTIDA := 'NO HI HAN DADES ACTUALITZABLES';
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                   COMMIT;
              END IF;
              
              EXIT WHEN C03%NOTFOUND;
              V_DIFERENCIA_OLD := (V_TRANSPORT_MAX_OLD - V_CONSUM_OLD);
        
              IF (V_DIFERENCIA_NEW > V_DIFERENCIA_OLD)
              THEN
                   V_ID_LINIA_COM_NEW := V_ID_LINIA_COM_OLD;
                   V_DIFERENCIA_NEW := V_DIFERENCIA_OLD;
                   V_TRANSPORT_MAX_NEW := V_TRANSPORT_MAX_OLD;
                   V_CONSUM_NEW := V_CONSUM_OLD;
                   V_ID_LINIA_COM_OLD := 0;
                   V_DIFERENCIA_OLD := 0;
              END IF;
        END LOOP;
        CLOSE C03;
    
        -- TRUNQUEM LA TAULA
        DELETE FROM ESTADISTICA_03;
        
        -- INSERTEM LES DADES OBTINGUDES A LA TAULA ESTADISTICA_03
        INSERT INTO ESTADISTICA_03
        VALUES(V_ID_LINIA_COM_NEW, V_TRANSPORT_MAX_NEW, V_CONSUM_NEW, NULL, NULL);
                
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        V_PARAM_SORTIDA := 'LINIA_COMUNICACIO: '||V_ID_LINIA_COM_NEW||' / TRANSPORT_MAX: '||V_TRANSPORT_MAX_NEW||
                           ' / CONSUM: '||V_CONSUM_NEW;
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
        COMMIT;
    
        EXCEPTION 
          WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               E_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS 
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);   
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               E_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);
            
    END MODUL_ESTADISTIC_03;

/* ACTUALITZA DADES ESTADISTICA_04 */
    PROCEDURE MODUL_ESTADISTIC_04 (
        E_RSP OUT NOCOPY LOGS.RSP%TYPE) IS

          V_PERCENT_LINIES ESTADISTICA_04.PERCENTATGE_LINIES%TYPE;
          V_TOTAL_LINIES NUMBER;
          V_TOTAL_50 NUMBER;
          V_CONT_PERIODE  NUMBER;
                   
         
          CURSOR C04 IS 
                 SELECT COUNT(TAB1.ID_LINIA_COM), TAB1.PERIODE
                 FROM (  SELECT PD1.ID_LINIA_COM, EXTRACT(YEAR FROM HC1.DATA_MENSUAL) AS PERIODE, SUM(HC1.CONSUM)
                         FROM   HIST_COMPTADORS HC1, CON_PRODUCCIO_DISTRIBUCIO PD1, CON_DISTRIBUCIO_COMPTADOR DC1
                         WHERE  HC1.ID_COMPTADOR = DC1.ID_COMPTADOR AND DC1.ID_CENTL_DIST = PD1.ID_CENTL_DIST 
                         HAVING SUM(HC1.CONSUM) >  (
                                        SELECT SUM(HC2.CONSUM) * 0.5         
                                        FROM   HIST_COMPTADORS HC2, CON_PRODUCCIO_DISTRIBUCIO PD2, CON_DISTRIBUCIO_COMPTADOR DC2
                                        WHERE  EXTRACT(YEAR FROM HC2.DATA_MENSUAL) = EXTRACT(YEAR FROM HC1.DATA_MENSUAL) AND 
                                        HC2.ID_COMPTADOR = DC2.ID_COMPTADOR AND DC2.ID_CENTL_DIST = PD2.ID_CENTL_DIST
                                        GROUP BY EXTRACT(YEAR FROM HC2.DATA_MENSUAL) )        
                         GROUP BY PD1.ID_LINIA_COM, EXTRACT(YEAR FROM HC1.DATA_MENSUAL)
                       ) TAB1
                  GROUP BY TAB1.PERIODE;
          
          BEGIN
          V_PROCEDIMENT := 'MODUL_ESTADISTIC_04';	
          E_RSP := 'OK'; 
      
          -- OBTENIM LA INFORMACIÓ A INSERTAR A LA TAULA ESTADISTICA_04
          OPEN C04;
          LOOP
               FETCH C04 INTO V_TOTAL_50, V_PERIODE;
               /* EL CURSOR NO RETORNA CAP REGISTRE, NO S'ACTUALITZA LA TAULA */
               IF (C04%NOTFOUND = TRUE AND C04%ROWCOUNT = 0)
               THEN
                     V_PARAM_SORTIDA := 'NO HI HAN DADES ACTUALITZABLES';
                     INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                     COMMIT;
               END IF;
               
               EXIT WHEN C04%NOTFOUND;
              
               -- OBTENIM EL TOTAL DE LÍNIES SEGONS L'ANY RETORNAT PEL CURSOR
               SELECT COUNT(TAB2.ID_LINIA_COM)
               INTO V_TOTAL_LINIES
               FROM ( SELECT PD.ID_LINIA_COM, EXTRACT(YEAR FROM HC.DATA_MENSUAL) AS PERIODE, SUM(HC.CONSUM)
                      FROM   HIST_COMPTADORS HC, CON_PRODUCCIO_DISTRIBUCIO PD, CON_DISTRIBUCIO_COMPTADOR DC
                      WHERE  HC.ID_COMPTADOR = DC.ID_COMPTADOR AND DC.ID_CENTL_DIST = PD.ID_CENTL_DIST AND
                             EXTRACT(YEAR FROM HC.DATA_MENSUAL) = V_PERIODE
                      GROUP BY PD.ID_LINIA_COM, EXTRACT(YEAR FROM HC.DATA_MENSUAL)
                     ) TAB2
               GROUP BY TAB2.PERIODE;
             
               --ACTUALITZEM LES DADES SI EL REGISTRE EXISTEIX, I INSERTEM EN CAS CONTRARI
               SELECT COUNT(PERIODE_ANUAL)
               INTO   V_CONT_PERIODE
               FROM   ESTADISTICA_04
               WHERE  PERIODE_ANUAL = V_PERIODE;
                 
               V_PERCENT_LINIES := (V_TOTAL_50 / V_TOTAL_LINIES)* 100;
               IF (V_CONT_PERIODE > 0)
               THEN 
                     -- ACTUALITZEM DADES A LA TAULA ESTADISTICA_04
                     UPDATE ESTADISTICA_04
                     SET    PERCENTATGE_LINIES = V_PERCENT_LINIES
                     WHERE  PERIODE_ANUAL = V_PERIODE;
               ELSE 
                     -- INSERTEM LES DADES A LA TAULA ESTADISTICA_04
                     INSERT INTO ESTADISTICA_04
                     VALUES (V_PERIODE, V_PERCENT_LINIES, NULL, NULL);
               END IF;
          
                -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
                V_PARAM_SORTIDA := 'ANY: '||V_PERIODE||' / PERCENTATGE LINIES: '||V_PERCENT_LINIES;
                INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                COMMIT;
  
          END LOOP;
          CLOSE C04;
          
            
      
        EXCEPTION 
            WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 E_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS 
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);   
           WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 E_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);

      END MODUL_ESTADISTIC_04;
      
/* ACTUALITZA DADES ESTADISTICA_05 */
  PROCEDURE MODUL_ESTADISTIC_05 (
          E_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 
           
          V_TOTAL_CENT_PROD  NUMBER;
          V_COUNT_PERIODE  NUMBER;
                    
          CURSOR C05 IS 
                 SELECT COUNT(TAB1.ID_CENT_PROD), TAB1.PERIODE
                 FROM  (SELECT HP1.ID_CENT_PROD, EXTRACT(YEAR FROM HP1.DATA_MENSUAL) AS PERIODE, SUM(HP1.ENERGIA_PROD)
                        FROM HIST_CENTRALS_PRODUCCIO HP1
                        GROUP BY HP1.ID_CENT_PROD, EXTRACT(YEAR FROM HP1.DATA_MENSUAL)
                        HAVING SUM(HP1.ENERGIA_PROD) < (
                                      SELECT 0.3 * SUM(HP2.ENERGIA_PROD)
                                      FROM HIST_CENTRALS_PRODUCCIO HP2
                                      WHERE EXTRACT(YEAR FROM HP2.DATA_MENSUAL) = EXTRACT(YEAR FROM HP1.DATA_MENSUAL)
                                      GROUP BY EXTRACT(YEAR FROM HP2.DATA_MENSUAL)) 
                        ORDER BY EXTRACT(YEAR FROM HP1.DATA_MENSUAL)
                        ) TAB1
                 GROUP BY TAB1.PERIODE;
          
          BEGIN
          V_PROCEDIMENT := 'MODUL_ESTADISTIC_05';	
          E_RSP := 'OK'; 
          
          
          -- GENEREM LA INFORMACIÓ A INSERTAR A LA TAULA ESTADISTICA_05
          OPEN C05;
          LOOP
              FETCH C05 INTO V_TOTAL_CENT_PROD, V_PERIODE;
              /* EL CURSOR NO RETORNA CAP REGISTRE, NO S'ACTUALITZA LA TAULA */
               IF (C05%NOTFOUND = TRUE AND C05%ROWCOUNT = 0)
               THEN
                     V_PARAM_SORTIDA := 'NO HI HAN DADES ACTUALITZABLES';
                     INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                     COMMIT;
               END IF;
               
               EXIT WHEN C05%NOTFOUND;
              
               -- ACTUALITZEM LES DADES SI EL REGISTRE EXISTEIS, EN CAS CONTRARI INSERTEM
               -- UN NOU REGISTRE
               SELECT COUNT(PERIODE_ANUAL)
               INTO   V_COUNT_PERIODE
               FROM   ESTADISTICA_05
               WHERE  PERIODE_ANUAL = V_PERIODE;
          
               IF (V_COUNT_PERIODE > 0)
               THEN
                     -- ACTUALITZEM DADES A LA TAULA ESTADISTICA_05
                     UPDATE ESTADISTICA_05
                     SET    TOTAL_CENT_PROD = V_TOTAL_CENT_PROD
                     WHERE  PERIODE_ANUAL = V_PERIODE;
               ELSE 
                     -- INSERTEM LES DADES OBTINGUDES A LA TAULA ESTADISTICA_05
                     INSERT INTO ESTADISTICA_05
                     VALUES(V_PERIODE, V_TOTAL_CENT_PROD, NULL, NULL);
               END IF;
          
               -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
               V_PARAM_SORTIDA := 'PERIODE_ANUAL: '||V_PERIODE||' / TOTAL CENTRALS PRODUCCIO: '||V_TOTAL_CENT_PROD;
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
               COMMIT;
                    
          END LOOP;
          CLOSE C05;
          
        EXCEPTION 
            WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 E_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS 
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);   
            WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 E_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);

      END MODUL_ESTADISTIC_05;

/* ACTUALITZA DADES ESTADISTICA_06 */
    PROCEDURE MODUL_ESTADISTIC_06 (
          E_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
  
          CURSOR C06 IS 
                 SELECT ID_COMPTADOR, SUM(CONSUM)
                 FROM   HIST_COMPTADORS
                 GROUP BY ID_COMPTADOR
                 ORDER BY SUM(CONSUM) DESC;
          
          BEGIN
          V_PROCEDIMENT := 'MODUL_ESTADISTIC_06';	
          E_RSP := 'OK'; 
      
            -- TRUNQUEM LA TAULA
          DELETE FROM ESTADISTICA_06;
        
         -- GENEREM I INSERTEM LA INFORMACIÓ PER LA TAULA ESTADISTICA_06
          OPEN C06;
          LOOP
             FETCH C06 INTO V_ID_COMPTADOR, V_CONSUM;
              /* EL CURSOR NO RETORNA CAP REGISTRE, NO S'ACTUALITZA LA TAULA */
              IF (C06%NOTFOUND = TRUE AND C06%ROWCOUNT = 0)
              THEN
                   V_PARAM_SORTIDA := 'NO HI HAN DADES ACTUALITZABLES';
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                   COMMIT;
              END IF;
             
             
             EXIT WHEN C06%ROWCOUNT > 10 OR C06%NOTFOUND;
                INSERT INTO ESTADISTICA_06
                VALUES(V_ID_COMPTADOR, V_CONSUM, NULL, NULL);
                -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
                V_PARAM_SORTIDA := 'ID_COMPTADOR: '||V_ID_COMPTADOR||' / CONSUM: '||V_CONSUM|| ' / ROWNUM : '||C06%ROWCOUNT;
                INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                COMMIT;
          END LOOP;
          CLOSE C06;
          
          IF (C06%ROWCOUNT < 1)
          THEN
                V_PARAM_SORTIDA := 'NO HI HAN DADES ACTUALITZABLES';
                INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                COMMIT;
          END IF;
                      
          EXCEPTION 
            WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 E_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS 
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);   
            WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 E_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);
    END MODUL_ESTADISTIC_06;

/* ACTUALITZA DADES ESTADISTICA_07 */
    PROCEDURE MODUL_ESTADISTIC_07 (
          E_RSP OUT NOCOPY LOGS.RSP%TYPE) IS

    
          V_ID_CLIENT ESTADISTICA_07.ID_CLIENT%TYPE;
          V_CONSUM_MITJA ESTADISTICA_07.CONSUM_MITJA%TYPE;
          V_COUNT NUMBER;
      
          CURSOR C07 IS 
                 SELECT CO.ID_CLIENT, AVG(HC.CONSUM)
                 FROM   CONTRACTES CO, HIST_COMPTADORS HC 
                 WHERE  CO.ID_COMPTADOR = HC.ID_COMPTADOR
                 GROUP BY HC.ID_COMPTADOR, CO.ID_CLIENT;
            
          BEGIN
          V_PROCEDIMENT := 'MODUL_ESTADISTIC_07';	
          E_RSP := 'OK'; 
      
          -- GENEREM LA INFORMACIÓ A INSERTAR A LA TAULA ESTADISTICA_07
          OPEN C07;
          LOOP
              FETCH C07 INTO V_ID_CLIENT, V_CONSUM_MITJA;
               /* EL CURSOR NO RETORNA CAP REGISTRE, NO S'ACTUALITZA LA TAULA */
              IF (C07%NOTFOUND = TRUE AND C07%ROWCOUNT = 0)
              THEN
                   V_PARAM_SORTIDA := 'NO HI HAN DADES ACTUALITZABLES';
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                   COMMIT;
              END IF;
              
              EXIT WHEN C07%NOTFOUND;
                  -- ACTUALITZEM LES DADES SI EL REGISTRE EXISTEIX, EN CAS CONTRARI INSERTEM UN NOU REGISTRE
                  SELECT COUNT(ID_CLIENT)
                  INTO   V_COUNT
                  FROM   ESTADISTICA_07
                  WHERE  ID_CLIENT = V_ID_CLIENT;
                  
                  IF (V_COUNT > 0)
                  THEN
                      -- ACTUALITZEM LES DADES A LA TAULA ESTADISTICA_07
                      UPDATE ESTADISTICA_07
                      SET    CONSUM_MITJA = V_CONSUM_MITJA
                      WHERE  ID_CLIENT = V_ID_CLIENT;
                  ELSE
                      -- INSERTEM LES DADES OBTINGUDES A LA TAULA ESTADISTICA_05
                      INSERT INTO ESTADISTICA_07
                      VALUES(V_ID_CLIENT, V_CONSUM_MITJA, NULL, NULL);
                  END IF;
                  
                  -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
                  V_PARAM_SORTIDA := 'CLIENT: '||V_ID_CLIENT||' / CONSUM MITJÀ: '||V_CONSUM_MITJA;
                  INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                  COMMIT;
          END LOOP;
          CLOSE C07;
      
          COMMIT;
      
        EXCEPTION 
            WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 E_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS 
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);   
            WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 E_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, NULL, V_PARAM_SORTIDA, NULL, NULL, E_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||E_RSP);
    END MODUL_ESTADISTIC_07;

/* CRIDA ESTADÍSTIQUES */
    PROCEDURE ESTADISTIQUES (
          E_RSP OUT NOCOPY LOGS.RSP%TYPE)     IS                        
          
         BEGIN
     
        MODUL_ESTADISTIC_01(E_RSP);
        MODUL_ESTADISTIC_02(E_RSP);
        MODUL_ESTADISTIC_03(E_RSP);
        MODUL_ESTADISTIC_04(E_RSP);
        MODUL_ESTADISTIC_05(E_RSP);
        MODUL_ESTADISTIC_06(E_RSP);
        MODUL_ESTADISTIC_07(E_RSP);
     
    END ESTADISTIQUES;

END PKG_MODUL_ESTADISTIC;
/
COMMIT;
/