/*========================================================================================================================================
  NOM: PGK_VISTES_PARAM
  DESCRIPCIÓ: Package amb diferents procediments que permeten donar i retnra valor a les valiables indicades
  PARÀMETRES:   
      Input: I_LOCALITAT, I_DATA_1, I_DATA_2, I_ID_CENT_PROD
      Output: LOCALITAT, DATA_1, DATA_2, ID_CENT_PROD
========================================================================================================================================*/              
CREATE OR REPLACE PACKAGE PKG_VISTES_PARAM AS
    
    PROCEDURE SET_LOCALITAT (I_LOCALITAT IN COMPTADORS.LOCALITAT%TYPE);
    PROCEDURE SET_DATA_1 (I_DATA_1 IN DATE);
    PROCEDURE SET_DATA_2 (I_DATA_2 IN DATE);
    PROCEDURE SET_ID_CENT_PROD (I_ID_CENT_PROD IN CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE);
    
    FUNCTION GET_LOCALITAT RETURN COMPTADORS.LOCALITAT%TYPE;
    FUNCTION GET_DATA_1 RETURN DATE;
    FUNCTION GET_DATA_2 RETURN DATE;
    FUNCTION GET_ID_CENT_PROD RETURN CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE;
    
END PKG_VISTES_PARAM;
/
CREATE OR REPLACE PACKAGE BODY PKG_VISTES_PARAM AS
      LOCALITAT COMPTADORS.LOCALITAT%TYPE;
      DATA_1 DATE;
      DATA_2 DATE;
      ID_CENT_PROD CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE;

/* DONAR VALOR A LOCALITAT */
    PROCEDURE SET_LOCALITAT (I_LOCALITAT IN COMPTADORS.LOCALITAT%TYPE)
      IS
        BEGIN
        LOCALITAT := I_LOCALITAT;
    END;

/* DONAR VALOR A DATA 1 */        
    PROCEDURE SET_DATA_1 (I_DATA_1 IN DATE)
      IS
        BEGIN
        DATA_1 := I_DATA_1;
    END;
    
/* DONAR VALOR A DATA 2 */    
    PROCEDURE SET_DATA_2 (I_DATA_2 IN DATE)
      IS
        BEGIN
        DATA_2 := I_DATA_2;
    END;

/* DONAR VALOR A ID_CENT_PROD */    
   PROCEDURE SET_ID_CENT_PROD (I_ID_CENT_PROD CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE)
      IS
        BEGIN
        ID_CENT_PROD := I_ID_CENT_PROD;
    END;

/* RETORNAR VALOR DE LOCALITAT */
    FUNCTION GET_LOCALITAT RETURN COMPTADORS.LOCALITAT%TYPE
      IS
      BEGIN
        RETURN LOCALITAT;
      END;
/* RETORNAR VALOR DE DATA 1 */      
    FUNCTION GET_DATA_1 RETURN DATE
      IS
      BEGIN
        RETURN DATA_1;
      END;   
/* RETORNAR VALOR DE DATA 2 */      
     FUNCTION GET_DATA_2 RETURN DATE
      IS
      BEGIN
        RETURN DATA_2;
      END;    
/* RETORNAR VALOR ID_CENT_PROD */      
     FUNCTION GET_ID_CENT_PROD RETURN CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE
      IS
      BEGIN
        RETURN ID_CENT_PROD;
      END;
      
END PKG_VISTES_PARAM;
/

/************** VISTES **********************************************************************************************/
/*VISTA: CONSULTA_01
  DESCRIPCIÓ: DONADA UNA CIUTAT I UNA DATA COM A PARÀMETRES, EL LLISTAT DE TOTS ELS COMPTADORS ON EL CONSUM MENSUAL DE LA DATA INDICADA HA SUPERAT EL 
              80% DEL CONSUM MITJÀ DE TOTS ELS COMPTADORS DE LA CIUTAT EN AQUELL MATEIX PERÍODE DE TEMPS */
CREATE OR REPLACE VIEW CONSULTA_01 AS
              SELECT  TAB1.ID_CONTRACTE, TAB1.POTENCIA_CONTRACTADA, ((TAB2.CONSUM *100) / TAB3.CONSUM_MITJA) AS PERCENTATGE
              FROM    (SELECT CO.ID_CONTRACTE, CO.POTENCIA_CONTRACTADA, CO.ID_COMPTADOR
                       FROM  CONTRACTES CO, COMPTADORS CT
                       WHERE CT.LOCALITAT = PKG_VISTES_PARAM.GET_LOCALITAT AND CO.ID_COMPTADOR = CT.ID_COMPTADOR
                       ) TAB1,
                       (SELECT HC.CONSUM AS CONSUM, HC.ID_COMPTADOR
                        FROM   HIST_COMPTADORS HC, COMPTADORS CT
                        WHERE  CT.LOCALITAT = PKG_VISTES_PARAM.GET_LOCALITAT AND TO_CHAR(HC.DATA_MENSUAL,'MM/YYYY') = TO_CHAR(PKG_VISTES_PARAM.GET_DATA_1,'MM/YYYY')
                               AND HC.ID_COMPTADOR = CT.ID_COMPTADOR
                       ) TAB2,
                       (SELECT AVG(HC.CONSUM) AS CONSUM_MITJA
                        FROM   HIST_COMPTADORS HC, COMPTADORS CT
                        WHERE  CT.LOCALITAT = PKG_VISTES_PARAM.GET_LOCALITAT AND TO_CHAR(HC.DATA_MENSUAL,'MM/YYYY') = TO_CHAR(PKG_VISTES_PARAM.GET_DATA_1,'MM/YYYY')
                        GROUP BY TO_CHAR(HC.DATA_MENSUAL,'MM/YYYY')
                       ) TAB3
              WHERE TAB1.ID_COMPTADOR = TAB2.ID_COMPTADOR AND TAB2.CONSUM > TAB3.CONSUM_MITJA * 0.8
              ORDER BY PERCENTATGE ASC;

/*VISTA: CONSULTA_02
  DESCRIPCIÓ: LLISTAT DE LES 10 CENTRALETES DE DISTRIBUCIÓ QUE DISTRIBUEIXEN MÉS ENERGIA */
CREATE OR REPLACE VIEW CONSULTA_02 AS
            SELECT TAB1.DOMICILI_CENTRALETA, TAB1.ENERGIA_EMESA, TAB1.ENERGIA_MAX_SUB, ROWNUM
             FROM (
                 SELECT CD.ID_CENTL_DIST AS DOMICILI_CENTRALETA, ABS(SUM(HC.CONSUM)) AS ENERGIA_EMESA, CD.ENERGIA_MAX_SUB
                 FROM CENTRALETES_DISTRIBUCIO CD, HIST_COMPTADORS HC, CON_DISTRIBUCIO_COMPTADOR CDC
                 WHERE (CD.ID_CENTL_DIST = CDC.ID_CENTL_DIST AND CDC.ID_COMPTADOR = HC.ID_COMPTADOR)
                 GROUP BY CD.ID_CENTL_DIST, CD.ENERGIA_MAX_SUB
                 ORDER BY ENERGIA_EMESA DESC) TAB1
             WHERE ROWNUM < 11;
SELECT * FROM CONSULTA_02;

/*VISTA: CONSULTA_03
  DESCRIPCIÓ: LLISTAT DE LES 10 LÍNIES DE COMUNICACIÓ MÉS CARREGADES EN RELACIÓ A LA SEVA PRÒPIA CAPACITAT MÀXIMA */
CREATE OR REPLACE VIEW CONSULTA_03 AS
              SELECT TAB1.ID_LINIA_COM, TAB1.CARREGA_LINIA
              FROM   (SELECT LC.ID_LINIA_COM, ABS(SUM(HC.CONSUM)) / LC.TRANSPORT_MAX AS CARREGA_LINIA
                      FROM   HIST_COMPTADORS HC, CON_DISTRIBUCIO_COMPTADOR DC,CON_PRODUCCIO_DISTRIBUCIO PD, LINIES_COMUNICACIO LC
                      WHERE LC.ID_LINIA_COM = PD.ID_LINIA_COM AND PD.ID_CENTL_DIST = DC.ID_CENTL_DIST AND DC.ID_COMPTADOR = HC.ID_COMPTADOR
                      GROUP BY LC.ID_LINIA_COM, LC.TRANSPORT_MAX
                      ORDER BY CARREGA_LINIA DESC) TAB1
              WHERE ROWNUM < 11;               

/*VISTA: CONSULTA_04
  DESCRIPCIÓ: LLISTAT DELS CLIENTS QUE DISPOSEN DE COMPTADORS AMB SERVEI EN ALTA DISPONIBILITAT TANT DE 
              CENTRALS DE DISTRIBUCIO, COM DE LÍNIES DE COMUNICACIÓ I DE CENTRALS DE PRODUCCIÓ. */
CREATE OR REPLACE VIEW CONSULTA_04 AS
             SELECT CL.NIF, CO.ID_CONTRACTE AS CONTRACTE, CM.ID_MODEL_COMPTADOR AS MODEL_COMPTADOR
             FROM   CLIENTS CL, CONTRACTES CO, COMPTADORS CM
             WHERE  CL.ID_CLIENT = CO.ID_CLIENT AND CO.ID_COMPTADOR = CM.ID_COMPTADOR AND
                    CO.ID_COMPTADOR IN
                        (
                        SELECT CDC.ID_COMPTADOR
                         FROM CON_DISTRIBUCIO_COMPTADOR CDC, CON_PRODUCCIO_DISTRIBUCIO CPD
                         WHERE CDC.ID_CENTL_DIST = CPD.ID_CENTL_DIST
                         HAVING COUNT(CDC.ID_COMPTADOR) > 1 OR COUNT(CPD.ID_LINIA_COM) > 1
                         GROUP BY CDC.ID_COMPTADOR);
SELECT * FROM CONSULTA_04;

/*VISTA: CONSULTA_05
  DESCRIPCIÓ: DONADA UNA CENTRAL DE PROUDUCCIÓ I UN INTERVAL DE TEMPS, ES LLISTA EL CONSUM PRODUÏT PELS COMPTADORS I
              L'ENERGIA PRODUÏDA PER LA CENTRAL EN AQUELL PERÍODE */
CREATE OR REPLACE VIEW CONSULTA_05 AS
            SELECT HP.ID_CENT_PROD, ABS(SUM(HC.CONSUM)) AS CONSUM, ABS(SUM(HP.ENERGIA_PROD)) AS PRODUCCIO
             FROM HIST_CENTRALS_PRODUCCIO HP, HIST_COMPTADORS HC, CON_PRODUCCIO_DISTRIBUCIO CP, CON_DISTRIBUCIO_COMPTADOR CD
            WHERE HP.ID_CENT_PROD = PKG_VISTES_PARAM.GET_ID_CNENT_PROD AND HP.ID_CENT_PROD = CP.ID_CENT_PROD AND CP.ID_CENTL_DIST = CD.ID_CENTL_DIST AND
                  CD.ID_COMPTADOR = HC.ID_COMPTADOR AND
                  HC.DATA_MENSUAL BETWEEN TO_DATE(PKG_VISTES_PARAM.GET_DATA_1,'DD/MM/YYYY') AND TO_DATE(PKG_VISTES_PARAM.GET_DATA_2,'DD/MM/YYYY') AND
                  HP.DATA_MENSUAL BETWEEN TO_DATE (PKG_VISTES_PARAM.GET_DATA_1,'DD/MM/YYYY') AND TO_DATE(PKG_VISTES_PARAM.GET_DATA_2,'DD/MM/YYYY')
         GROUP BY HP.ID_CENT_PROD;



/*VISTA: CONSULTA_06
  DESCRIPCIÓ: PERCENTATGE DE LECTURES DE COMPTADORS EFECTUADES DE FORMA PRESENCIAL I DE FORMA TELEMÀTICA,
              EN UN PERÍODE DE TEMPS */
CREATE OR REPLACE VIEW CONSULTA_06 AS
             SELECT ROUND(TAB1.TP_TELEM/TAB3.TP_TOTAL,2) AS PERCENTAGTE_TELEMATIC, ROUND(TAB2.TP_PRESC/TAB3.TP_TOTAL,2) AS PERCENTAGTE_PRESENCIAL, TAB1.PERIODE AS PERIODE
             FROM ( SELECT COUNT(TIPUS_LECTURA) AS TP_TELEM, TO_CHAR(DATA_LECTURA,'MM/YYYY') AS PERIODE
                    FROM   CONTROL_LECTURES
                    WHERE  TIPUS_LECTURA = 'TELEMÀTICA'
                    GROUP  BY TO_CHAR(DATA_LECTURA,'MM/YYYY')
                  ) TAB1,
                  ( SELECT COUNT(TIPUS_LECTURA) AS TP_PRESC, TO_CHAR(DATA_LECTURA,'MM/YYYY') AS PERIODE
                    FROM   CONTROL_LECTURES
                    WHERE  TIPUS_LECTURA = 'PRESENCIAL'
                    GROUP  BY TO_CHAR(DATA_LECTURA,'MM/YYYY')
                  ) TAB2,
                  (  SELECT COUNT(TIPUS_LECTURA) AS TP_TOTAL, TO_CHAR(DATA_LECTURA,'MM/YYYY') AS PERIODE
                    FROM   CONTROL_LECTURES
                    GROUP  BY TO_CHAR(DATA_LECTURA,'MM/YYYY')
                  ) TAB3
              WHERE TAB1.PERIODE = TAB2.PERIODE AND TAB2.PERIODE = TAB3.PERIODE
              ORDER BY TAB1.PERIODE ASC;
SELECT * FROM CONSULTA_06;

/*VISTA: CONSULTA_07
  DESCRIPCIÓ: LLISTAT DE COMPTADORS QUE TINGUIN UN DETERMINAT NOMBRE D'ANYS D'ANTIGUITAT */
CREATE OR REPLACE VIEW CONSULTA_07 AS
             SELECT ID_COMPTADOR AS COMPTADOR, ROUND((MONTHS_BETWEEN (SYSDATE,DATA_INSTAL)/12),0) AS ANTIGUITAT_ANYS
             FROM   COMPTADORS
             ORDER BY ANTIGUITAT_ANYS DESC;


/************** SELECT **********************************************************************************************/            

--PARÀMETRES D'ENTRADA PER LA CONSULTA 01
EXEC PKG_VISTES_PARAM.SET_LOCALITAT('BARCELONA');
EXEC PKG_VISTES_PARAM.SET_DATA_1('02/05/2012');
-- CONSULTA 01
SELECT * FROM CONSULTA_01;
/
-- CONSULTA 02
SELECT * FROM CONSULTA_02;
/
-- CONSULTA 03
SELECT * FROM CONSULTA_03;
/
-- CONSULTA 04
SELECT * FROM CONSULTA_04;
/
--PARÀMETRES D'ENTRADA PER LA CONSULTA 05
EXEC PKG_VISTES_PARAM.SET_ID_CENT_PROD(190067);
EXEC PKG_VISTES_PARAM.SET_DATA_1('02/01/2011');
EXEC PKG_VISTES_PARAM.SET_DATA_2('02/05/2012');
-- CONSULTA 05
SELECT * FROM CONSULTA_05;
/
-- CONSULTA 06
SELECT * FROM CONSULTA_06;
/
-- CONSULTA 07
SELECT * FROM CONSULTA_07;

