CREATE OR REPLACE PACKAGE PKG_ABM_CLIENTS AS
    /*========================================================================================================================================
    NOM: ALTA_CLIENT
    DESCRIPCIÓ: PROCEDIMENT PER DONAR D'ALTA LES DADES D'UN CLIENT
    PARAMETERS:
          INPUT: ID_CLIENT, NOM, PRIMER_COGNOM, SEGON_COGNOM, DATA_NAIXEMENT, NIF, PROFESSIO, TELEFON_1, TELEFON_2, EMAIL, 
                 DOMICILI, CODI_POSTAL, LOCALITAT, PROVINCIA, ID_PAIS
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: ALTA_CLIENT (002506, 'Joan', 'Piqué', 'Llac', '10/02/1949', '36369887F', 'Químic', '935887447', '666589741',
                                  'joan_pique@gmail.com', 'Carrer d'Enric Granados 44, 1 2', '08036', 'Barcelona', 'Barcelona');
    ========================================================================================================================================*/
    PROCEDURE ALTA_CLIENT (
          A_ID_CLIENT     IN CLIENTS.ID_CLIENT%TYPE, 
          A_NOM           IN CLIENTS.NOM%TYPE,
          A_PRIMER_COGNOM IN CLIENTS.PRIMER_COGNOM%TYPE,
          A_SEGON_COGNOM  IN CLIENTS.SEGON_COGNOM%TYPE,
          A_DATA_NAIXEMENT IN CLIENTS.DATA_NAIXEMENT%TYPE,
          A_NIF           IN CLIENTS.NIF%TYPE,
          A_PROFESSIO     IN CLIENTS.PROFESSIO%TYPE,
          A_TELEFON_1     IN CLIENTS.TELEFON_1%TYPE,
          A_TELEFON_2     IN CLIENTS.TELEFON_2%TYPE,
          A_EMAIL         IN CLIENTS.EMAIL%TYPE,
          A_DOMICILI      IN CLIENTS.DOMICILI%TYPE,
          A_CODI_POSTAL   IN CLIENTS.CODI_POSTAL%TYPE,
          A_LOCALITAT     IN CLIENTS.LOCALITAT%TYPE,
          A_PROVINCIA     IN CLIENTS.PROVINCIA%TYPE,
          A_ID_PAIS       IN CLIENTS.ID_PAIS%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);

    /*========================================================================================================================================
    NOM: BAIXA_CLIENT
    DESCRIPCIÓ: PROCEDIMENT PER DONAR DE BAIXA UN CLIENT
    PARAMETERS:
          INPUT: ID_CLIENT
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: MODIFICACIO_CLIENT (002506);
    ========================================================================================================================================*/
    PROCEDURE BAIXA_CLIENT (
          B_ID_CLIENT     IN CLIENTS.ID_CLIENT%TYPE, 
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);


    /*========================================================================================================================================
    NOM: MODIFICACIO_CLIENT
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR LES DADES D'UN CLIENT
    PARAMETERS:
          INPUT: ID_CLIENT, NOM, PRIMER_COGNOM, SEGON_COGNOM, DATA_NAIXEMENT, NIF, PROFESSIO, TELEFON_1, TELEFON_2, EMAIL, 
                 DOMICILI, CODI_POSTAL, LOCALITAT, PROVINCIA, ID_PAIS
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: MODIFICACIO_CLIENT (002506, 'Joan', 'Piqué', 'Llac', '10/02/1949', '36369887F', 'Químic', '935887447', '666589741',
                                  'joan_pique@gmail.com', 'Carrer d'Enric Granados 44, 1 2', '08036', 'Barcelona', 'Barcelona');
========================================================================================================================================*/
    PROCEDURE MODIF_CLIENT (
          M_ID_CLIENT     IN CLIENTS.ID_CLIENT%TYPE, 
          M_NOM           IN CLIENTS.NOM%TYPE,
          M_PRIMER_COGNOM IN CLIENTS.PRIMER_COGNOM%TYPE,
          M_SEGON_COGNOM  IN CLIENTS.SEGON_COGNOM%TYPE,
          M_DATA_NAIXEMENT IN CLIENTS.DATA_NAIXEMENT%TYPE,
          M_NIF           IN CLIENTS.NIF%TYPE,
          M_PROFESSIO     IN CLIENTS.PROFESSIO%TYPE,
          M_TELEFON_1     IN CLIENTS.TELEFON_1%TYPE,
          M_TELEFON_2     IN CLIENTS.TELEFON_2%TYPE,
          M_EMAIL         IN CLIENTS.EMAIL%TYPE,
          M_DOMICILI      IN CLIENTS.DOMICILI%TYPE,
          M_CODI_POSTAL   IN CLIENTS.CODI_POSTAL%TYPE,
          M_LOCALITAT     IN CLIENTS.LOCALITAT%TYPE,
          M_PROVINCIA     IN CLIENTS.PROVINCIA%TYPE,
          M_ID_PAIS       IN CLIENTS.ID_PAIS%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);

END PKG_ABM_CLIENTS;
/

CREATE OR REPLACE PACKAGE BODY PKG_ABM_CLIENTS AS

    /* VARIABLES COMUNS */
    V_PROCEDIMENT LOGS.PROCEDIMENT%TYPE;
    V_PARAM_ENTRADA LOGS.PARAM_ENTRADA%TYPE; 
    V_ERROR MSG_ERRORS.ID_ERROR%TYPE;
    V_DESC  MSG_ERRORS.DESCRIPCIO%TYPE;
    V_ID_CLIENT CLIENTS.ID_CLIENT%TYPE;
    V_TOTAL NUMBER;
    EX_CLIENT_ACTIU EXCEPTION;
    EX_PK_NUL EXCEPTION;
    EX_FK_NUL EXCEPTION;
    PRAGMA EXCEPTION_INIT(EX_PK_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);
    
    
/* ALTA CLIENT */    
    PROCEDURE ALTA_CLIENT (
          A_ID_CLIENT IN CLIENTS.ID_CLIENT%TYPE, 
          A_NOM IN CLIENTS.NOM%TYPE,
          A_PRIMER_COGNOM IN CLIENTS.PRIMER_COGNOM%TYPE,
          A_SEGON_COGNOM IN CLIENTS.SEGON_COGNOM%TYPE,
          A_DATA_NAIXEMENT IN CLIENTS.DATA_NAIXEMENT%TYPE,
          A_NIF IN CLIENTS.NIF%TYPE,
          A_PROFESSIO IN CLIENTS.PROFESSIO%TYPE,
          A_TELEFON_1 IN CLIENTS.TELEFON_1%TYPE,
          A_TELEFON_2 IN CLIENTS.TELEFON_2%TYPE,
          A_EMAIL IN CLIENTS.EMAIL%TYPE,
          A_DOMICILI IN CLIENTS.DOMICILI%TYPE,
          A_CODI_POSTAL IN CLIENTS.CODI_POSTAL%TYPE,
          A_LOCALITAT IN CLIENTS.LOCALITAT%TYPE,
          A_PROVINCIA IN CLIENTS.PROVINCIA%TYPE,
          A_ID_PAIS IN CLIENTS.ID_PAIS%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 

    
        BEGIN
        V_PROCEDIMENT := 'ALTA_CLIENT';	
        V_PARAM_ENTRADA := 'CLIENT: '||A_ID_CLIENT||' / NOM: '||A_NOM||' / PRIMER_COGNOM: '||A_PRIMER_COGNOM||
                          ' / SEGON_COGNOM: '||A_SEGON_COGNOM||' / DATA_NAIXEMENT: '||A_DATA_NAIXEMENT||
                          ' / NIF: '||A_NIF||' / PROFESSIO: '||A_PROFESSIO|| 
                          ' / TELÈFON 1: '||A_TELEFON_1||' / TELÈFON 2: '||A_TELEFON_2||' / EMAIL: '||A_EMAIL||
                          ' / DOMICILI: '||A_DOMICILI||' / CODI_POSTAL: '||A_CODI_POSTAL||' / LOCALITAT: '||A_LOCALITAT|| 
                          ' / ID_PAIS: '||A_ID_PAIS;
        
        R_RSP := 'OK'; 
        
        -- INSERTEM EL CLIENT
        INSERT INTO CLIENTS 
        VALUES (A_ID_CLIENT, A_NOM, A_PRIMER_COGNOM, A_SEGON_COGNOM, A_DATA_NAIXEMENT, A_NIF, NULL, 
                A_PROFESSIO, A_TELEFON_1, A_TELEFON_2, A_EMAIL, A_DOMICILI, A_CODI_POSTAL, A_LOCALITAT, 
                A_PROVINCIA, A_ID_PAIS, NULL, NULL, NULL, NULL);
                
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
    
       EXCEPTION 
          WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||'. CLIENT';
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_FK_NUL THEN
               V_ERROR := -01002;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||'. PAIS: '||A_ID_PAIS;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
         WHEN DUP_VAL_ON_INDEX THEN
               V_ERROR := -01003;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' . CLIENT: '||A_ID_CLIENT;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
         WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END ALTA_CLIENT;

/* BAIXA CLIENT */

    PROCEDURE BAIXA_CLIENT (
          B_ID_CLIENT IN CLIENTS.ID_CLIENT%TYPE, 
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS

         BEGIN
         V_PROCEDIMENT := 'BAIXA_CLIENT';	
         V_PARAM_ENTRADA := 'CLIENT: '||B_ID_CLIENT;    
         R_RSP := 'OK'; 
          
          -- VERIFIQUEM QUE EL CLIENT EXISTEIXI A LA TAULA CLIENTS 
          SELECT ID_CLIENT
          INTO   V_ID_cLIENT
          FROM   CLIENTS
          WHERE ID_CLIENT = B_ID_CLIENT;
      
      
         -- VERIFIQUEM QUE EL CLIENT NO TINGUI ASSOCIAT CAP CONTRACTE
          SELECT COUNT(ID_CLIENT)
          INTO V_TOTAL
          FROM CONTRACTES
          WHERE ID_CLIENT = B_ID_CLIENT;
        
          IF (V_TOTAL > 0)
          THEN
                 RAISE EX_CLIENT_ACTIU;
          END IF;
      
          -- ESBORREM LES DADES DEL CLIENT
          -- 1. TAULA CLIENTS
          DELETE FROM CLIENTS WHERE ID_CLIENT = B_ID_CLIENT;    
      
           -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
          
         EXCEPTION 
            WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' . CLIENT: '||B_ID_CLIENT;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_CLIENT_ACTIU THEN
                 V_ERROR := -02002;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' . CLIENT: '||B_ID_CLIENT;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END BAIXA_CLIENT;

/* MODIFICACIÓ DEL CLIENT */
    PROCEDURE MODIF_CLIENT (
          M_ID_CLIENT IN CLIENTS.ID_CLIENT%TYPE, 
          M_NOM IN CLIENTS.NOM%TYPE,
          M_PRIMER_COGNOM IN CLIENTS.PRIMER_COGNOM%TYPE,
          M_SEGON_COGNOM IN CLIENTS.SEGON_COGNOM%TYPE,
          M_DATA_NAIXEMENT IN CLIENTS.DATA_NAIXEMENT%TYPE,
          M_NIF IN CLIENTS.NIF%TYPE,
          M_PROFESSIO IN CLIENTS.PROFESSIO%TYPE,
          M_TELEFON_1 IN CLIENTS.TELEFON_1%TYPE,
          M_TELEFON_2 IN CLIENTS.TELEFON_2%TYPE,
          M_EMAIL IN CLIENTS.EMAIL%TYPE,
          M_DOMICILI IN CLIENTS.DOMICILI%TYPE,
          M_CODI_POSTAL IN CLIENTS.CODI_POSTAL%TYPE,
          M_LOCALITAT IN CLIENTS.LOCALITAT%TYPE,
          M_PROVINCIA IN CLIENTS.PROVINCIA%TYPE,
          M_ID_PAIS IN CLIENTS.ID_PAIS%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 

  
         BEGIN
         V_PROCEDIMENT := 'MODIF_CLIENT';	
         V_PARAM_ENTRADA := 'CLIENT: '||M_ID_CLIENT||' / NOM: '||M_NOM||' / PRIMER_COGNOM: '||M_PRIMER_COGNOM||
                            ' / SEGON_COGNOM: '||M_SEGON_COGNOM||' / DATA_NAIXEMENT: '||M_DATA_NAIXEMENT||
                            ' / NIF: '||M_NIF||' / PROFESSIO: '||M_PROFESSIO||' / TELÈFON 1: '||M_TELEFON_1||
                            ' / TELÈFON 2: '||M_TELEFON_2||' / EMAIL: '||M_EMAIL||' / DOMICILI: '||M_DOMICILI||
                            ' / CODI_POSTAL: '||M_CODI_POSTAL||' / LOCALITAT: '||M_LOCALITAT||' / ID_PAIS: '||M_ID_PAIS;
    
         R_RSP := 'OK'; 
    
          -- VERIFIQUEM QUE EL CLIENT EXISTEIXI A LA TAULA CLIENTS 
          SELECT ID_CLIENT
          INTO   V_ID_CLIENT
          FROM   CLIENTS
          WHERE  ID_CLIENT = M_ID_CLIENT;
      
          -- MODIFIQUEM LES DADES DEL CLIENT
          UPDATE CLIENTS
          SET NOM = M_NOM,
              PRIMER_COGNOM = M_PRIMER_COGNOM, 
              SEGON_COGNOM = M_SEGON_COGNOM, 
              DATA_NAIXEMENT = M_DATA_NAIXEMENT,
              NIF = M_NIF, 
              PROFESSIO = M_PROFESSIO, 
              TELEFON_1 = M_TELEFON_1, 
              TELEFON_2 = M_TELEFON_2, 
              EMAIL = M_EMAIL, 
              DOMICILI = M_DOMICILI, 
              CODI_POSTAL = M_CODI_POSTAL, 
              LOCALITAT = M_LOCALITAT, 
              PROVINCIA = M_PROVINCIA, 
              ID_PAIS = M_ID_PAIS
          WHERE ID_CLIENT = M_ID_CLIENT;
      
          -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
      
          EXCEPTION 
            WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
      END MODIF_CLIENT;

END PKG_ABM_CLIENTS;
/
/**************************************************************************************************************************************************
**************************************************************************************************************************************************/
CREATE OR REPLACE PACKAGE PKG_ABM_COMPTADORS AS
   /*========================================================================================================================================
     NOM: ALTA_COMPTADOR
     DESCRIPCIÓ: PROCEDIMENT PER INSERTAR UN NOU COMPTADOR A LA TAULA COMPTADORS
     PARAMETERS:
           INPUT: ID_COMPTADOR, MODEL_COMPTADOR, NUM_SERIE, TIPUS_SUMINISTRE, TIPUS_LECTURA, KHW_CONSUMITS, DOMICILI, CODI_POSTAL, 
                  LOCALITAT, PROVINCIA, ID_PAIS, DATA_INSTAL·LACIO, EMPRESA_INSTAL·LACIO, TELF_EMPRESA
           OUTPUT: RST
     RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
     EXEMPLE DE CRIDA: ALTA_COMPTADOR(7886,'PAR896','GGNSFN63M31F205R','TRIFÀSIC','PRESENCIAL','75656','PASSEIG BUENA VISTA Nº 		
                                      54','21007','HUELVA','HUELVA','ES','27/05/2003','ENERCO CUELLAR SL.','921144871',RESP); 
    ========================================================================================================================================*/
    PROCEDURE ALTA_COMPTADOR (
          A_COMPTADOR     IN COMPTADORS.ID_COMPTADOR%TYPE,
          A_MODEL         IN COMPTADORS.ID_MODEL_COMPTADOR%TYPE,
          A_NUM_SERIE     IN COMPTADORS.NUM_SERIE%TYPE,
          A_TIPUS_SUMINISTRE IN COMPTADORS.TIPUS_SUMINISTRE%TYPE,
          A_TIPUS_LECTURA IN COMPTADORS.TIPUS_LECTURA%TYPE,
          A_KHW_CONSUMITS IN COMPTADORS.KHW_CONSUMITS%TYPE,
          A_DOMICILI      IN COMPTADORS.DOMICILI%TYPE,
          A_CODI_POSTAL   IN COMPTADORS.CODI_POSTAL%TYPE,
          A_LOCALITAT     IN COMPTADORS.LOCALITAT%TYPE, 
          A_PROVINCIA     IN COMPTADORS.PROVINCIA%TYPE,
          A_ID_PAIS       IN COMPTADORS.ID_PAIS%TYPE,
          A_DATA_INSTAL   IN COMPTADORS.DATA_INSTAL%TYPE,
          A_EMPRESA_INSTAL IN COMPTADORS.EMPRESA_INSTAL%TYPE,
          A_TELF_EMPRESA  IN COMPTADORS.TELF_EMPRESA%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
   /*========================================================================================================================================
    NOM: BAIXA_COMPTADOR
    DESCRIPCIÓ: PROCEDIMENT PER DONAR DE BAIXA UN COMPTADOR QUAN AQUEST NO ESTIGUI LLIGAT AMB CAP CONTRACTE.
                ES DONARÀ DE BAIXA LES DADES DEL COMPTADOR DE LES SEGÜENTS TAULES: CONTROL_LECTURES, CON_DISTRIBUCIO_COMPTADOR,
                HIST_COMPTADORS, COMPTADORS
    PARAMETERS:
          INPUT: ID_COMPTADOR
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: BAIXA_COMPTADOR (002506);
    ========================================================================================================================================*/
    PROCEDURE BAIXA_COMPTADOR (
          B_ID_COMPTADOR  IN COMPTADORS.ID_COMPTADOR%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
                    
   /*========================================================================================================================================
    NOM: MODIF_COMPTADOR_PROPIETATS
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR LES DADES (MODEL, NÚMERO DE SÈRIE, TIPUS DE SUMINISTRE, TIPUS_LECTURA) D'UN COMPTADOR A LA TAULA COMPTADORS
    PARAMETERS:
          INPUT: ID_COMPTADOR, MODEL_COMPTADOR, NUM_SERIE, TIPUS_SUMINISTRE
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: MODIF_COMPTADOR_PROPIETATS (7886,'PAR896','GGNSFN63M31F205U', 'MONOFÀSIC','PRESENCIAL',RESP);
  ========================================================================================================================================*/
    PROCEDURE MODIF_COMPTADOR_PROPIETATS (
          M_ID_COMPTADOR  IN COMPTADORS.ID_COMPTADOR%TYPE,
          M_MODEL         IN COMPTADORS.ID_MODEL_COMPTADOR%TYPE,
          M_NUM_SERIE     IN COMPTADORS.NUM_SERIE%TYPE,
          M_TIPUS_SUMINISTRE IN COMPTADORS.TIPUS_SUMINISTRE%TYPE,
          M_TIPUS_LECTURA IN COMPTADORS.TIPUS_LECTURA%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);

    /*========================================================================================================================================
    NOM: MODIF_COMPTADOR_LOCALITZACIO
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR LA UBICACIÓ D'UN COMPTADOR A LA TAULA COMPTADORS
    PARAMETERS:
          INPUT: ID_COMPTADOR, DOMICILI, CODI_POSTAL, LOCALITAT, PROVINCIA, ID_PAIS
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: MODIF_COMPTADOR_LOCALITZACIO (7886,'PASSEIG BUENA VISTA Nº 55',21007','HUELVA','HUELVA','ES',RESP)
    ========================================================================================================================================*/                              
    PROCEDURE MODIF_COMPTADOR_LOCALITZACIO (
          M_ID_COMPTADOR  IN COMPTADORS.ID_COMPTADOR%TYPE,
          M_DOMICILI      IN COMPTADORS.DOMICILI%TYPE,
          M_CODI_POSTAL   IN COMPTADORS.CODI_POSTAL%TYPE,
          M_LOCALITAT     IN COMPTADORS.LOCALITAT%TYPE, 
          M_PROVINCIA     IN COMPTADORS.PROVINCIA%TYPE,
          M_ID_PAIS       IN COMPTADORS.ID_PAIS%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
    
    /*========================================================================================================================================
    NOM: MODIF_COMPTADOR_INSTALLACIO
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR LES DADES REFERENTS A LA INSTAL·LACIO D'UN COMPTADOR A LA TAULA COMPTADORS
    PARAMETERS:
          INPUT: ID_COMPTADOR, DATA_INSTAL, EMPRESA_INSTAL, TELF_EMPRESA
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: MODIF_COMPTADOR_INSTALLACIO (7886,'27/05/2003','ENERCO CUELLAR SL.','921144871',RESP);  
    ========================================================================================================================================*/                       
    PROCEDURE MODIF_COMPTADOR_INSTALLACIO (
          M_ID_COMPTADOR  IN COMPTADORS.ID_COMPTADOR%TYPE,
          M_DATA_INSTAL   IN COMPTADORS.DATA_INSTAL%TYPE,
          M_EMPRESA_INSTAL IN COMPTADORS.EMPRESA_INSTAL%TYPE,
          M_TELF_EMPRESA  IN COMPTADORS.TELF_EMPRESA%TYPE, 
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
    
   /*========================================================================================================================================
    NOM: MODIF_COMPTADOR_CONSUM
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR EL CONSUM REALITZAT DES DE LA INSTAL·LACIO D'UN COMPTADOR A LA TAULA COMPTADORS
    PARAMETERS:
          INPUT: ID_COMPTADOR, KHW_CONSUMITS
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: MODIF_COMPTADOR_CONSUM (7886, 88.866);  
    ========================================================================================================================================*/
    PROCEDURE MODIF_COMPTADOR_CONSUM (
          M_ID_COMPTADOR  IN COMPTADORS.ID_COMPTADOR%TYPE,
          M_KHW_CONSUMITS IN COMPTADORS.KHW_CONSUMITS%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
                          
   /*========================================================================================================================================
   NOM: ALTA_CONTROL_LECTURA
   DESCRIPCIÓ: PROCEDIMENT PER INSERTAR UNA NOVA LECTURA
   PARAMETERS:
      INPUT: ID_COMPTADOR, DATA_LECTURA, CONSUM, TIPUS_LECTURA
      OUTPUT: RST
   RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
   EXEMPLE DE CRIDA: ALTA_CONTROL_LECTURA(59813,'12/05/2012',348,'TELEMÀTICA',RESP);
    ========================================================================================================================================*/
    PROCEDURE ALTA_CONTROL_LECTURA (
          A_ID_COMPTADOR  IN CONTROL_LECTURES.ID_COMPTADOR%TYPE,
          A_DATA_LECTURA  IN CONTROL_LECTURES.DATA_LECTURA%TYPE,
          A_CONSUM        IN CONTROL_LECTURES.CONSUM%TYPE,
          A_TIPUS_LECTURA IN CONTROL_LECTURES.TIPUS_LECTURA%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
  
    /*========================================================================================================================================
    NOM: MODIF_CONTROL_LECTURA
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR UNA LECTURA
    PARAMETERS:
          INPUT: ID_COMPTADOR, DATA_LECTURA, CONSUM, TIPUS_LECTURA
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: MODIF_CONTROL_LECTURA(59813,'12/05/2012',348,'TELEMÀTICA',RESP);
    ========================================================================================================================================*/
    PROCEDURE MODIF_CONTROL_LECTURA (
          M_ID_COMPTADOR IN CONTROL_LECTURES.ID_COMPTADOR%TYPE,
          M_DATA_LECTURA IN CONTROL_LECTURES.DATA_LECTURA%TYPE,
          M_CONSUM       IN CONTROL_LECTURES.CONSUM%TYPE,
          M_TIPUS_LECTURA IN CONTROL_LECTURES.TIPUS_LECTURA%TYPE,
          R_RSP          OUT NOCOPY LOGS.RSP%TYPE);

    /*========================================================================================================================================
    NOM: BAIXA_CONTROL_LECTURA
    DESCRIPCIÓ: PROCEDIMENT PER DONAR DE BAIXA UNA LECTURA
    PARAMETERS:
          INPUT: ID_COMPTADOR, DATA_LECTURA, CONSUM
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: BAIXA_CONTROL_LECTURA(59813,'12/05/2012',348,'TELEMÀTICA',RESP);
    ========================================================================================================================================*/
    PROCEDURE BAIXA_CONTROL_LECTURA (
          B_ID_COMPTADOR  IN CONTROL_LECTURES.ID_COMPTADOR%TYPE,
          B_DATA_LECTURA  IN CONTROL_LECTURES.DATA_LECTURA%TYPE,
          B_CONSUM        IN CONTROL_LECTURES.CONSUM%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
          
   /*========================================================================================================================================
   NOM: ALTA_HIST_COMPTADORS
   DESCRIPCIÓ: PROCEDIMENT PER INSERTAR UN CONSUM HISTÒRIC
   PARAMETERS:
      INPUT: ID_COMPTADOR, DATA_MENSUAL, CONSUM
      OUTPUT: RST
   RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
   EXEMPLE DE CRIDA: ALTA_HIST_COMPTADORS(10294,'16/02/2012',140,RESP);
    ========================================================================================================================================*/
    PROCEDURE ALTA_HIST_COMPTADORS (
          A_ID_COMPTADOR  IN HIST_COMPTADORS.ID_COMPTADOR%TYPE,
          A_DATA_MENSUAL  IN HIST_COMPTADORS.DATA_MENSUAL%TYPE,
          A_CONSUM        IN HIST_COMPTADORS.CONSUM%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
          
   /*========================================================================================================================================
   NOM: BAIXA_HIST_COMPTADORS
   DESCRIPCIÓ: PROCEDIMENT PER DONAR DE BAIXA UN CONSUM HISTÒRIC
   PARAMETERS:
      INPUT: ID_COMPTADOR, DATA_MENSUAL
      OUTPUT: RST
   RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
   EXEMPLE DE CRIDA: BAIXA_HIST_COMPTADORS(10294,'16/02/2012',RESP);
    ========================================================================================================================================*/
    PROCEDURE BAIXA_HIST_COMPTADORS (
          B_ID_COMPTADOR  IN HIST_COMPTADORS.ID_COMPTADOR%TYPE,
          B_DATA_MENSUAL  IN HIST_COMPTADORS.DATA_MENSUAL%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
          
   /*========================================================================================================================================
   NOM: MODIF_HIST_COMPTADORS
   DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR UN UN CONSUM HISTÒRIC
   PARAMETERS:
      INPUT: ID_COMPTADOR, DATA_MENSUAL, CONSUM
      OUTPUT: RST
   RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
   EXEMPLE DE CRIDA: MODIF_HIST_COMPTADORS(10294,'16/02/2012',140,RESP);
    ========================================================================================================================================*/
    PROCEDURE MODIF_HIST_COMPTADORS (
          M_ID_COMPTADOR  IN HIST_COMPTADORS.ID_COMPTADOR%TYPE,
          M_DATA_MENSUAL  IN HIST_COMPTADORS.DATA_MENSUAL%TYPE,
          M_CONSUM        IN HIST_COMPTADORS.CONSUM%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);      
          
END PKG_ABM_COMPTADORS;
/

/* DECLARACIÓ COS (BODY) DEL PACKAGE */
CREATE OR REPLACE PACKAGE BODY PKG_ABM_COMPTADORS AS
   
    /* VARIABLES COMUNS */
    V_PROCEDIMENT LOGS.PROCEDIMENT%TYPE;
    V_PARAM_ENTRADA LOGS.PARAM_ENTRADA%TYPE; 
    V_PARAM_SORTIDA LOGS.PARAM_SORTIDA%TYPE; 
    V_ERROR MSG_ERRORS.ID_ERROR%TYPE;
    V_DESC  MSG_ERRORS.DESCRIPCIO%TYPE;
    V_ID_COMPTADOR COMPTADORS.ID_COMPTADOR%TYPE;
    V_TIPUS_LECTURA COMPTADORS.TIPUS_LECTURA%TYPE;
    V_DATA_MENSUAL HIST_COMPTADORS.DATA_MENSUAL%TYPE;
    V_DATA_AUX HIST_COMPTADORS.DATA_MENSUAL%TYPE;
    V_TOTAL NUMBER;
    V_TOTAL_REG NUMBER;
    V_CONSUM NUMBER;
    V_CONSUM_AUX NUMBER;
    V_CONSUM_TOT NUMBER;
    EX_COMPTADOR_ACTIU EXCEPTION;
    EX_COMPTADOR_NACTIU EXCEPTION;
    EX_TIPUS_LECTURA_NOK EXCEPTION;
    EX_LECTURA_EXIST EXCEPTION;
    EX_LECTURA_NEXIST EXCEPTION;
    EX_PK_NUL EXCEPTION;
    EX_FK_NUL EXCEPTION;
    PRAGMA EXCEPTION_INIT(EX_PK_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);
    
    
/* ALTA DEL COMPTADOR */
    PROCEDURE ALTA_COMPTADOR (
          A_COMPTADOR IN COMPTADORS.ID_COMPTADOR%TYPE,
          A_MODEL IN COMPTADORS.ID_MODEL_COMPTADOR%TYPE,
          A_NUM_SERIE IN COMPTADORS.NUM_SERIE%TYPE,
          A_TIPUS_SUMINISTRE IN COMPTADORS.TIPUS_SUMINISTRE%TYPE,
          A_TIPUS_LECTURA IN COMPTADORS.TIPUS_LECTURA%TYPE,
          A_KHW_CONSUMITS IN COMPTADORS.KHW_CONSUMITS%TYPE,
          A_DOMICILI IN COMPTADORS.DOMICILI%TYPE,
          A_CODI_POSTAL IN COMPTADORS.CODI_POSTAL%TYPE,
          A_LOCALITAT IN COMPTADORS.LOCALITAT%TYPE, 
          A_PROVINCIA IN COMPTADORS.PROVINCIA%TYPE,
          A_ID_PAIS IN COMPTADORS.ID_PAIS%TYPE,
          A_DATA_INSTAL IN COMPTADORS.DATA_INSTAL%TYPE,
          A_EMPRESA_INSTAL IN COMPTADORS.EMPRESA_INSTAL%TYPE,
          A_TELF_EMPRESA IN COMPTADORS.TELF_EMPRESA%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE)  IS      
     
         BEGIN
         V_PROCEDIMENT := 'ALTA_COMPTADOR';	
         V_PARAM_ENTRADA := 'COMPTADOR: '||A_COMPTADOR||' / MODEL: '||A_MODEL||' / NUM_SERIE: '||A_NUM_SERIE||
                          ' / TIPUS_SUMINISTRE: '||A_TIPUS_SUMINISTRE||' / TIPUS_LECTURA: '||A_TIPUS_LECTURA||
                          ' / KHW_CONSUMITS: '||A_KHW_CONSUMITS||
                          ' / DOMICILI: '||A_DOMICILI||' / CODI_POSTAL: '||A_CODI_POSTAL||' / LOCALITAT: '||A_LOCALITAT|| 
                          ' / PROVINCIA: '||A_PROVINCIA||' / ID_PAIS: '||A_ID_PAIS||' / DATA_INSTAL: '||A_DATA_INSTAL||
                          ' / EMPRESA_INSTAL: '||A_EMPRESA_INSTAL||' / TELF_EMPRESA: '||A_TELF_EMPRESA; 
         R_RSP := 'OK'; 
        
         -- INSERTEM EL COMPTADOR
         INSERT INTO COMPTADORS 
         VALUES (A_COMPTADOR, A_MODEL, A_NUM_SERIE, A_TIPUS_SUMINISTRE, A_TIPUS_LECTURA, A_KHW_CONSUMITS, A_DOMICILI, A_CODI_POSTAL, 
                 A_LOCALITAT, A_PROVINCIA, A_ID_PAIS, A_DATA_INSTAL, A_EMPRESA_INSTAL, A_TELF_EMPRESA, NULL, NULL, NULL, NULL);
    
         -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
         INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
         COMMIT;
    
         EXCEPTION 
         WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' . COMPTADOR';
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
         WHEN EX_FK_NUL THEN
               V_ERROR := -01002;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' (MODEL_COMPTADOR, TIPUS_SUMINISTRE, TIPUS_LECTURA I/O PAÍS)';
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
         WHEN DUP_VAL_ON_INDEX THEN
               V_ERROR := -01003;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' . COMPTADOR: '||A_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
         WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
   END ALTA_COMPTADOR;
    
/* BAIXA DEL COMPTADOR */
   PROCEDURE BAIXA_COMPTADOR(
        B_ID_COMPTADOR IN COMPTADORS.ID_COMPTADOR%TYPE,
        R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
            
        BEGIN
        V_PROCEDIMENT := 'BAIXA_COMPTADOR';	
        V_PARAM_ENTRADA := 'COMPTADOR: '||B_ID_COMPTADOR;
        R_RSP := 'OK'; 
    
        -- VERIFIQUEM QUE EL COMPTADOR A MODIFICAR EXISTEIX A LA TAULA
        SELECT ID_COMPTADOR
        INTO V_ID_COMPTADOR
        FROM COMPTADORS
        WHERE ID_COMPTADOR = B_ID_COMPTADOR;
    
        -- VERIFIQUEM QUE EL COMPTADOR NO ESTIGUI ASSOCIAT A CAP CONTRACTE
        SELECT COUNT(ID_COMPTADOR)
        INTO V_TOTAL
        FROM CONTRACTES
        WHERE ID_COMPTADOR = B_ID_COMPTADOR;
      
        IF (V_TOTAL > 0)
        THEN
             RAISE EX_COMPTADOR_ACTIU;
        END IF;
        
        -- ESBORREM LES DADES DEL COMPTADOR
        -- 1. TAULA CONTROL_LECTURES
        DELETE FROM CONTROL_LECTURES WHERE ID_COMPTADOR = B_ID_COMPTADOR;    
        -- 2. TAULA HISTORIC_COMPTADORS
        DELETE FROM HIST_COMPTADORS WHERE ID_COMPTADOR = B_ID_COMPTADOR;    
        -- 3. TAULA REL_COMPTADOR_CENTL_DIST
        DELETE FROM CON_DISTRIBUCIO_COMPTADOR WHERE ID_COMPTADOR = B_ID_COMPTADOR;    
        -- 4. TAULA COMPTADORS
        DELETE FROM COMPTADORS WHERE ID_COMPTADOR = B_ID_COMPTADOR;    
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
        -- ACTUALITZEM VALORS DE LES ESTADÍSTIQUES
        PKG_MODUL_ESTADISTIC.ESTADISTIQUES(R_RSP);
                
        EXCEPTION 
          WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' . COMPTADOR: '||B_ID_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
           WHEN EX_COMPTADOR_ACTIU THEN
               V_ERROR := -02001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' . COMPTADOR: '||B_ID_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
           WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END BAIXA_COMPTADOR;
    
/* MODIFICACIÓ PROPIETATS DEL COMPTADOR  */                        
    PROCEDURE MODIF_COMPTADOR_PROPIETATS (
        M_ID_COMPTADOR IN COMPTADORS.ID_COMPTADOR%TYPE,
        M_MODEL IN COMPTADORS.ID_MODEL_COMPTADOR%TYPE,
        M_NUM_SERIE IN COMPTADORS.NUM_SERIE%TYPE,
        M_TIPUS_SUMINISTRE IN COMPTADORS.TIPUS_SUMINISTRE%TYPE,
        M_TIPUS_LECTURA IN COMPTADORS.TIPUS_LECTURA%TYPE,
        R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
    
        BEGIN
        V_PROCEDIMENT := 'MODIF_COMPTADOR_PROPIETATS';	
        V_PARAM_ENTRADA := 'COMPTADOR: '||M_ID_COMPTADOR||' / MODEL: '||M_MODEL||' / NUM_SERIE: '||M_NUM_SERIE||
                          ' / TIPUS_SUMINISTRE: '||M_TIPUS_SUMINISTRE||' / TIPUS_LECTURA: '||M_TIPUS_LECTURA; 
        R_RSP := 'OK'; 
    
        -- VERIFIQUEM QUE EL COMPTADOR A MODIFICAR EXISTEIXI A LA TAULA
        SELECT ID_COMPTADOR
        INTO V_ID_COMPTADOR
        FROM COMPTADORS
        WHERE ID_COMPTADOR = M_ID_COMPTADOR;
    
        -- MODIFIQUEM LES DADES DEL COMPTADOR
        UPDATE COMPTADORS
        SET ID_MODEL_COMPTADOR = M_MODEL, 
            NUM_SERIE = M_NUM_SERIE,
            TIPUS_SUMINISTRE = M_TIPUS_SUMINISTRE,
            TIPUS_LECTURA = M_TIPUS_LECTURA
        WHERE ID_COMPTADOR = M_ID_COMPTADOR;
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
       EXCEPTION 
        WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR';
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
         WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR: '||M_ID_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
         WHEN EX_FK_NUL THEN
               V_ERROR := -01002;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               IF (M_MODEL IS NULL) 
               THEN
                    R_RSP := V_ERROR|| ': '||V_DESC||'. MODEL_COMPTADOR';
               END IF;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END MODIF_COMPTADOR_PROPIETATS;
    
/* MODIFICACIÓ LOCALITZACIÓ DEL COMPTADOR  */
    PROCEDURE MODIF_COMPTADOR_LOCALITZACIO (
        M_ID_COMPTADOR IN COMPTADORS.ID_COMPTADOR%TYPE,
        M_DOMICILI IN COMPTADORS.DOMICILI%TYPE,
        M_CODI_POSTAL IN COMPTADORS.CODI_POSTAL%TYPE,
        M_LOCALITAT IN COMPTADORS.LOCALITAT%TYPE, 
        M_PROVINCIA IN COMPTADORS.PROVINCIA%TYPE,
        M_ID_PAIS IN COMPTADORS.ID_PAIS%TYPE,
        R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
    
     
        BEGIN
        V_PROCEDIMENT := 'MODIF_COMPTADOR_LOCALITZACIO';	
        V_PARAM_ENTRADA := 'COMPTADOR: '||M_ID_COMPTADOR||' / DOMICILI: '||M_DOMICILI||' / CODI_POSTAL: '||M_CODI_POSTAL||
                          ' / LOCALITAT: '||M_LOCALITAT||' / ID_PAIS: '||M_ID_PAIS;
        R_RSP := 'OK'; 
    
        -- VERIFIQUEM QUE EL COMPTADOR A MODIFICAR EXISTEIX A LA TAULA
        SELECT ID_COMPTADOR
        INTO V_ID_COMPTADOR
        FROM COMPTADORS
        WHERE ID_COMPTADOR = M_ID_COMPTADOR;
    
        -- MODIFIQUEM LES DADES DEL COMPTADOR
        UPDATE COMPTADORS
        SET DOMICILI = M_DOMICILI, 
            CODI_POSTAL = M_CODI_POSTAL,
            LOCALITAT = M_LOCALITAT, 
            PROVINCIA = M_PROVINCIA, 
            ID_PAIS = M_ID_PAIS
        WHERE ID_COMPTADOR = M_ID_COMPTADOR;
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
       EXCEPTION 
       WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR';
                -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
       WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR: '||M_ID_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
       WHEN EX_FK_NUL THEN
               V_ERROR := -01002;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               IF (M_ID_PAIS IS NULL)
               THEN 
                    R_RSP := V_ERROR|| ': '||V_DESC||'. PAIS';
               END IF;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
       WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END MODIF_COMPTADOR_LOCALITZACIO;
    

/* MODIFICACIÓ DADES INSTAL·LACIÓ DEL COMPTADOR  */
    PROCEDURE MODIF_COMPTADOR_INSTALLACIO (
        M_ID_COMPTADOR IN COMPTADORS.ID_COMPTADOR%TYPE,
        M_DATA_INSTAL IN COMPTADORS.DATA_INSTAL%TYPE,
        M_EMPRESA_INSTAL IN COMPTADORS.EMPRESA_INSTAL%TYPE,
        M_TELF_EMPRESA IN COMPTADORS.TELF_EMPRESA%TYPE, 
        R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
    
              
        BEGIN
        V_PROCEDIMENT := 'MODIF_COMPTADOR_INSTALLACIO';	
        V_PARAM_ENTRADA := 'COMPTADOR: '||M_ID_COMPTADOR||' / DATA_INSTAL: '||M_DATA_INSTAL||' / EMPRESA INSTAL : '||M_EMPRESA_INSTAL||
                          ' / TELF_EMPRESA: '||M_TELF_EMPRESA;
        R_RSP := 'OK'; 
    
        -- VERIFIQUEM QUE EL COMPTADOR A MODIFICAR EXISTEIX A LA TAULA
        SELECT ID_COMPTADOR
        INTO V_ID_cOMPTADOR
        FROM COMPTADORS
        WHERE ID_COMPTADOR = M_ID_COMPTADOR;
    
        -- MODIFIQUEM LES DADES DEL COMPTADOR
        UPDATE COMPTADORS
        SET DATA_INSTAL = M_DATA_INSTAL, 
            EMPRESA_INSTAL = M_EMPRESA_INSTAL,
            TELF_EMPRESA = M_TELF_EMPRESA
        WHERE ID_COMPTADOR = M_ID_COMPTADOR;
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
       EXCEPTION 
       WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR';
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
       WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR: '||M_ID_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
       WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END MODIF_COMPTADOR_INSTALLACIO;
 
/* MODIFICACIÓ DEL CONSUM DEL COMPTADOR DES DE LA SEVA INSTAL·LACIÓ   */
    PROCEDURE MODIF_COMPTADOR_CONSUM (
        M_ID_COMPTADOR IN COMPTADORS.ID_COMPTADOR%TYPE,
        M_KHW_CONSUMITS IN COMPTADORS.KHW_CONSUMITS%TYPE,
        R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
                
        BEGIN
        V_PROCEDIMENT := 'MODIF_COMPTADOR_CONSUM';	
        V_PARAM_ENTRADA := 'COMPTADOR: '||M_ID_COMPTADOR||' / CONSUM: '||M_KHW_CONSUMITS;
        R_RSP := 'OK'; 
    
        -- VERIFIQUEM QUE EL COMPTADOR A MODIFICAR EXISTEIX A LA TAULA
        SELECT ID_COMPTADOR
        INTO V_ID_COMPTADOR
        FROM COMPTADORS
        WHERE ID_COMPTADOR = M_ID_COMPTADOR;
    
        -- MODIFIQUEM LES DADES DEL COMPTADOR
        UPDATE COMPTADORS
        SET KHW_CONSUMITS = M_KHW_CONSUMITS
        WHERE ID_COMPTADOR = M_ID_COMPTADOR;
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
       EXCEPTION 
       WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR';
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
       WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR: '||M_ID_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
       WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END MODIF_COMPTADOR_CONSUM;
    
/* ALTA D'UNA LECTURA */
    PROCEDURE ALTA_CONTROL_LECTURA (
          A_ID_COMPTADOR IN CONTROL_LECTURES.ID_COMPTADOR%TYPE,
          A_DATA_LECTURA IN CONTROL_LECTURES.DATA_LECTURA%TYPE,
          A_CONSUM IN CONTROL_LECTURES.CONSUM%TYPE,
          A_TIPUS_LECTURA IN CONTROL_LECTURES.TIPUS_LECTURA%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
   
        BEGIN
        V_PROCEDIMENT := 'ALTA_CONTROL_LECTURA';	
        V_PARAM_ENTRADA := 'COMPTADOR: '||A_ID_COMPTADOR||' / DATA_LECTURA: '||A_DATA_LECTURA||' / CONSUM : '||A_CONSUM||
                          ' / TIPUS_LECTURA: '||A_TIPUS_LECTURA;
        R_RSP := 'OK'; 
        
         -- VERIFIQUEM QUE EL COMPTADOR TINGUI CONTRACTE ASSOCIAT 
        SELECT COUNT(1)
        INTO   V_TOTAL_REG
        FROM   CONTRACTES
        WHERE  ID_COMPTADOR = A_ID_COMPTADOR;
    
        IF (V_TOTAL_REG = 0)
        THEN
            -- EL COMPTADOR NO TÉ CONTRACTE ASSOCIAT
            RAISE EX_COMPTADOR_NACTIU;
        END IF;
        
        -- VALIDEM SI EL TIPUS DE LECTURA ÉS CORRECTE
        SELECT TIPUS_LECTURA
        INTO V_TIPUS_LECTURA
        FROM COMPTADORS
        WHERE ID_COMPTADOR = A_ID_COMPTADOR;
    
        IF (V_TIPUS_LECTURA <> A_TIPUS_LECTURA)
        THEN 
            RAISE EX_TIPUS_LECTURA_NOK;
        END IF;
    
        -- INSERTEM LA LECTURA
        INSERT INTO CONTROL_LECTURES
        VALUES (A_ID_COMPTADOR, A_DATA_LECTURA, A_CONSUM, A_TIPUS_LECTURA, NULL, NULL, NULL, NULL);
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
       EXCEPTION 
          WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN DUP_VAL_ON_INDEX THEN
               V_ERROR := -01003;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_TIPUS_LECTURA_NOK THEN
               V_ERROR := -02011;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR)||' . TIPUS_LECTURA: '||A_TIPUS_LECTURA;
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_COMPTADOR_NACTIU THEN
               V_ERROR := -02021;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR: '||A_ID_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END ALTA_CONTROL_LECTURA;

/* BAIXA D'UNA LECTURA */
    PROCEDURE BAIXA_CONTROL_LECTURA (
          B_ID_COMPTADOR IN CONTROL_LECTURES.ID_COMPTADOR%TYPE,
          B_DATA_LECTURA IN CONTROL_LECTURES.DATA_LECTURA%TYPE,
          B_CONSUM IN CONTROL_LECTURES.CONSUM%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS

    	  
          BEGIN
          V_PROCEDIMENT := 'BAIXA_CONTROL_LECTURA';	
          V_PARAM_ENTRADA := 'COMPTADOR: '||B_ID_COMPTADOR||' / DATA_LECTURA: '||B_DATA_LECTURA||' / CONSUM: '||B_CONSUM;
          R_RSP := 'OK'; 
          
          -- VERIFIQUEM QUE LA LECTURA EXISTEIXI A LA TAULA
          SELECT COUNT(1)
          INTO   V_TOTAL_REG
          FROM   CONTROL_LECTURES
          WHERE  ID_COMPTADOR = B_ID_COMPTADOR AND TO_CHAR(DATA_LECTURA,'DD/MM/YYYY') = TO_CHAR(B_DATA_LECTURA,'DD/MM/YYYY') 
                 AND CONSUM = B_CONSUM;
      
          IF (V_TOTAL = 0)
          THEN
              RAISE EX_LECTURA_NEXIST;
          END IF;
       
          -- DONEM DE BAIXA LA LECTURA
          DELETE FROM CONTROL_LECTURES 
          WHERE  ID_COMPTADOR = B_ID_COMPTADOR AND DATA_LECTURA = B_DATA_LECTURA AND CONSUM = B_CONSUM; 
      
          -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
      
         EXCEPTION 
            WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
             WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END BAIXA_CONTROL_LECTURA;
    
/* MODIFICACIÓ D'UNA LECTURA */
    PROCEDURE MODIF_CONTROL_LECTURA (
          M_ID_COMPTADOR IN CONTROL_LECTURES.ID_COMPTADOR%TYPE,
          M_DATA_LECTURA IN CONTROL_LECTURES.DATA_LECTURA%TYPE,
          M_CONSUM IN CONTROL_LECTURES.CONSUM%TYPE,
          M_TIPUS_LECTURA IN CONTROL_LECTURES.TIPUS_LECTURA%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS

        BEGIN
        V_PROCEDIMENT := 'MODIF_CONTROL_LECTURA';	
        V_PARAM_ENTRADA := 'COMPTADOR: '||M_ID_COMPTADOR||' / DATA_LECTURA: '||M_DATA_LECTURA||' / CONSUM : '||M_CONSUM||
                          ' / TIPUS_LECTURA: '||M_TIPUS_LECTURA;
        R_RSP := 'OK'; 
        
    
        -- VERIFIQUEM QUE LA LECTURA EXISTEIXI A LA TAULA
        SELECT CONSUM
        INTO   V_TOTAL
        FROM   CONTROL_LECTURES
        WHERE  ID_COMPTADOR = M_ID_COMPTADOR AND TO_CHAR(DATA_LECTURA,'DD/MM/YYYY') = TO_CHAR(M_DATA_LECTURA,'DD/MM/YYYY') 
               AND CONSUM = M_CONSUM;
    
        IF (V_TOTAL = 0)
        THEN 
            RAISE NO_DATA_FOUND;
        END IF;
    
        -- VALIDEM SI EL TIPUS DE LECTURA ÉS CORRECTE
        SELECT TIPUS_LECTURA
        INTO   V_TIPUS_LECTURA
        FROM   COMPTADORS
        WHERE  ID_COMPTADOR = M_ID_COMPTADOR;
    
        IF (V_TIPUS_LECTURA <> M_TIPUS_LECTURA)
        THEN 
             RAISE EX_TIPUS_LECTURA_NOK;
        END IF;
            
        -- MODIFIQUEM LA LECTURA
        UPDATE CONTROL_LECTURES 
        SET    ID_COMPTADOR = M_ID_COMPTADOR,
               DATA_LECTURA = M_DATA_LECTURA,
               CONSUM = M_CONSUM,
               TIPUS_LECTURA = M_TIPUS_LECTURA
        WHERE  ID_COMPTADOR = M_ID_COMPTADOR AND TO_CHAR(DATA_LECTURA,'DD/MM/YYYY') = TO_CHAR(M_DATA_LECTURA,'DD/MM/YYYY') 
               AND CONSUM = M_CONSUM;
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
        EXCEPTION 
          WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' . COMPTADOR, DATA LECTURA I/O CONSUM';
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
           WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
           WHEN EX_TIPUS_LECTURA_NOK THEN
               V_ERROR := -02011;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR)||' . TIPUS_LECTURA: '||M_TIPUS_LECTURA;
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END MODIF_CONTROL_LECTURA;

/* ALTA D'UN CONSUM HISTÒRIC */
    PROCEDURE ALTA_HIST_COMPTADORS (
          A_ID_COMPTADOR IN HIST_COMPTADORS.ID_COMPTADOR%TYPE,
          A_DATA_MENSUAL IN HIST_COMPTADORS.DATA_MENSUAL%TYPE,
          A_CONSUM IN HIST_COMPTADORS.CONSUM%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
   
        BEGIN
        V_PROCEDIMENT := 'ALTA_HIST_COMPTADORS';	
        V_PARAM_ENTRADA := 'COMPTADOR: '||A_ID_COMPTADOR||' / DATA_MENSUAL: '||A_DATA_MENSUAL||' / CONSUM : '||A_CONSUM;
        R_RSP := 'OK'; 
        
         -- VERIFIQUEM QUE EL COMPTADOR TINGUI CONTRACTE ASSOCIAT 
        SELECT COUNT(1)
        INTO   V_TOTAL_REG
        FROM   CONTRACTES
        WHERE  ID_COMPTADOR = A_ID_COMPTADOR;
    
        IF (V_TOTAL_REG = 0)
        THEN
            -- EL COMPTADOR NO TÉ CONTRACTE ASSOCIAT
            RAISE EX_COMPTADOR_NACTIU;
        END IF;
        
        -- INSERTEM LA LECTURA
        INSERT INTO HIST_COMPTADORS
        VALUES (A_ID_COMPTADOR, A_DATA_MENSUAL, A_CONSUM, NULL, NULL, NULL, NULL);
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
        -- ACTUALITZEM LES ESTADÍSTIQUES
        PKG_MODUL_ESTADISTIC.ESTADISTIQUES(R_RSP);
        
       EXCEPTION 
          WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN DUP_VAL_ON_INDEX THEN
               V_ERROR := -01003;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_COMPTADOR_NACTIU THEN
               V_ERROR := -02021;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR: '||A_ID_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END ALTA_HIST_COMPTADORS;

/* BAIXA D'UN CONSUM HISTÒRIC */
    PROCEDURE BAIXA_HIST_COMPTADORS (
          B_ID_COMPTADOR IN HIST_COMPTADORS.ID_COMPTADOR%TYPE,
          B_DATA_MENSUAL IN HIST_COMPTADORS.DATA_MENSUAL%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
   
        BEGIN
        V_PROCEDIMENT := 'BAIXA_HIST_COMPTADORS';	
        V_PARAM_ENTRADA := 'COMPTADOR: '||B_ID_COMPTADOR||' / DATA_MENSUAL: '||B_DATA_MENSUAL;
        R_RSP := 'OK'; 
        
        -- VERIFIQUEM QUE EL CONSUM MENSUAL EXISTEIXI A LA TAULA
        SELECT COUNT(1)
        INTO V_TOTAL
        FROM HIST_COMPTADORS
        WHERE ID_COMPTADOR = B_ID_COMPTADOR AND TO_CHAR(DATA_MENSUAL,'DD/MM/YYYY') = TO_CHAR(B_DATA_MENSUAL,'DD/MM/YYYY');
        
        IF (V_TOTAL = 0)
        THEN
            -- REGISTRE NO EXISTEIX
            RAISE NO_DATA_FOUND;
        END IF;
        
        -- ESBORREM EL REGISTRE
        DELETE FROM  HIST_COMPTADORS
        WHERE ID_COMPTADOR = B_ID_COMPTADOR AND TO_CHAR(DATA_MENSUAL,'DD/MM/YYYY') = TO_CHAR(B_DATA_MENSUAL,'DD/MM/YYYY'); 
        
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
        -- ACTUALITZEM LES ESTADÍSTIQUES
        PKG_MODUL_ESTADISTIC.ESTADISTIQUES(R_RSP);
        
       EXCEPTION 
          WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR)||'. COMPTADOR: '||B_ID_COMPTADOR;
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END BAIXA_HIST_COMPTADORS;

/* MODIFICACIO D'UN CONSUM HISTÒRIC */
    PROCEDURE MODIF_HIST_COMPTADORS (
          M_ID_COMPTADOR IN HIST_COMPTADORS.ID_COMPTADOR%TYPE,
          M_DATA_MENSUAL IN HIST_COMPTADORS.DATA_MENSUAL%TYPE,
          M_CONSUM IN HIST_COMPTADORS.CONSUM%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
   
        BEGIN
        V_PROCEDIMENT := 'MODIF_HIST_COMPTADORS';	
        V_PARAM_ENTRADA := 'COMPTADOR: '||M_ID_COMPTADOR||' / DATA_MENSUAL: '||M_DATA_MENSUAL||' / CONSUM : '||M_CONSUM||
                           ' / CONSUM: '||M_CONSUM;
        R_RSP := 'OK'; 
        
         -- VERIFIQUEM QUE EL REGISTRE EXISTEIXI 
        SELECT COUNT(1)
        INTO   V_TOTAL_REG
        FROM   HIST_COMPTADORS
        WHERE  ID_COMPTADOR = M_ID_COMPTADOR AND TO_CHAR(DATA_MENSUAL,'DD/MM/YYYY') = TO_CHAR(M_DATA_MENSUAL,'DD/MM/YYYY');
        
        IF (V_TOTAL_REG = 0)
        THEN
            -- EL REGISTRE NO EXISTEIX
            RAISE NO_DATA_FOUND;
        END IF;
        
         -- VERIFIQUEM QUE EL COMPTADOR TINGUI CONTRACTE ASSOCIAT 
        SELECT COUNT(1)
        INTO   V_TOTAL_REG
        FROM   CONTRACTES
        WHERE  ID_COMPTADOR = M_ID_COMPTADOR;
    
        IF (V_TOTAL_REG = 0)
        THEN
            -- EL COMPTADOR NO TÉ CONTRACTE ASSOCIAT
            RAISE EX_COMPTADOR_NACTIU;
        END IF;
        
        -- MODIFIQUEM EL CONSUM HISTÒRIC
        UPDATE HIST_COMPTADORS
        SET    CONSUM = M_CONSUM
        WHERE  ID_COMPTADOR = M_ID_COMPTADOR AND TO_CHAR(DATA_MENSUAL,'DD/MM/YYYY') = TO_CHAR(M_DATA_MENSUAL,'DD/MM/YYYY');
        
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
        -- ACTUALITZEM LES ESTADÍSTIQUES
        PKG_MODUL_ESTADISTIC.ESTADISTIQUES(R_RSP);
        
       EXCEPTION 
          WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
           WHEN EX_COMPTADOR_NACTIU THEN
               V_ERROR := -02021;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR: '||M_ID_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR)||'.COMPTADOR: '||M_ID_COMPTADOR;
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END MODIF_HIST_COMPTADORS;

END PKG_ABM_COMPTADORS;
/
/**************************************************************************************************************************************************
**************************************************************************************************************************************************/
CREATE OR REPLACE PACKAGE PKG_ABM_COMUNICACIO AS
    /*========================================================================================================================================
    NOM: ALTA_LINIA_COMUNICACIO
    DESCRIPCIÓ: PROCEDIMENT PER INSERTAR UNA NOVA LÍNIA DE COMUNICACIÓ
    PARAMETERS:
          INPUT: ID_LINIA_COM, TRANSPORT_MAX, PROVINCIA, ID_PAIS
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: ALTA_LINIA_COMUNICACIO (002506,'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE ALTA_LINIA_COMUNICACIO (
          A_ID_LINIA_COM  IN LINIES_COMUNICACIO.ID_LINIA_COM%TYPE,
          A_TRANSPORT_MAX IN LINIES_COMUNICACIO.TRANSPORT_MAX%TYPE,
          A_PROVINCIA     IN LINIES_COMUNICACIO.PROVINCIA%TYPE,
          A_ID_PAIS     IN LINIES_COMUNICACIO.ID_PAIS%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
  
    /*========================================================================================================================================
    NOM: BAIXA_LINIA_COMUNICACIO
    DESCRIPCIÓ: PROCEDIMENT PER DONAR DE BAIXA UNA LÍNIA DE COMUNICACIÓ
    PARAMETERS:
          INPUT: ID_LINIA_COM
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: ALTA_LINIA_COMUNICACIO (002506,'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE BAIXA_LINIA_COMUNICACIO (
          B_ID_LINIA_COM  IN LINIES_COMUNICACIO.ID_LINIA_COM%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);

   /*========================================================================================================================================
    NOM: MODIF_LINIA_COMUNICACIO
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR UNA LÍNIA DE COMUNICACIÓ
    PARAMETERS:
          INPUT: ID_LINIA_COM, TRANSPORT_MAX
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: ALTA_LINIA_COMUNICACIO (002506,'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE MODIF_LINIA_COMUNICACIO (
          M_ID_LINIA_COM  IN LINIES_COMUNICACIO.ID_LINIA_COM%TYPE,
          M_TRANSPORT_MAX IN LINIES_COMUNICACIO.TRANSPORT_MAX%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
          
    /*========================================================================================================================================
    NOM: MODIF_LINIA_COMUNICACIO_LOCALITZACIO
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR LA LOCALITZACIÓ D'UNA  LÍNIA DE COMUNICACIÓ
    PARAMETERS:
          INPUT: ID_LINIA_COM,PROVINCIA, PAIS
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: ALTA_LINIA_COMUNICACIO (002506,'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE MODIF_LINIA_COMUNICACIO_LOC (
          M_ID_LINIA_COM  IN LINIES_COMUNICACIO.ID_LINIA_COM%TYPE,
          M_PROVINCIA     IN LINIES_COMUNICACIO.PROVINCIA%TYPE,
          M_ID_PAIS     IN LINIES_COMUNICACIO.ID_PAIS%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);

END PKG_ABM_COMUNICACIO;
/

/* DECLARACIÓ COS (BODY) DEL PACKAGE */
CREATE OR REPLACE PACKAGE BODY PKG_ABM_COMUNICACIO AS
    
    /* VARIABLES COMUNS */
    V_PROCEDIMENT LOGS.PROCEDIMENT%TYPE;
    V_PARAM_ENTRADA LOGS.PARAM_ENTRADA%TYPE; 
    V_ERROR MSG_ERRORS.ID_ERROR%TYPE;
    V_DESC  MSG_ERRORS.DESCRIPCIO%TYPE;
    V_ID_LINIA_COM LINIES_COMUNICACIO.ID_LINIA_COM%TYPE;
    V_COUNT NUMBER;
    EX_LINIA_ACTIVA EXCEPTION;
        
    
/* ALTA LÍNIA DE COMUNICACIÓ */
    PROCEDURE ALTA_LINIA_COMUNICACIO (
          A_ID_LINIA_COM IN LINIES_COMUNICACIO.ID_LINIA_COM%TYPE,
          A_TRANSPORT_MAX IN LINIES_COMUNICACIO.TRANSPORT_MAX%TYPE,
          A_PROVINCIA IN LINIES_COMUNICACIO.PROVINCIA%TYPE,
          A_ID_PAIS IN LINIES_COMUNICACIO.ID_PAIS%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 

         BEGIN
         V_PROCEDIMENT := 'ALTA_LINIA_COMUNICACIO';	
         V_PARAM_ENTRADA := 'LINIA_COM: '||A_ID_LINIA_COM||' / TRANSPORT_MAXD: '||A_TRANSPORT_MAX||
                            ' / PROVINCIA: '||A_PROVINCIA||' / PAIDS: '||A_ID_PAIS;
         R_RSP := 'OK'; 
                                     
          -- INSERTEM LA LÍNIA DE COMUNICACIÓ
          INSERT INTO LINIES_COMUNICACIO 
          VALUES (A_ID_LINIA_COM, A_TRANSPORT_MAX, A_PROVINCIA, A_ID_PAIS, NULL, NULL, NULL, NULL);
      
          -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
      
          EXCEPTION 
            WHEN DUP_VAL_ON_INDEX THEN
                 V_ERROR := -01003;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC|| ' LINIA_COMUNICACIO: '||A_ID_LINIA_COM;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END ALTA_LINIA_COMUNICACIO;

/* BAIXA LÍNIA DE COMUNICACIÓ */
    PROCEDURE BAIXA_LINIA_COMUNICACIO (
          B_ID_LINIA_COM IN LINIES_COMUNICACIO.ID_LINIA_COM%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 

         BEGIN
         V_PROCEDIMENT := 'BAIXA_LINIA_COMUNICACIO';	
         V_PARAM_ENTRADA := 'LINIA_COM: '||B_ID_LINIA_COM;
         R_RSP := 'OK'; 
    
    
          -- VALIDEM QUE LA LÍNIA NO EXISTEIXI
          SELECT COUNT(ID_LINIA_COM)
          INTO   V_COUNT
          FROM   LINIES_COMUNICACIO
          WHERE  ID_LINIA_COM = B_ID_LINIA_COM;
          
          IF (V_COUNT = 0)
          THEN
              RAISE NO_DATA_FOUND;
          END IF;
                        
          -- VERIFIQUEM QUE LA LÍNIA DE COMUNICACIÓ NO TINGUI CAP COMPTADOR EN ACTIU (COMPTADORS ASSOCIATS A CONTRACTES);
          SELECT COUNT(CO.ID_COMPTADOR)
          INTO   V_COUNT
          FROM   CONTRACTES CO
          WHERE  CO.ID_COMPTADOR IN (
                  SELECT REL.ID_COMPTADOR
                  FROM   CON_DISTRIBUCIO_COMPTADOR REL
                  WHERE  REL.ID_CENTL_DIST IN (
                         SELECT LI.ID_CENTL_DIST 
                         FROM   CON_PRODUCCIO_DISTRIBUCIO LI
                         WHERE  LI.ID_LINIA_COM = B_ID_LINIA_COM));
 
        IF (V_COUNT > 0)
        THEN
           	RAISE EX_LINIA_ACTIVA;
        END IF;

         -- DONEM DE BAIXA LA LÍNIA DE COMUNICACIÓ
        DELETE FROM LINIES_COMUNICACIO WHERE ID_LINIA_COM = B_ID_LINIA_COM; 

        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
      
        EXCEPTION 
           WHEN NO_DATA_FOUND THEN
                V_ERROR := -01004;
                -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                V_DESC := MISSATGE_ERROR(V_ERROR);
                R_RSP := V_ERROR|| ': '||V_DESC||' LINIA_COMUNICACIO: '||B_ID_LINIA_COM;
                -- INSERTEM L'ERROR A LA TAULA DE LOGS
                INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
           WHEN EX_LINIA_ACTIVA THEN
                 V_ERROR := -02010;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR)||' LINIA_COMUNICACIO: '||B_ID_LINIA_COM;
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END BAIXA_LINIA_COMUNICACIO;

/* MODIFICACIO LÍNIA DE COMUNICACIÓ */
    PROCEDURE MODIF_LINIA_COMUNICACIO (
          M_ID_LINIA_COM IN LINIES_COMUNICACIO.ID_LINIA_COM%TYPE,
          M_TRANSPORT_MAX IN LINIES_COMUNICACIO.TRANSPORT_MAX%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 

         BEGIN
         V_PROCEDIMENT := 'MODIF_LINIA_COMUNICACIO';	
         V_PARAM_ENTRADA := 'LINIA_COM: '||M_ID_LINIA_COM||' TRANSPORT_MAX: '||M_TRANSPORT_MAX;
         R_RSP := 'OK'; 
    
    
          -- VERIFIQUEM QUE LA LÍNIA DE COMUNICACIÓ EXISTEIXI A LA TAULA
          SELECT COUNT(ID_LINIA_COM)
          INTO   V_COUNT
          FROM LINIES_COMUNICACIO
          WHERE  ID_LINIA_COM = M_ID_LINIA_COM;
      
          IF (V_COUNT = 0)
          THEN
              RAISE NO_DATA_FOUND;
          END IF;
          
          -- MODIFIQUEM LA LÍNIA DE COMUNICACIÓ
          UPDATE LINIES_COMUNICACIO 
          SET    TRANSPORT_MAX = M_TRANSPORT_MAX
          WHERE  ID_LINIA_COM = V_ID_LINIA_COM;                
        
          -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
      
          EXCEPTION 
             WHEN NO_DATA_FOUND THEN
                  V_ERROR := -01004;
                  -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                  V_DESC := MISSATGE_ERROR(V_ERROR);
                  R_RSP := V_ERROR|| ': '||V_DESC||' LINIA_COMUNICACIO: '||M_ID_LINIA_COM;
                  -- INSERTEM L'ERROR A LA TAULA DE LOGS
                  INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                  DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
             WHEN OTHERS THEN
                   V_ERROR := SQLCODE;
                   V_DESC := SUBSTR(SQLERRM, 1, 256);
                   R_RSP := V_ERROR|| ': '||V_DESC;
                   -- INSERTEM L'ERROR A LA TAULA DE LOGS
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                   DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
      END MODIF_LINIA_COMUNICACIO;

/* MODIFICACIO LÍNIA DE COMUNICACIÓ */
    PROCEDURE MODIF_LINIA_COMUNICACIO_LOC (
          M_ID_LINIA_COM IN LINIES_COMUNICACIO.ID_LINIA_COM%TYPE,
          M_PROVINCIA IN LINIES_COMUNICACIO.PROVINCIA%TYPE,
          M_ID_PAIS IN LINIES_COMUNICACIO.ID_PAIS%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 

         BEGIN
         V_PROCEDIMENT := 'MODIF_LINIA_COMUNICACIO_LOC';	
         V_PARAM_ENTRADA := 'LINIA_COM: '||M_ID_LINIA_COM||' PROVINCIA: '||M_PROVINCIA||' / PAIS: '||M_ID_PAIS;
         R_RSP := 'OK'; 
    
    
          -- VERIFIQUEM QUE LA LÍNIA DE COMUNICACIÓ EXISTEIXI A LA TAULA
          SELECT COUNT(ID_LINIA_COM)
          INTO   V_COUNT
          FROM LINIES_COMUNICACIO
          WHERE  ID_LINIA_COM = M_ID_LINIA_COM;
      
          IF (V_COUNT = 0)
          THEN
              RAISE NO_DATA_FOUND;
          END IF;
          
          -- MODIFIQUEM LA LÍNIA DE COMUNICACIÓ
          UPDATE LINIES_COMUNICACIO 
          SET    PROVINCIA = M_PROVINCIA,
                 ID_PAIS = M_ID_PAIS
          WHERE  ID_LINIA_COM = V_ID_LINIA_COM;                
        
          -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
      
          EXCEPTION 
             WHEN NO_DATA_FOUND THEN
                  V_ERROR := -01004;
                  -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                  V_DESC := MISSATGE_ERROR(V_ERROR);
                  R_RSP := V_ERROR|| ': '||V_DESC||' LINIA_COMUNICACIO: '||M_ID_LINIA_COM;
                  -- INSERTEM L'ERROR A LA TAULA DE LOGS
                  INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                  DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
             WHEN OTHERS THEN
                   V_ERROR := SQLCODE;
                   V_DESC := SUBSTR(SQLERRM, 1, 256);
                   R_RSP := V_ERROR|| ': '||V_DESC;
                   -- INSERTEM L'ERROR A LA TAULA DE LOGS
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                   DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
      END MODIF_LINIA_COMUNICACIO_LOC;


END PKG_ABM_COMUNICACIO;
/
/**************************************************************************************************************************************************
**************************************************************************************************************************************************/
CREATE OR REPLACE PACKAGE PKG_ABM_CONNEXIONS AS
    /*========================================================================================================================================
    NOM: ALTA_CON_PROD_DIST
    DESCRIPCIÓ: PROCEDIMENT PER INSERTAR UNA NOVA CONNEXIÓ ENTRE UNA CENTRAL DE PRODUCCIÓ I UNA CENTRALETA DE 
                DISTRIBUCIÓ
    PARAMETERS:
          INPUT: ID_LINIA_COM, ID_CENT_PROD, ID_CENTL_DIST
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: ALTA_LINIA_COMUNICACIO (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE ALTA_CON_PROD_DIST (
          A_ID_LINIA_COM   IN CON_PRODUCCIO_DISTRIBUCIO.ID_LINIA_COM%TYPE,
          A_ID_CENT_PROD   IN CON_PRODUCCIO_DISTRIBUCIO.ID_CENT_PROD%TYPE,
          A_ID_CENTL_DIST  IN CON_PRODUCCIO_DISTRIBUCIO.ID_CENTL_DIST%TYPE,
          R_RSP            OUT NOCOPY LOGS.RSP%TYPE);

    /*========================================================================================================================================
    NOM: BAIXA_CON_PROD_DIST
    DESCRIPCIÓ: PROCEDIMENT PER DONAR DE BAIXA UNA LÍNIA DE COMUNICACIÓ
    PARAMETERS:
          INPUT: ID_LINIA_COM, ID_CENT_PROD
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: BAIXA_LINIA_COMUNICACIO (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE BAIXA_CON_PROD_DIST (
          B_ID_LINIA_COM  IN CON_PRODUCCIO_DISTRIBUCIO.ID_LINIA_COM%TYPE,
          B_ID_CENT_PROD  IN CON_PRODUCCIO_DISTRIBUCIO.ID_CENT_PROD%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);      

    /*========================================================================================================================================
    NOM: MODIF_CON_PROD_DIST
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR UNA LÍNIA DE COMUNICACIÓ
    PARAMETERS:
          INPUT: ID_LINIA_COM, ID_CENT_PROD, ID_CENTL_DIST
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: MODIFICACIO_LINIA_COMUNICACIO (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE MODIF_CON_PROD_DIST (
          M_ID_LINIA_COM  IN CON_PRODUCCIO_DISTRIBUCIO.ID_LINIA_COM%TYPE,
          M_ID_CENT_PROD  IN CON_PRODUCCIO_DISTRIBUCIO.ID_CENT_PROD%TYPE,
          M_ID_CENTL_DIST IN CON_PRODUCCIO_DISTRIBUCIO.ID_CENTL_DIST%TYPE,
          M_ID_CENTL_DIST_UPD IN CON_PRODUCCIO_DISTRIBUCIO.ID_CENTL_DIST%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);


    /*========================================================================================================================================
    NOM: ALTA_CON_DIST_COMP
    DESCRIPCIÓ: PROCEDIMENT PER INSERTAR UNA CONNEXIÓ ENTRE UN COMPTADOR I UNA CENTRALETA DE DISTRIBUCIÓ
    PARAMETERS:
          INPUT: ID_COMPTADOR, ID_CENTL_DIST
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: ALTA_REL_COMPTADOR_CENT_DIST (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE ALTA_CON_DIST_COMP (
          A_ID_COMPTADOR  IN CON_DISTRIBUCIO_COMPTADOR.ID_COMPTADOR%TYPE,
          A_ID_CENTL_DIST IN CON_DISTRIBUCIO_COMPTADOR.ID_CENTL_DIST%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
          
    
    /*========================================================================================================================================
    NOM: BAIXA_CON_DIST_COMP
    DESCRIPCIÓ: PROCEDIMENT PER DONAR DE BAIXA LA RELACIÓ ENTRE UN COMPTADOR I LA CENTRALETA DE DISTRIBUCIÓ ASSOCIADA
    PARAMETERS:
          INPUT: ID_LINIA_COM
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: BAIXA_REL_COMPTADOR_CENT_DIST (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE BAIXA_CON_DIST_COMP (
          B_ID_COMPTADOR  IN CON_DISTRIBUCIO_COMPTADOR.ID_COMPTADOR%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
          
    /*========================================================================================================================================
    NOM: MODIF_CON_DIST_COMP 
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR LA RELACIÓ ENTRE UN COMPTADOR I UNA CENTRALETA DE DISTRIBUCIÓ
    PARAMETERS:
          INPUT: ID_COMPTADOR, ID_CENTL_DIST
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: MODIFICACIO_REL_COMPTADOR_CENT_DIST (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE MODIF_CON_DIST_COMP (
          M_ID_COMPTADOR  IN  CON_DISTRIBUCIO_COMPTADOR.ID_COMPTADOR%TYPE,
          M_ID_CENTL_DIST IN  CON_DISTRIBUCIO_COMPTADOR.ID_CENTL_DIST%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);


END PKG_ABM_CONNEXIONS;
/

CREATE OR REPLACE PACKAGE BODY PKG_ABM_CONNEXIONS AS

    /* VARIABLES COMUNS */
    V_PROCEDIMENT LOGS.PROCEDIMENT%TYPE;
    V_PARAM_ENTRADA LOGS.PARAM_ENTRADA%TYPE; 
    V_ERROR MSG_ERRORS.ID_ERROR%TYPE;
    V_DESC  MSG_ERRORS.DESCRIPCIO%TYPE;
    V_ID_PAIS_1 PAIS.ID_PAIS%TYPE;
    V_ID_PAIS_2 PAIS.ID_PAIS%TYPE;
    V_PROVINCIA_1 CENTRALS_PRODUCCIO.PROVINCIA%TYPE;
    V_PROVINCIA_2 CENTRALETES_DISTRIBUCIO.PROVINCIA%TYPE;
    V_ID_LINIA_COM CON_PRODUCCIO_DISTRIBUCIO.ID_LINIA_COM%TYPE;
    V_ID_COMPTADOR CON_DISTRIBUCIO_COMPTADOR.ID_COMPTADOR%TYPE;
    V_COUNT NUMBER;
    V_COUNT_UPD NUMBER;
    V_COUNT_CDIST NUMBER;
    V_COUNT_COMPT NUMBER;
    EX_LINIA_EXIST EXCEPTION;
    EX_LINIA_ACTIVA EXCEPTION;
    EX_DIFERENT_PAIS EXCEPTION;
    EX_DIFERENT_PROVINCIA EXCEPTION;
    EX_CENTL_DIST_NOK EXCEPTION;
    EX_CENTL_DIST_ENERGIA_NOK EXCEPTION;
    EX_COMPTADOR_ACTIU EXCEPTION;
    EX_PK_NUL EXCEPTION;
    EX_FK_NUL EXCEPTION;
    PRAGMA EXCEPTION_INIT(EX_PK_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02290);
    
          
/* ALTA CONNEXIÓ ENTRE CENTRAL DE PRODUCCIO I CENTRALETA DE DISTRIBUCIÓ */
    PROCEDURE ALTA_CON_PROD_DIST (
          A_ID_LINIA_COM IN CON_PRODUCCIO_DISTRIBUCIO.ID_LINIA_COM%TYPE,
          A_ID_CENT_PROD IN CON_PRODUCCIO_DISTRIBUCIO.ID_CENT_PROD%TYPE,
          A_ID_CENTL_DIST IN CON_PRODUCCIO_DISTRIBUCIO.ID_CENTL_DIST%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 
    
        BEGIN
        V_PROCEDIMENT := 'ALTA_CON_PROD_DIST';	
        V_PARAM_ENTRADA := 'LINIA_COM: '||A_ID_LINIA_COM||' / CENT_PROD: '||A_ID_CENT_PROD||' / CENTL_DIST: '||A_ID_CENTL_DIST;
        R_RSP := 'OK'; 
        
    
        -- VALIDEM QUE LA CONNEXIÓ NO EXISTEIXI
        SELECT COUNT(1)
        INTO   V_COUNT
        FROM   CON_PRODUCCIO_DISTRIBUCIO
        WHERE  ID_LINIA_COM = A_ID_LINIA_COM AND ID_CENT_PROD = A_ID_CENT_PROD AND ID_CENTL_DIST = A_ID_CENTL_DIST;
        
        IF (V_COUNT > 0)
        THEN
            RAISE EX_LINIA_EXIST;
        END IF;
        
        -- VALIDEM QUE LA CENTRAL DE PRODUCCIÓ I LA CENTRALETA DE DISTRIBUCIÓ PERTANYIN AL MATEIX PAÍS I PROVÍNCIA
        SELECT CP.ID_PAIS, CP.PROVINCIA, CD.ID_PAIS, CD.PROVINCIA
        INTO   V_ID_PAIS_1, V_PROVINCIA_1, V_ID_PAIS_2, V_PROVINCIA_2
        FROM   CENTRALS_PRODUCCIO CP, CENTRALETES_DISTRIBUCIO CD
        WHERE  CP.ID_CENT_PROD = A_ID_CENT_PROD AND CD.ID_CENTL_DIST = A_ID_CENTL_DIST;
    
        IF (V_ID_PAIS_1 <> V_ID_PAIS_2)
        THEN
             RAISE EX_DIFERENT_PAIS;
        ELSE
            IF (V_PROVINCIA_1 <> V_PROVINCIA_2)
            THEN
                 RAISE EX_DIFERENT_PROVINCIA;
            END IF;
        END IF;
    
        -- INSERTEM LA LÍNIA DE COMUNICACIÓ
        INSERT INTO CON_PRODUCCIO_DISTRIBUCIO 
        VALUES (A_ID_LINIA_COM, A_ID_CENT_PROD, A_ID_CENTL_DIST, NULL, NULL, NULL, NULL);
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
    
        EXCEPTION 
          WHEN EX_FK_NUL THEN
               V_ERROR := -01002;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' (CENTRAL_PROD O CENTRAL_DIST)';
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_LINIA_EXIST THEN
               V_ERROR := -01003;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_DIFERENT_PAIS THEN
               V_ERROR := -02008;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_DIFERENT_PROVINCIA THEN
               V_ERROR := -02009;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
         WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END ALTA_CON_PROD_DIST;

/* BAIXA CONNEXIÓ ENTRE CENTRAL DE PRODUCCIO I CENTRALETA DE DISTRIBUCIÓ */
    PROCEDURE BAIXA_CON_PROD_DIST (
          B_ID_LINIA_COM IN CON_PRODUCCIO_DISTRIBUCIO.ID_LINIA_COM%TYPE,
          B_ID_CENT_PROD IN CON_PRODUCCIO_DISTRIBUCIO.ID_CENT_PROD%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 
	  
        BEGIN
        V_PROCEDIMENT := 'BAIXA_CON_PROD_DIST';	
        V_PARAM_ENTRADA := 'LINIA_COM: '||B_ID_LINIA_COM||' / CENT_PRODUCCIO: '||B_ID_CENT_PROD;
        R_RSP := 'OK'; 
        
    
        -- VERIFIQUEM QUE LA LÍNIA DE COMUNICACIÓ EXISTEIXI A LA TAULA
        SELECT ID_LINIA_COM
        INTO   V_ID_LINIA_COM
        FROM   CON_PRODUCCIO_DISTRIBUCIO
        WHERE  ID_LINIA_COM = B_ID_LINIA_COM AND ID_CENT_PROD = B_ID_CENT_PROD;
    
    
        -- VERIFIQUEM QUE LA LÍNIA DE COMUNICACIÓ NO TINGUI CAP COMPTADOR EN ACTIU (COMPTADORS ASSOCIATS A CONTRACTES);
        SELECT COUNT(CO.ID_COMPTADOR)
        INTO   V_COUNT
        FROM   CONTRACTES CO
        WHERE  CO.ID_COMPTADOR IN (
                  SELECT REL.ID_CENTL_DIST
                  FROM   CON_DISTRIBUCIO_COMPTADOR REL
                  WHERE  REL.ID_CENTL_DIST IN (
                          SELECT LI.ID_CENTL_DIST 
                          FROM   CON_PRODUCCIO_DISTRIBUCIO LI
                          WHERE  LI.ID_LINIA_COM = B_ID_LINIA_COM));
     
        IF (V_COUNT > 0)
        THEN
              RAISE EX_LINIA_ACTIVA;
        END IF;
    
        -- DONEM DE BAIXA LA LÍNIA DE COMUNICACIÓ
        DELETE FROM CON_PRODUCCIO_DISTRIBUCIO WHERE ID_LINIA_COM = B_ID_LINIA_COM AND ID_CENT_PROD = B_ID_CENT_PROD;
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
        -- ACTUALITZEM ESTADISTIQUES
        PKG_MODUL_ESTADISTIC.ESTADISTIQUES(R_RSP);
    
       EXCEPTION 
          WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' . LINIA COMUNICACIO I/O CENTRAL PRODUCCIÓ';
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .LINIA COMUNICACIÓ: '||B_ID_LINIA_COM||' / CENT_PROD: '||B_ID_CENT_PROD;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_LINIA_ACTIVA THEN
               V_ERROR := -02010;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||'.  LINIA COM: '||B_ID_LINIA_COM||' / CENT_PROD: '||B_ID_CENT_PROD;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END BAIXA_CON_PROD_DIST;

/* MODIFICACIÓ CONNEXIÓ ENTRE CENTRAL DE PRODUCCIO I CENTRALETA DE DISTRIBUCIÓ */
    PROCEDURE MODIF_CON_PROD_DIST (
          M_ID_LINIA_COM IN CON_PRODUCCIO_DISTRIBUCIO.ID_LINIA_COM%TYPE,
          M_ID_CENT_PROD IN CON_PRODUCCIO_DISTRIBUCIO.ID_CENT_PROD%TYPE,
          M_ID_CENTL_DIST IN CON_PRODUCCIO_DISTRIBUCIO.ID_CENTL_DIST%TYPE,
          M_ID_CENTL_DIST_UPD IN CON_PRODUCCIO_DISTRIBUCIO.ID_CENTL_DIST%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS

        BEGIN
        V_PROCEDIMENT := 'MODIF_CON_PROD_DIST';	
        V_PARAM_ENTRADA := 'LINIA_COM: '||M_ID_LINIA_COM||' / CENT_PROD: '||M_ID_CENT_PROD||' / CENTL_DIST: '||M_ID_CENTL_DIST||
                           ' / CENT_IDT_UPD: '||M_ID_CENTL_DIST_UPD;
        R_RSP := 'OK'; 
        
    
        -- VERIFIQUEM QUE LA LÍNIA DE COMUNICACIÓ EXISTEIXI A LA TAULA
        SELECT ID_LINIA_COM
        INTO V_ID_LINIA_COM
        FROM CON_PRODUCCIO_DISTRIBUCIO
        WHERE  ID_LINIA_COM = M_ID_LINIA_COM AND ID_CENT_PROD = M_ID_CENT_PROD;
    
        -- VERIFIQUEM QUE LA LÍNIA DE COMUNICACIÓ A MODIFICAR NO EXISTEIXI A LA TAULA
        SELECT COUNT(ID_LINIA_COM)
        INTO V_COUNT_UPD
        FROM CON_PRODUCCIO_DISTRIBUCIO
        WHERE  ID_LINIA_COM = M_ID_LINIA_COM AND ID_CENT_PROD = M_ID_CENT_PROD;
        
        IF (V_COUNT_UPD > 1)
        THEN
            RAISE DUP_VAL_ON_INDEX;
        END IF;
    
        -- VALIDEM QUE LA CENTRAL DE PRODUCCIÓ I LA CENTRALETA DE DISTRIBUCIÓ PERTANYIN AL MATEIX PAÍS I PROVÍNCIA
        SELECT CP.ID_PAIS, CP.PROVINCIA, CD.ID_PAIS, CD.PROVINCIA
        INTO V_ID_PAIS_1, V_PROVINCIA_1, V_ID_PAIS_2, V_PROVINCIA_2
        FROM CENTRALS_PRODUCCIO CP, CENTRALETES_DISTRIBUCIO CD
        WHERE CP.ID_CENT_PROD = M_ID_CENT_PROD AND CD.ID_CENTL_DIST = M_ID_CENTL_DIST_UPD;
    
        IF (V_ID_PAIS_1 <> V_ID_PAIS_2)
        THEN
             RAISE EX_DIFERENT_PAIS;
        ELSE
            IF (V_PROVINCIA_1 <> V_PROVINCIA_2)
            THEN
                RAISE EX_DIFERENT_PROVINCIA;
            END IF;
        END IF;
    
        -- MODIFIQUEM LA LÍNIA DE COMUNICACIÓ
        UPDATE CON_PRODUCCIO_DISTRIBUCIO
        SET    ID_CENTL_DIST = M_ID_CENTL_DIST_UPD
        WHERE  ID_LINIA_COM = M_ID_LINIA_COM AND ID_CENT_PROD = M_ID_CENT_PROD;
       
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
        -- ACTUALITZEM ESTADISTIQUES
        PKG_MODUL_ESTADISTIC.ESTADISTIQUES(R_RSP);
    
       EXCEPTION 
          WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' . LINIA COMUNICACIO';
                -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN DUP_VAL_ON_INDEX THEN
               V_ERROR := -01003;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .LINIA COMUNICACIÓ: '||M_ID_LINIA_COM||' / CENTRAL PRODUCCIO: '||M_ID_CENT_PROD;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_DIFERENT_PAIS THEN
               V_ERROR := -02008;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_DIFERENT_PROVINCIA THEN
               V_ERROR := -02009;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
         WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
           INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END MODIF_CON_PROD_DIST;
    
/* ALTA ENTRE CENTRALETA DE DISTRIBUCIÓ I COMPTADOR */
    PROCEDURE ALTA_CON_DIST_COMP (
          A_ID_COMPTADOR IN CON_DISTRIBUCIO_COMPTADOR.ID_COMPTADOR%TYPE,
          A_ID_CENTL_DIST IN CON_DISTRIBUCIO_COMPTADOR.ID_CENTL_DIST%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS

        BEGIN
        V_PROCEDIMENT := 'ALTA_CON_DIST_COMP';	
        V_PARAM_ENTRADA := 'COMPTADOR: '||A_ID_COMPTADOR||' / CENTL_DIST: '||A_ID_CENTL_DIST;
        R_RSP := 'OK'; 
        
        
        --VALIDEM QUE EL COMPTADOR EXISTEIXI
        SELECT COUNT(ID_COMPTADOR)
        INTO V_COUNT_COMPT
        FROM COMPTADORS
        WHERE ID_COMPTADOR = A_ID_COMPTADOR;
        
        IF(V_COUNT_COMPT = 0)
        THEN 
            RAISE EX_PK_NUL;
        END IF;
        
        --VALIDEM QUE LA CENTRALETA DE DISTRIBUCIÓ EXISTEIXI
        SELECT COUNT(ID_CENTL_DIST)
        INTO V_COUNT_CDIST
        FROM CENTRALETES_DISTRIBUCIO
        WHERE ID_CENTL_DIST = A_ID_CENTL_DIST;
        
        IF(V_COUNT_CDIST = 0)
        THEN 
            RAISE EX_FK_NUL;
        END IF;
        
        -- VALIDEM QUE LA CENTRALETA DE DISTRIBUCIÓ TINGUI ASSOCIADA UNA CENTRAL DE PRODUCCIÓ
        SELECT COUNT (ID_CENT_PROD)
        INTO   V_COUNT
        FROM   CON_PRODUCCIO_DISTRIBUCIO
        WHERE  ID_CENTL_DIST = A_ID_CENTL_DIST;
    
        IF (V_COUNT = 0)
        THEN
             RAISE EX_CENTL_DIST_NOK;
        END IF;
    
        -- VALIDEM QUE EL COMPTADOR I LA CENTRALETA DE DISTRIBUCIÓ PERTANYIN AL MATEIX PAÍS I PROVÍNCIA
        SELECT CO.ID_PAIS, CO.PROVINCIA, CD.ID_PAIS, CD.PROVINCIA
        INTO   V_ID_PAIS_1, V_PROVINCIA_1, V_ID_PAIS_2, V_PROVINCIA_2
        FROM   COMPTADORS CO, CENTRALETES_DISTRIBUCIO CD
        WHERE  CO.ID_COMPTADOR = A_ID_COMPTADOR AND CD.ID_CENTL_DIST = A_ID_CENTL_DIST;
    
        IF (V_ID_PAIS_1 <> V_ID_PAIS_2)
        THEN
            RAISE EX_DIFERENT_PAIS;
        ELSE
            IF (V_PROVINCIA_1 <> V_PROVINCIA_2)
            THEN
               RAISE EX_DIFERENT_PROVINCIA;
            END IF;
        END IF;
    
        -- INSERTEM LA LÍNIA DE COMUNICACIÓ
        INSERT INTO CON_DISTRIBUCIO_COMPTADOR
        VALUES (A_ID_COMPTADOR, A_ID_CENTL_DIST, NULL, NULL, NULL, NULL);
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
        
       EXCEPTION 
          WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' . COMPTADOR';
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_FK_NUL THEN
               V_ERROR := -01002;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' . CENTRALETA DISTRIBUCIÓ: '||A_ID_CENTL_DIST;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN DUP_VAL_ON_INDEX THEN
               V_ERROR := -01003;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' . COMPTADOR: '||A_ID_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_CENTL_DIST_NOK THEN
               V_ERROR := -02018;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||'. CENTRALETA DISTRIBUCIÓ: '||A_ID_CENTL_DIST;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_DIFERENT_PAIS THEN
               V_ERROR := -02008;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_DIFERENT_PROVINCIA THEN
               V_ERROR := -02009;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END ALTA_CON_DIST_COMP;
    
    
/* BAIXA DE LA CONNEXIÓ ENTRE UNA CENTRALETA DE DISTRIBUCIÓ I UN COMPTADOR */
    PROCEDURE BAIXA_CON_DIST_COMP (
          B_ID_COMPTADOR IN CON_DISTRIBUCIO_COMPTADOR.ID_COMPTADOR%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 

        BEGIN
        V_PROCEDIMENT := 'BAIXA_CON_DIST_COMP';	
        V_PARAM_ENTRADA := 'COMPTADOR: '||B_ID_COMPTADOR;
        R_RSP := 'OK'; 
        
    
        -- VERIFIQUEM QUE EL COMPTADOR EXISTEIXI
        SELECT ID_COMPTADOR
        INTO   V_ID_COMPTADOR
        FROM   CON_DISTRIBUCIO_COMPTADOR
        WHERE  ID_COMPTADOR = B_ID_COMPTADOR;
    
    
        -- VERIFIQUEM QUE EL COMPTADOR NO TINGUI CAP CONTRACTE ASSOCIAT
        SELECT COUNT(ID_COMPTADOR)
        INTO   V_COUNT
        FROM   CONTRACTES
        WHERE  ID_COMPTADOR = B_ID_COMPTADOR;
     
        IF V_COUNT > 0
        THEN
              RAISE EX_COMPTADOR_ACTIU;
        END IF;
    
        -- DONEM DE BAIXA LA LÍNIA DE COMUNICACIÓ
        DELETE FROM CON_DISTRIBUCIO_COMPTADOR WHERE ID_COMPTADOR = B_ID_COMPTADOR;
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
    
        -- ACTUALITZEM ESTADISTIQUES
        PKG_MODUL_ESTADISTIC.ESTADISTIQUES(R_RSP);
    
       EXCEPTION 
          WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' . COMPTADOR';
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR: '||B_ID_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_COMPTADOR_ACTIU THEN
               V_ERROR := -02001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR: '||B_ID_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END BAIXA_CON_DIST_COMP;

/* MODIFICACIÓ DE LA CONNEXIÓ ENTRE UNA CENTRALETA DE DISTRIBUCIÓ I UN COMPTADOR */
    PROCEDURE MODIF_CON_DIST_COMP (
          M_ID_COMPTADOR IN  CON_DISTRIBUCIO_COMPTADOR.ID_COMPTADOR%TYPE,
          M_ID_CENTL_DIST IN  CON_DISTRIBUCIO_COMPTADOR.ID_CENTL_DIST%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
 
        BEGIN
        V_PROCEDIMENT := 'MODIF_CON_DIST_COMP';	
        V_PARAM_ENTRADA := 'COMPTADOR: '||M_ID_COMPTADOR||' / CENTL_DIST: '||M_ID_CENTL_DIST;
        R_RSP := 'OK'; 
        
    
        -- VERIFIQUEM QUE EL COMPTADOR EXISTEIXI A LA TAULA
        SELECT ID_COMPTADOR
        INTO   V_ID_COMPTADOR
        FROM   CON_DISTRIBUCIO_COMPTADOR
        WHERE  ID_COMPTADOR = M_ID_COMPTADOR;
        
        -- VALIDEM QUE EL COMPTADOR I LA CENTRALETA DE DISTRIBUCIÓ PERTANYIN AL MATEIX PAÍS I PROVÍNCIA
        SELECT CO.ID_PAIS, CO.PROVINCIA, CD.ID_PAIS, CD.PROVINCIA
        INTO V_ID_PAIS_1, V_PROVINCIA_1, V_ID_PAIS_2, V_PROVINCIA_2
        FROM COMPTADORS CO, CENTRALETES_DISTRIBUCIO CD
        WHERE CO.ID_COMPTADOR = M_ID_COMPTADOR AND CD.ID_CENTL_DIST = M_ID_CENTL_DIST;
    
        IF (V_ID_PAIS_1 <> V_ID_PAIS_2)
        THEN
            RAISE EX_DIFERENT_PAIS;
        ELSE
            IF (V_PROVINCIA_1 <> V_PROVINCIA_2)
            THEN
                  RAISE EX_DIFERENT_PROVINCIA;
            END IF;
        END IF;
    
      -- VALIDEM SI LA CENTRALETA DE DISTRIBUCIÓ TÉ SUBMINISTRAMENT ELÈCTRIC
        SELECT COUNT(ID_CENT_PROD)
        INTO   V_COUNT
        FROM   CON_PRODUCCIO_DISTRIBUCIO
        WHERE  ID_CENTL_DIST = M_ID_CENTL_DIST;
    
        IF (V_COUNT = 0)
        THEN 
            RAISE EX_CENTL_DIST_ENERGIA_NOK;
        END IF;
    
    
        -- MODIFIQUEM LA CONNEXIO ENTRE COMPTADOR I CENTRALETA DE DISTRIBUCIÓ
        UPDATE CON_DISTRIBUCIO_COMPTADOR
        SET    ID_CENTL_DIST = M_ID_CENTL_DIST
        WHERE  ID_COMPTADOR = M_ID_COMPTADOR;
       
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
        -- ACTUALITZEM ESTADISTIQUES
        PKG_MODUL_ESTADISTIC.ESTADISTIQUES(R_RSP);
    
    
       EXCEPTION 
          WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' . COMPTADOR';
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN NO_DATA_FOUND THEN
               V_ERROR := -01004;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||' .COMPTADOR: '||M_ID_COMPTADOR;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_DIFERENT_PAIS THEN
               V_ERROR := -02008;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_DIFERENT_PROVINCIA THEN
               V_ERROR := -02009;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_CENTL_DIST_ENERGIA_NOK THEN
               V_ERROR := -02009;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC||'. CENTRALETA DISTRIBUCIÓ: '||M_ID_CENTL_DIST;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
           INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END MODIF_CON_DIST_COMP;


END PKG_ABM_CONNEXIONS;
/
/**************************************************************************************************************************************************
**************************************************************************************************************************************************/        
CREATE OR REPLACE PACKAGE PKG_ABM_CONTRACTES AS
    /*========================================================================================================================================
    NOM: ALTA_CONTRACTE
    DESCRIPCIÓ: PROCEDIMENT PER INSERTAR UN NOU CONTRACTE
    PARAMETERS:
          INPUT: ID_CONTRACTE, ID_CLIENT, ID_COMPTADOR, CUPS, POTENCIA_CONTRACTADA, TARIFA_ACCES, DATA_FI_CONTRACTE
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: ALTA_CONTRACTE (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE ALTA_CONTRACTE (
          A_ID_CONTRACTE  IN CONTRACTES.ID_CONTRACTE%TYPE,
          A_ID_CLIENT     IN CONTRACTES.ID_CLIENT%TYPE,
          A_ID_COMPTADOR  IN CONTRACTES.ID_COMPTADOR%TYPE,
          A_CUPS          IN CONTRACTES.CUPS%TYPE,
          A_POTENCIA_CONTRACTADA IN CONTRACTES.POTENCIA_CONTRACTADA%TYPE,
          A_TARIFA_ACCES  IN CONTRACTES.TARIFA_ACCES%TYPE,
          A_DATA_FI_CONTRACTE IN CONTRACTES.DATA_FI_CONTRACTE%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);

    /*========================================================================================================================================
    NOM: MODIF_CONTRACTE_PROPIETATS
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR EL CLIENT, EL COMPTADOR I EL CUPS D'UN CONTARCTE
    PARAMETERS:
          INPUT: ID_CONTRACTE, ID_CLIENT, ID_COMPTADOR, CUPS
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: MODIFICACIO_CONTRACT_PROPIETATS (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE MODIF_CONTRACTE_PROPIETATS (
          M_ID_CONTRACTE  IN CONTRACTES.ID_CONTRACTE%TYPE,
          M_ID_CLIENT     IN CONTRACTES.ID_CLIENT%TYPE,
          M_ID_COMPTADOR  IN CONTRACTES.ID_COMPTADOR%TYPE,
          M_CUPS          IN CONTRACTES.CUPS%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
          
   /*========================================================================================================================================
    NOM: BAIXA_CONTRACTE
    DESCRIPCIÓ: PROCEDIMENT PER DONAR DE BAIXA UN CONTRACTE
    PARAMETERS:
          INPUT: ID_CONTRACTE
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: BAIXA_CONTRACTE (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE BAIXA_CONTRACTE (
          B_ID_CONTRACTE IN CONTRACTES.ID_CONTRACTE%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE);

    /*========================================================================================================================================
    NOM: MODIF_CONTRACTE_CONDICIONS
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR LES CONDICIONS CONTRACTADES D'UN CONTRACTE
    PARAMETERS:
          INPUT: ID_CONTRACTE, POTENCIA_CONTRACTADA, TARIFA_ACCES, DATA_FI_CONTRACTE
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: MODIF_CONTRACTE_CONDICIONS (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE MODIF_CONTRACTE_CONDICIONS (
          M_ID_CONTRACTE  IN CONTRACTES.ID_CONTRACTE%TYPE,
          M_POTENCIA_CONTRACTADA IN CONTRACTES.POTENCIA_CONTRACTADA%TYPE,
          M_TARIFA_ACCES  IN CONTRACTES.TARIFA_ACCES%TYPE,
          M_DATA_FI_CONTRACTE IN CONTRACTES.DATA_FI_CONTRACTE%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);

END PKG_ABM_CONTRACTES;
/

CREATE OR REPLACE PACKAGE BODY PKG_ABM_CONTRACTES AS

    /* VARIABLES COMUNS */
    V_PROCEDIMENT LOGS.PROCEDIMENT%TYPE;
    V_PARAM_ENTRADA LOGS.PARAM_ENTRADA%TYPE; 
    V_ERROR MSG_ERRORS.ID_ERROR%TYPE;
    V_DESC MSG_ERRORS.DESCRIPCIO%TYPE;
    V_ID_CONTRACTE CONTRACTES.ID_CONTRACTE%TYPE;
    
    V_COUNT NUMBER;
    V_COUNT_CL NUMBER;
    V_COUNT_CO NUMBER;
    
    EX_COMPTADOR_ENERGIA_NOK EXCEPTION;
    EX_DIFERENT_PAIS EXCEPTION;
    EX_DIFERENT_PROVINCIA EXCEPTION;
    EX_CLIENT_NOK EXCEPTION;
    EX_COMPTADOR_NOK EXCEPTION;
    EX_DATA_FI_NOK EXCEPTION;
    EX_PK_NUL EXCEPTION;
    EX_FK_NUL EXCEPTION;
    PRAGMA EXCEPTION_INIT(EX_PK_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);
    
    
/* ALTA CONTRACTE */
    PROCEDURE ALTA_CONTRACTE (
          A_ID_CONTRACTE IN CONTRACTES.ID_CONTRACTE%TYPE,
          A_ID_CLIENT IN CONTRACTES.ID_CLIENT%TYPE,
          A_ID_COMPTADOR IN CONTRACTES.ID_COMPTADOR%TYPE,
          A_CUPS IN CONTRACTES.CUPS%TYPE,
          A_POTENCIA_CONTRACTADA IN CONTRACTES.POTENCIA_CONTRACTADA%TYPE,
          A_TARIFA_ACCES IN CONTRACTES.TARIFA_ACCES%TYPE,
          A_DATA_FI_CONTRACTE IN CONTRACTES.DATA_FI_CONTRACTE%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS

         BEGIN
         V_PROCEDIMENT := 'ALTA_CONTRACTE';	
         V_PARAM_ENTRADA := 'CONTRACTE: '||A_ID_CONTRACTE||' / CLIENT: '||A_ID_CLIENT||' / COMPTADOR : '||A_ID_COMPTADOR||
                            ' / CUPS: '||A_CUPS||' / POTENCIA_CONTRACTADA: '||A_POTENCIA_CONTRACTADA||
                            ' / TARIFA_ACCES: '||A_TARIFA_ACCES||' / DATA_FI_CONTRACTE: '||A_DATA_FI_CONTRACTE;
         R_RSP := 'OK'; 
    

        -- VERIFIQUEM QUE EL CLIENT EXISTEIXI 
        SELECT COUNT(ID_CLIENT)
        INTO   V_COUNT_CL
        FROM   CLIENTS
        WHERE  ID_CLIENT = A_ID_CLIENT;
    
        IF (V_COUNT_CL = 0)
        THEN 
            RAISE EX_CLIENT_NOK;
        END IF;
    
        -- VERIFIQUEM QUE EL COMPTADOR EXISTEIXI 
        SELECT COUNT(ID_COMPTADOR)
        INTO   V_COUNT_CO
        FROM   COMPTADORS
        WHERE  ID_COMPTADOR = A_ID_COMPTADOR;
    
        IF (V_COUNT_CO = 0)
        THEN 
             RAISE EX_COMPTADOR_NOK;
        END IF;
    
         -- VALIDEM SI EL COMPTADOR TÉ SUBMINISTRAMENT ELÈCTRIC
        SELECT COUNT(ID_CENT_PROD)
        INTO   V_COUNT
        FROM   CON_PRODUCCIO_DISTRIBUCIO
        WHERE  ID_CENTL_DIST IN (
               SELECT ID_CENTL_DIST
               FROM CON_DISTRIBUCIO_COMPTADOR
               WHERE ID_COMPTADOR = A_ID_COMPTADOR);
    
        IF (V_COUNT = 0)
        THEN 
            RAISE EX_COMPTADOR_ENERGIA_NOK;
        END IF;
    
        -- INSERTEM EL CONTRACTE
        INSERT INTO CONTRACTES
        VALUES (A_ID_CONTRACTE, A_ID_CLIENT, A_ID_COMPTADOR, A_CUPS, A_POTENCIA_CONTRACTADA, A_TARIFA_ACCES, A_DATA_FI_CONTRACTE,
                NULL, NULL, NULL, NULL );
    
        -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
        INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
        COMMIT;
        
           
       EXCEPTION 
          WHEN EX_PK_NUL THEN
               V_ERROR := -01001;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR)||'. CONTRACTE';
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_FK_NUL THEN
               V_ERROR := -01002;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN DUP_VAL_ON_INDEX THEN
               V_ERROR := -01003;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR)||'. CONTRACTE: '||A_ID_CONTRACTE;
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_CLIENT_NOK THEN
               V_ERROR := -02014;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR)||' . CLIENT: '||A_ID_CLIENT;
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_COMPTADOR_NOK THEN
               V_ERROR := -02015;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR)||' . COMPTADOR: '||A_ID_COMPTADOR;
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN EX_COMPTADOR_ENERGIA_NOK THEN
               V_ERROR := -02012;
               -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
               V_DESC := MISSATGE_ERROR(V_ERROR)||' . COMPTADOR: '||A_ID_COMPTADOR;
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
               V_ERROR := SQLCODE;
               V_DESC := SUBSTR(SQLERRM, 1, 256);
               R_RSP := V_ERROR|| ': '||V_DESC;
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
               DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END ALTA_CONTRACTE;

/* BAIXA DEL CONTRACTE */
    PROCEDURE BAIXA_CONTRACTE (
          B_ID_CONTRACTE IN CONTRACTES.ID_CONTRACTE%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 

         BEGIN
         V_PROCEDIMENT := 'BAIXA_CONTRACTE';	
         V_PARAM_ENTRADA := 'CONTRACTE: '||B_ID_CONTRACTE;
         R_RSP := 'OK'; 
    

           -- VERIFIQUEM QUE EL CONTRACTE EXISTEIXI
          SELECT ID_CONTRACTE
          INTO   V_ID_CONTRACTE
          FROM   CONTRACTES
          WHERE  ID_CONTRACTE = B_ID_CONTRACTE;
      
      
          -- DONEM DE BAIXA EL CONTRACTE
          DELETE FROM CONTRACTES 
          WHERE  ID_CONTRACTE = B_ID_CONTRACTE;
      
          -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
      
          -- ACTUALITZEM ESTADISTIQUES
          PKG_MODUL_ESTADISTIC.ESTADISTIQUES(R_RSP);
    
      
         EXCEPTION 
            WHEN EX_PK_NUL THEN
                 V_ERROR := -01001;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' . CONTRACTE';
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' . CONTRACTE: '||B_ID_CONTRACTE;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END BAIXA_CONTRACTE;

/* MODIFICACIÓ PROPIETATS DEL CONTRACTE */
    PROCEDURE MODIF_CONTRACTE_PROPIETATS (
          M_ID_CONTRACTE IN CONTRACTES.ID_CONTRACTE%TYPE,
          M_ID_CLIENT IN CONTRACTES.ID_CLIENT%TYPE,
          M_ID_COMPTADOR IN CONTRACTES.ID_COMPTADOR%TYPE,
          M_CUPS IN CONTRACTES.CUPS%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 

         BEGIN
         V_PROCEDIMENT := 'MODIF_CONTRACTE_PROPIETATS';	
         V_PARAM_ENTRADA := 'CONTRACTE: '||M_ID_CONTRACTE||' / CLIENT: '||M_ID_CLIENT||' / COMPTADOR : '||M_ID_COMPTADOR||
                            ' / CUPS: '||M_CUPS;
         R_RSP := 'OK'; 
    

          -- VERIFIQUEM QUE EL CONTRACTE EXISTEIXI A LA TAULA DE CONTRACTES
         SELECT ID_CONTRACTE
         INTO   V_ID_CONTRACTE
         FROM   CONTRACTES
         WHERE  ID_CONTRACTE = M_ID_CONTRACTE;
      
         -- VALIDEM SI EL CLIENT EXISTEIX A LA TAULA DE CLIENTS
         SELECT COUNT(ID_CLIENT)
         INTO V_COUNT_CL
         FROM CLIENTS
         WHERE ID_CLIENT = M_ID_CLIENT;
          
         IF (V_COUNT_CL = 0)
         THEN
             RAISE EX_CLIENT_NOK;
         END IF;
                 -- VALIDEM SI EL COMPTADOR EXISTEIX A LA TAULA DE COMPTADORS
         SELECT COUNT(ID_COMPTADOR)
         INTO V_COUNT_CO
         FROM COMPTADORS
         WHERE ID_COMPTADOR = M_ID_COMPTADOR;
         
         IF (V_COUNT_CO = 0)
         THEN
             RAISE EX_COMPTADOR_NOK;
          END IF;
      
         -- MODIFIQUEM EL CONTRACTE
         UPDATE CONTRACTES
         SET    ID_CLIENT = M_ID_CLIENT,
                ID_COMPTADOR = M_ID_COMPTADOR,
                CUPS = M_CUPS
         WHERE  ID_CONTRACTE = M_ID_CONTRACTE;
               -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
         INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
         COMMIT;
         
         -- ACTUALITZEM ESTADISTIQUES
          PKG_MODUL_ESTADISTIC.ESTADISTIQUES(R_RSP);
    
          EXCEPTION 
           WHEN EX_PK_NUL THEN
                V_ERROR := -01001;
                -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                V_DESC := MISSATGE_ERROR(V_ERROR);
                R_RSP := V_ERROR|| ': '||V_DESC||' . CONTRACTE';
               -- INSERTEM L'ERROR A LA TAULA DE LOGS
                INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN NO_DATA_FOUND THEN
                V_ERROR := -01004;
                -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                V_DESC := MISSATGE_ERROR(V_ERROR);
                R_RSP := V_ERROR|| ': '||V_DESC||'. CONTRACTE: '||M_ID_CONTRACTE;
                -- INSERTEM L'ERROR A LA TAULA DE LOGS
                INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
           WHEN DUP_VAL_ON_INDEX THEN
                V_ERROR := -01003;
                -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                V_DESC := MISSATGE_ERROR(V_ERROR);
                R_RSP := V_ERROR|| ': '||V_DESC||'. CONTRACTE: '||M_ID_CONTRACTE;
                -- INSERTEM L'ERROR A LA TAULA DE LOGS
                INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
           WHEN EX_CLIENT_NOK THEN
                V_ERROR := -02014;
                -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                V_DESC := MISSATGE_ERROR(V_ERROR)||' . COMPTADOR: '||M_ID_COMPTADOR;
                R_RSP := V_ERROR|| ': '||V_DESC;
                -- INSERTEM L'ERROR A LA TAULA DE LOGS
               INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
           WHEN EX_COMPTADOR_NOK THEN
                V_ERROR := -02015;
                -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                V_DESC := MISSATGE_ERROR(V_ERROR)||' . COMPTADOR: '||M_ID_COMPTADOR;
                R_RSP := V_ERROR|| ': '||V_DESC;
                -- INSERTEM L'ERROR A LA TAULA DE LOGS
                INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
           WHEN OTHERS THEN
                V_ERROR := SQLCODE;
                V_DESC := SUBSTR(SQLERRM, 1, 256);
                R_RSP := V_ERROR|| ': '||V_DESC;
                -- INSERTEM L'ERROR A LA TAULA DE LOGS
                INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
      END MODIF_CONTRACTE_PROPIETATS;

/* MODIFICACIÓ DE LES CONDICIONS DEL CONTRACTE */
    PROCEDURE MODIF_CONTRACTE_CONDICIONS (
          M_ID_CONTRACTE IN CONTRACTES.ID_CONTRACTE%TYPE,
          M_POTENCIA_CONTRACTADA IN CONTRACTES.POTENCIA_CONTRACTADA%TYPE,
          M_TARIFA_ACCES IN CONTRACTES.TARIFA_ACCES%TYPE,
          M_DATA_FI_CONTRACTE IN CONTRACTES.DATA_FI_CONTRACTE%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
	  
         BEGIN
         V_PROCEDIMENT := 'MODIF_CONTRACTE_CONDICIONS';	
         V_PARAM_ENTRADA := 'CONTRACTE: '||M_ID_CONTRACTE||' / POTENCIA_CONTRACTADA: '||M_POTENCIA_CONTRACTADA||
                            ' / TARIFA_ACCES: '||M_TARIFA_ACCES||' / DATA_FI_CONTRACTE: '||M_DATA_FI_CONTRACTE;
         R_RSP := 'OK'; 
    

         -- VERIFIQUEM QUE EL CONTRACTE EXISTEIXI
         SELECT ID_CONTRACTE
         INTO   V_ID_CONTRACTE
         FROM   CONTRACTES
         WHERE  ID_CONTRACTE = M_ID_CONTRACTE;
      
         -- VALIDEM QUE LA DATA FI CONTRACTE SIGUI POSTERIOR AL SYSDATE
         IF (M_DATA_FI_CONTRACTE > SYSDATE)
         THEN
             RAISE EX_DATA_FI_NOK;
         END IF;
      
         -- MODIFIQUEM EL CONTRACTE
         UPDATE CONTRACTES
         SET    POTENCIA_CONTRACTADA = M_POTENCIA_CONTRACTADA,
                TARIFA_ACCES = M_TARIFA_ACCES,
                DATA_FI_cONTRACTE = M_DATA_FI_CONTRACTE
         WHERE  ID_CONTRACTE = M_ID_CONTRACTE;
                -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
         INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
         COMMIT;
         
         EXCEPTION 
           WHEN EX_PK_NUL THEN
                V_ERROR := -01001;
                -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                V_DESC := MISSATGE_ERROR(V_ERROR);
                R_RSP := V_ERROR|| ': '||V_DESC||' . CONTRACTE: '||M_ID_CONTRACTE;
                -- INSERTEM L'ERROR A LA TAULA DE LOGS
                INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
           WHEN NO_DATA_FOUND THEN
                V_ERROR := -01004;
                -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                V_DESC := MISSATGE_ERROR(V_ERROR);
                R_RSP := V_ERROR|| ': '||V_DESC||'. CONTRACTE: '||M_ID_CONTRACTE;
                -- INSERTEM L'ERROR A LA TAULA DE LOGS
                INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
           WHEN EX_DATA_FI_NOK THEN
                V_ERROR := -02013;
                -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                V_DESC := MISSATGE_ERROR(V_ERROR)||' . DATA FI CONTRACTE: '||M_DATA_FI_CONTRACTE;
                R_RSP := V_ERROR|| ': '||V_DESC;
                -- INSERTEM L'ERROR A LA TAULA DE LOGS
                INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
           WHEN OTHERS THEN
                V_ERROR := SQLCODE;
                V_DESC := SUBSTR(SQLERRM, 1, 256);
                R_RSP := V_ERROR|| ': '||V_DESC;
                -- INSERTEM L'ERROR A LA TAULA DE LOGS
                INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
      END MODIF_CONTRACTE_CONDICIONS;

END PKG_ABM_CONTRACTES;
/
/**************************************************************************************************************************************************
**************************************************************************************************************************************************/        
CREATE OR REPLACE PACKAGE PKG_ABM_DISTRIBUCIO AS
   /*========================================================================================================================================
    NOM: ALTA_CENTL_DIST
    DESCRIPCIÓ: PROCEDIMENT PER INSERTAR UNA NOVA CENTRALETA DE DISTRIBUCIÓ
    PARAMETERS:
          INPUT: ID_COMPTADOR, MODEL_COMPTADOR, NUM_SERIE, TIPUS_SUMINISTRE, KHW_CONSUMITS, DOMICILI, CODI_POSTAL, 
                 LOCALITAT, PROVINCIA, ID_PAIS, DATA_INSTAL·LACIO, EMPRESA_INSTAL·LACIO, TELF_EMPRESA
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: ALTA_CENTL_DIST (002506,'098-AC', '00123698547', 'Monofàisc', 88.866,'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                       'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
   ========================================================================================================================================*/
   PROCEDURE ALTA_CENTL_DIST (
         A_ID_CENTL_DIST     IN CENTRALETES_DISTRIBUCIO.ID_CENTL_DIST%TYPE,
         A_ENERGIA_MAX_SUB   IN CENTRALETES_DISTRIBUCIO.ENERGIA_MAX_SUB%TYPE,
         A_DATA_ULT_INSP_TECNICA IN CENTRALETES_DISTRIBUCIO.DATA_ULT_INSP_TECNICA%TYPE,
         A_DOMICILI          IN CENTRALETES_DISTRIBUCIO.DOMICILI%TYPE,
         A_CODI_POSTAL       IN CENTRALETES_DISTRIBUCIO.CODI_POSTAL%TYPE,
         A_LOCALITAT         IN CENTRALETES_DISTRIBUCIO.LOCALITAT%TYPE,
         A_PROVINCIA         IN CENTRALETES_DISTRIBUCIO.PROVINCIA%TYPE,
         A_ID_PAIS           IN CENTRALETES_DISTRIBUCIO.ID_PAIS%TYPE,
         R_RSP               OUT NOCOPY LOGS.RSP%TYPE);

    /*========================================================================================================================================
      NOM: BAIXA_CENTL_DIST
      DESCRIPCIÓ: PROCEDIMENT PER DONAR DE BAIXA UNA CENTRALETA DE DISTRIBUCIÓ
      PARAMETERS:
            INPUT: ID_CENTL_DIST
            OUTPUT: RST
      RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
      EXEMPLE DE CRIDA: BAIXA_CENTL_DIST (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE BAIXA_CENTL_DIST (
          B_ID_CENTL_DIST     IN CENTRALETES_DISTRIBUCIO.ID_CENTL_DIST%TYPE,
          R_RSP               OUT NOCOPY LOGS.RSP%TYPE);

   /*========================================================================================================================================
    NOM: MODIF_CENTL_DIST_PROPIETATS
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR LES PROPIETATS D'UNA CENTRALETA DE DISTRIBUCIO
    PARAMETERS:
          INPUT: ID_CENTL_DIST, MODEL_COMPTADOR, NUM_SERIE, TIPUS_SUMINISTRE, KHW_CONSUMITS, DOMICILI, CODI_POSTAL, 
                 LOCALITAT, PROVINCIA, ID_PAIS, DATA_INSTAL·LACIO, EMPRESA_INSTAL·LACIO, TELF_EMPRESA
          OUTPUT: RST
   RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
   EXEMPLE DE CRIDA: MODIF_CENTL_DIST_PROPIETATS (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
   ========================================================================================================================================*/
   PROCEDURE MODIF_CENTL_DIST_PROPIETATS (
         M_ID_CENTL_DIST     IN CENTRALETES_DISTRIBUCIO.ID_CENTL_DIST%TYPE,
         M_ENERGIA_MAX_SUB   IN CENTRALETES_DISTRIBUCIO.ENERGIA_MAX_SUB%TYPE,
         R_RSP               OUT NOCOPY LOGS.RSP%TYPE);
         
  
END PKG_ABM_DISTRIBUCIO;

/

/* DECLARACIÓ COS (BODY) DEL PACKAGE */
CREATE OR REPLACE PACKAGE BODY PKG_ABM_DISTRIBUCIO AS

    /* VARIABLES COMUNS */
    V_PROCEDIMENT LOGS.PROCEDIMENT%TYPE;
    V_PARAM_ENTRADA LOGS.PARAM_ENTRADA%TYPE; 
    V_ERROR MSG_ERRORS.ID_ERROR%TYPE;
    V_DESC  MSG_ERRORS.DESCRIPCIO%TYPE;
    EX_PK_NUL EXCEPTION;
    EX_FK_NUL EXCEPTION;
    PRAGMA EXCEPTION_INIT(EX_PK_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);
    V_ID_CENTL_DIST CENTRALETES_DISTRIBUCIO.ID_CENTL_DIST%TYPE;
    V_TOTAL NUMBER;
    
    
    /* ALTA CENTRALETA DE DISTRIBUCIO */
    PROCEDURE ALTA_CENTL_DIST (
            A_ID_CENTL_DIST IN CENTRALETES_DISTRIBUCIO.ID_CENTL_DIST%TYPE,
            A_ENERGIA_MAX_SUB IN CENTRALETES_DISTRIBUCIO.ENERGIA_MAX_SUB%TYPE,
            A_DATA_ULT_INSP_TECNICA IN CENTRALETES_DISTRIBUCIO.DATA_ULT_INSP_TECNICA%TYPE,
            A_DOMICILI IN CENTRALETES_DISTRIBUCIO.DOMICILI%TYPE,
            A_CODI_POSTAL IN CENTRALETES_DISTRIBUCIO.CODI_POSTAL%TYPE,
            A_LOCALITAT IN CENTRALETES_DISTRIBUCIO.LOCALITAT%TYPE,
            A_PROVINCIA IN CENTRALETES_DISTRIBUCIO.PROVINCIA%TYPE,
            A_ID_PAIS IN CENTRALETES_DISTRIBUCIO.ID_PAIS%TYPE,
            R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
            
        BEGIN
            V_PROCEDIMENT := 'ALTA_CENTL_DIST';	
            V_PARAM_ENTRADA := 'CENT_DIST: '||A_ID_CENTL_DIST||' / ENERGIA_MAX_SUB: '||A_ENERGIA_MAX_SUB||
                              ' / DATA_ULT_INSP_TECNICA: '||A_DATA_ULT_INSP_TECNICA||' / DOMICILI: '||A_DOMICILI||
                              ' / CODI_POSTAL: '||A_CODI_POSTAL||' / LOCALITAT: '||A_LOCALITAT||' / PROVINCIA: '||
                              A_PROVINCIA||' / ID_PAIS: '||A_ID_PAIS;
            R_RSP := 'OK'; 
            
            -- INSERTEM LA CENTRALETA DE DISTRIBUCIÓ
            INSERT INTO CENTRALETES_DISTRIBUCIO 
            VALUES (A_ID_CENTL_DIST, A_ENERGIA_MAX_SUB, A_DATA_ULT_INSP_TECNICA, A_DOMICILI, A_CODI_POSTAL, 
                    A_LOCALITAT, A_PROVINCIA, A_ID_PAIS, NULL, NULL, NULL, NULL);
        
            -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
            COMMIT;
        
           EXCEPTION 
             WHEN EX_PK_NUL THEN
                   V_ERROR := -01001;
                   -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                   V_DESC := MISSATGE_ERROR(V_ERROR);
                   R_RSP := V_ERROR|| ': '||V_DESC||' . CENTRALETA DISTRIBUCIÓ';
                   -- INSERTEM L'ERROR A LA TAULA DE LOGS
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                   DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_FK_NUL THEN
                   V_ERROR := -01002;
                   -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                   V_DESC := MISSATGE_ERROR(V_ERROR);
                   R_RSP := V_ERROR|| ': '||V_DESC;
                   -- INSERTEM L'ERROR A LA TAULA DE LOGS
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                   DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN DUP_VAL_ON_INDEX THEN
                   V_ERROR := -01003;
                   -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                   V_DESC := MISSATGE_ERROR(V_ERROR);
                   R_RSP := V_ERROR|| ': '||V_DESC||' . CENTRALETA DISTRIBUCIÓ: '||A_ID_PAIS||' - '||A_LOCALITAT||' - '||A_CODI_POSTAL||' - '||A_DOMICILI;
                   -- INSERTEM L'ERROR A LA TAULA DE LOGS
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                   DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN OTHERS THEN
                   V_ERROR := SQLCODE;
                   V_DESC := SUBSTR(SQLERRM, 1, 256);
                   R_RSP := V_ERROR|| ': '||V_DESC;
                   -- INSERTEM L'ERROR A LA TAULA DE LOGS
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                   DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END ALTA_CENTL_DIST;     
    
    /* BAIXA D'UNA CENTRALETA DE DISTRIBUCIÓ */
    PROCEDURE BAIXA_CENTL_DIST (
            B_ID_CENTL_DIST IN CENTRALETES_DISTRIBUCIO.ID_CENTL_DIST%TYPE,
            R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS

    
        EX_CENTL_DIST_ACTIVA EXCEPTION;
	  
        BEGIN
            V_PROCEDIMENT := 'BAIXA_CENTL_DIST';	
            V_PARAM_ENTRADA := 'CENT_DIST: '||B_ID_CENTL_DIST;
            R_RSP := 'OK'; 
    
  
             -- VERIFIQUEM QUE LA CENTRALETA DE DISTRIBUCIÓ A MODIFICAR EXISTEIXI A LA TAULA
            SELECT ID_CENTL_DIST
            INTO   V_ID_CENTL_DIST
            FROM   CENTRALETES_DISTRIBUCIO
            WHERE  ID_CENTL_DIST = B_ID_CENTL_DIST;
        
             -- VERIFIQUEM QUE LA CENTRALETA DE DISTRIBUCIÓ NO TINGUI COMPTADORS ASSOCIATS EN ACTIU (CONTADORS ASSOCIATS A CONTRACTES)
            SELECT COUNT(1)
            INTO   V_TOTAL
            FROM   CONTRACTES
            WHERE  ID_COMPTADOR IN (
                       SELECT ID_COMPTADOR
                       FROM   CON_DISTRIBUCIO_COMPTADOR
                       WHERE  ID_CENTL_DIST = B_ID_CENTL_DIST);
          
            IF (V_TOTAL > 0)
            THEN
                  RAISE EX_CENTL_DIST_ACTIVA;
            END IF;
            
            -- ESBORREM LES DADES DE LA CENTRALETA DE DISTRIBUCIÓ 
            -- 1. TAULA REL_COMPTADOR_CENTL_DIST
            DELETE FROM CON_DISTRIBUCIO_COMPTADOR WHERE ID_CENTL_DIST = B_ID_CENTL_DIST;    
            -- 2. TAULA CON_PRODUCCIO_DISTRIBUCIO
            DELETE FROM CON_PRODUCCIO_DISTRIBUCIO WHERE ID_CENTL_DIST = B_ID_CENTL_DIST;    
            -- 3. TAULA CENTRALETES_DISTRIBUCIO
            DELETE FROM CENTRALETES_DISTRIBUCIO WHERE ID_CENTL_DIST = B_ID_CENTL_DIST;   
         
            -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
            COMMIT;
            
            -- ACTUALITZEM ESTADISTIQUES
            PKG_MODUL_ESTADISTIC.ESTADISTIQUES(R_RSP);
    
            
           EXCEPTION 
             WHEN EX_PK_NUL THEN
                   V_ERROR := -01001;
                   -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                   V_DESC := MISSATGE_ERROR(V_ERROR);
                   R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRALETA DISTRIBUCIO';
                   -- INSERTEM L'ERROR A LA TAULA DE LOGS
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                   DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
              WHEN NO_DATA_FOUND THEN
                   V_ERROR := -01004;
                   -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                   V_DESC := MISSATGE_ERROR(V_ERROR);
                   R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRALETA DISTRIBUCIO: '||B_ID_CENTL_DIST;
                   -- INSERTEM L'ERROR A LA TAULA DE LOGS
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                   DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
              WHEN EX_CENTL_DIST_ACTIVA THEN
                   V_ERROR := -02003;
                   -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                   V_DESC := MISSATGE_ERROR(V_ERROR);
                   R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRALETA DISTRIBUCIO: '||B_ID_CENTL_DIST;
                   -- INSERTEM L'ERROR A LA TAULA DE LOGS
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                   DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
              WHEN OTHERS THEN
                   V_ERROR := SQLCODE;
                   V_DESC := SUBSTR(SQLERRM, 1, 256);
                   R_RSP := V_ERROR|| ': '||V_DESC;
                   -- INSERTEM L'ERROR A LA TAULA DE LOGS
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                   DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END BAIXA_CENTL_DIST;
    
    /* MODIFICAICÓ DE LES PROPIETATS D'UNA CENTRALETA */
    PROCEDURE MODIF_CENTL_DIST_PROPIETATS (
          M_ID_CENTL_DIST IN CENTRALETES_DISTRIBUCIO.ID_CENTL_DIST%TYPE,
          M_ENERGIA_MAX_SUB IN CENTRALETES_DISTRIBUCIO.ENERGIA_MAX_SUB%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
          
       BEGIN
            V_PROCEDIMENT := 'MODIF_CENTL_DIST_PROPIETATS';	
            V_PARAM_ENTRADA := 'CENT_DIST: '||M_ID_CENTL_DIST||' / ENERGIA_MAX_SUB: '||M_ENERGIA_MAX_SUB;
            R_RSP := 'OK'; 
            
          
            -- VERIFIQUEM QUE LA CENTRALETA DE DISTRIBUCIÓ A MODIFICAR EXISTEIXI A LA TAULA
            SELECT ID_CENTL_DIST
            INTO   V_ID_CENTL_DIST
            FROM   CENTRALETES_DISTRIBUCIO
            WHERE  ID_CENTL_DIST = M_ID_CENTL_DIST;
        
            -- MODIFIQUEM LES DADES DE LA CENTRALETA DE DISTRIBUCIÓ
            UPDATE CENTRALETES_DISTRIBUCIO
            SET    ENERGIA_MAX_SUB = M_ENERGIA_MAX_SUB
            WHERE  ID_CENTL_DIST = M_ID_CENTL_DIST;
        
            -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
            INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
            COMMIT;
            
           EXCEPTION 
              WHEN EX_PK_NUL THEN
                   V_ERROR := -01001;
                   -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                   V_DESC := MISSATGE_ERROR(V_ERROR);
                   R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRALETA DISTRIBUCIO';
                   -- INSERTEM L'ERROR A LA TAULA DE LOGS
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                   DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
              WHEN NO_DATA_FOUND THEN
                   V_ERROR := -01004;
                   -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                   V_DESC := MISSATGE_ERROR(V_ERROR);
                   R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRALETA DISTRIBUCIO: '||M_ID_CENTL_DIST;
                   -- INSERTEM L'ERROR A LA TAULA DE LOGS
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                   DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
              WHEN OTHERS THEN
                   V_ERROR := SQLCODE;
                   V_DESC := SUBSTR(SQLERRM, 1, 256);
                   R_RSP := V_ERROR|| ': '||V_DESC;
                   -- INSERTEM L'ERROR A LA TAULA DE LOGS
                   INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                   DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
      END MODIF_CENTL_DIST_PROPIETATS;     
    
END PKG_ABM_DISTRIBUCIO;      
/
/**************************************************************************************************************************************************
**************************************************************************************************************************************************/        
CREATE OR REPLACE PACKAGE PKG_ABM_PRODUCCIO AS
   /*========================================================================================================================================
   NOM: ALTA_CENT_PROD
   DESCRIPCIÓ: PROCEDIMENT PER INSERTAR UNA NOVA CENTRAL DE PRODUCCIÓ
   PARAMETERS:
      INPUT: ID_CENT_PROD, TIPUS_CENTRAL, ENERGIA_MIN, EMISSIONS, ENERGIA_MAX_PROD, DATA_ULT_INSP_TECNICA, TELEFON, 
	     DOMICILI, CODI_POSTAL, LOCALITAT, PROVINCIA, ID_PAIS
      OUTPUT: RST
   RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
   EXEMPLE DE CRIDA: ALTA_CENT_PROD (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
   ========================================================================================================================================*/
   PROCEDURE ALTA_CENT_PROD (
         A_ID_CENT_PROD   IN CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
         A_TIPUS_CENTRAL  IN CENTRALS_PRODUCCIO.TIPUS_CENTRAL%TYPE,
         A_ENERGIA_MIN    IN CENTRALS_PRODUCCIO.ENERGIA_MIN%TYPE,
         A_REBUIG         IN CENTRALS_PRODUCCIO.REBUIG%TYPE,
         A_EMISSIONS      IN CENTRALS_PRODUCCIO.EMISSIONS%TYPE,
         A_MOLINS         IN CENTRALS_PRODUCCIO.MOLINS%TYPE,
         A_PANELLS        IN CENTRALS_PRODUCCIO.PANELLS%TYPE,
         A_ENERGIA_MAX_PROD IN CENTRALS_PRODUCCIO.ENERGIA_MAX_PROD%TYPE,
         A_DATA_ULT_INSP_TECNICA IN CENTRALS_PRODUCCIO.DATA_ULT_INSP_TECNICA%TYPE,
         A_TELEFON        IN CENTRALS_PRODUCCIO.TELEFON%TYPE,
         A_DOMICILI       IN CENTRALS_PRODUCCIO.DOMICILI%TYPE,
         A_CODI_POSTAL    IN CENTRALS_PRODUCCIO.CODI_POSTAL%TYPE,
         A_LOCALITAT      IN CENTRALS_PRODUCCIO.LOCALITAT%TYPE,
         A_PROVINCIA      IN CENTRALS_PRODUCCIO.PROVINCIA%TYPE,
         A_ID_PAIS        IN CENTRALS_PRODUCCIO.ID_PAIS%TYPE,
         R_RSP            OUT NOCOPY LOGS.RSP%TYPE);
      
   /*========================================================================================================================================
   NOM: BAIXA_CENT_PROD
   DESCRIPCIÓ: PROCEDIMENT PER DONAR DE BAIXA UNA CENTRAL DE PRODUCCIÓ
   PARAMETERS:
      INPUT: ID_CENT_PROD
      OUTPUT: RST
   RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
   EXEMPLE DE CRIDA: BAIXA_CENT_PROD (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
   ========================================================================================================================================*/
   PROCEDURE BAIXA_CENT_PROD (
         B_ID_CENT_PROD   IN CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
         R_RSP            OUT NOCOPY LOGS.RSP%TYPE);

   /*========================================================================================================================================
   NOM: MODIF_CENT_PROD_PROPIETATS
   DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR LES PROPIETATS D'UNA CENTRAL DE PRODUCCIÓ
   PARAMETERS:
         INPUT: ID_CENT_PROD, TIPUS_CENTRAL, ENERGIA_MIN, EMISSIONS, 
         OUTPUT: RST
   RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
   EXEMPLE DE CRIDA: MODIF_CENT_PROD_PROPIETATS (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
   ========================================================================================================================================*/
   PROCEDURE MODIF_CENT_PROD_PROPIETATS (
         M_ID_CENT_PROD   IN CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
         M_TIPUS_CENTRAL  IN CENTRALS_PRODUCCIO.TIPUS_CENTRAL%TYPE,
         M_ENERGIA_MIN    IN CENTRALS_PRODUCCIO.ENERGIA_MIN%TYPE,
         M_REBUIG         IN CENTRALS_PRODUCCIO.REBUIG%TYPE,
         M_EMISSIONS      IN CENTRALS_PRODUCCIO.EMISSIONS%TYPE,
         M_MOLINS         IN CENTRALS_PRODUCCIO.MOLINS%TYPE,
         M_PANELLS        IN CENTRALS_PRODUCCIO.PANELLS%TYPE,
         R_RSP            OUT NOCOPY LOGS.RSP%TYPE);
  
    /*========================================================================================================================================
    NOM: MODIF_CENT_PROD_LOCALITZACIO
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR LA LOCALITZACIO D'UNA CENTRAL DE PRODUCCIÓ
    PARAMETERS:
          INPUT: ID_CENT_PROD, TELEFON, DOMICILI, CODI_POSTAL, LOCALITAT, PROVINCIA, ID_PAIS
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: MODIF_CENTL_DIST_LOCALITZACIO (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
		                     'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                                     'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE MODIF_CENT_PROD_LOCALITZACIO (
          M_ID_CENT_PROD  IN CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
          M_TELEFON       IN CENTRALS_PRODUCCIO.TELEFON%TYPE,
          M_DOMICILI      IN CENTRALS_PRODUCCIO.DOMICILI%TYPE,
          M_CODI_POSTAL   IN CENTRALS_PRODUCCIO.CODI_POSTAL%TYPE,
          M_LOCALITAT     IN CENTRALS_PRODUCCIO.LOCALITAT%TYPE,
          M_PROVINCIA     IN CENTRALS_PRODUCCIO.PROVINCIA%TYPE,
          M_ID_PAIS       IN CENTRALS_PRODUCCIO.ID_PAIS%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);

    /*========================================================================================================================================
    NOM: MODIF_CENT_PROD_ENERG_PROD
    DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR L'ENERGIA MÀXIMA PRODUñIDA PER UNA CENTRAL DE PRODUCCIÓ
    PARAMETERS:
          INPUT: ID_CENT_PROD, ENERGIA_MAX_PROD 
          OUTPUT: RST
    RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
    EXEMPLE DE CRIDA: MODIF_CENT_PROD_ENERG_PROD (002506,'098-AC', '00123698547', 'Monofàisc', 88.866, 
                           'Carrer Roselló 12, 1 4', 08036, 'Barcelona', 'Barcelona',
                            'ES', 25/04/1995, 'Instal·lacions Riera S.L', '935550206');  
    ========================================================================================================================================*/
    PROCEDURE MODIF_CENT_PROD_ENERG_MAX_PROD (
          M_ID_CENT_PROD  IN CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
          M_ENERGIA_MAX_PROD IN CENTRALS_PRODUCCIO.ENERGIA_MAX_PROD%TYPE,
          R_RSP           OUT NOCOPY LOGS.RSP%TYPE);
          
  /*========================================================================================================================================
   NOM: ALTA_HIST_COMPTADORS
   DESCRIPCIÓ: PROCEDIMENT PER DOAR D'ALTA UNA PRODUCCIÓ HISTÒRICA
   PARAMETERS:
      INPUT: ID_CENT_PROD, DATA_MENSUAL, ENERGIA_PROD
      OUTPUT: RST
   RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
   EXEMPLE DE CRIDA: ALTA_HIST_CENT_PROD(192053,'02/08/2011',6780,RESP);
    ========================================================================================================================================*/
    PROCEDURE ALTA_HIST_CENT_PROD (
          A_ID_CENT_PROD IN HIST_CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
          A_DATA_MENSUAL IN HIST_CENTRALS_PRODUCCIO.DATA_MENSUAL%TYPE,
          A_ENERGIA_PROD IN HIST_CENTRALS_PRODUCCIO.ENERGIA_PROD%TYPE,
          R_RSP          OUT NOCOPY LOGS.RSP%TYPE);
          
  /*========================================================================================================================================
   NOM: BAIXA_HIST_COMPTADORS
   DESCRIPCIÓ: PROCEDIMENT PER DONAR DE BAIXA UNA PRODUCCIÓ HISTÒRICA
   PARAMETERS:
      INPUT: ID_CENT_PROD, DATA_MENSUAL
      OUTPUT: RST
   RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
   EXEMPLE DE CRIDA: BAIXA_HIST_CENT_PROD(192053,'02/08/2011',RESP);
    ========================================================================================================================================*/
    PROCEDURE BAIXA_HIST_CENT_PROD (
          B_ID_CENT_PROD IN HIST_CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
          B_DATA_MENSUAL IN HIST_CENTRALS_PRODUCCIO.DATA_MENSUAL%TYPE,
          R_RSP          OUT NOCOPY LOGS.RSP%TYPE);
          
   /*========================================================================================================================================
   NOM: MODIF_HIST_COMPTADORS
   DESCRIPCIÓ: PROCEDIMENT PER MODIFICAR UNA PRODUCCIÓ HISTÒRICA
   PARAMETERS:
      INPUT: ID_CENT_PROD, DATA_MENSUAL, ENERGIA_PROD
      OUTPUT: RST
   RETORNA: VALOR 'OK' SI L'EXECUCIÓ HA ACABAT CORRECTAMENT O VALOR 'ERROR + TIPUS D'ERROR' EN CAS CONTRARI
   EXEMPLE DE CRIDA: MODIF_HIST_CENT_PROD(192053,'02/08/2011',6780,RESP);
    ========================================================================================================================================*/
    PROCEDURE MODIF_HIST_CENT_PROD (
          M_ID_CENT_PROD IN HIST_CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
          M_DATA_MENSUAL IN HIST_CENTRALS_PRODUCCIO.DATA_MENSUAL%TYPE,
          M_ENERGIA_PROD IN HIST_CENTRALS_PRODUCCIO.ENERGIA_PROD%TYPE,
          R_RSP          OUT NOCOPY LOGS.RSP%TYPE);
          
  
END PKG_ABM_PRODUCCIO;
/

/* DECLARACIÓ COS (BODY) DEL PACKAGE */
CREATE OR REPLACE PACKAGE BODY PKG_ABM_PRODUCCIO AS

    /* VARIABLES COMUNS */
    V_PROCEDIMENT LOGS.PROCEDIMENT%TYPE;
    V_PARAM_ENTRADA LOGS.PARAM_ENTRADA%TYPE; 
    V_PARAM_SORTIDA LOGS.PARAM_SORTIDA%TYPE; 
    V_ERROR MSG_ERRORS.ID_ERROR%TYPE;
    V_DESC  MSG_ERRORS.DESCRIPCIO%TYPE;
    V_ENERGIA_MIN  CENTRALS_PRODUCCIO.ENERGIA_MIN%TYPE;
    V_REBUIG CENTRALS_PRODUCCIO.REBUIG%TYPE;
    V_EMISSIONS CENTRALS_PRODUCCIO.EMISSIONS%TYPE;
    V_MOLINS CENTRALS_PRODUCCIO.MOLINS%TYPE;
    V_PANELLS CENTRALS_PRODUCCIO.PANELLS%TYPE;
    V_ID_CENT_PROD CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE;
    V_TOTAL NUMBER;
    EX_CENT_PROD_ACTIU EXCEPTION;
    EX_NUCLEAR EXCEPTION;
    EX_TERMICA EXCEPTION;
    EX_CARBO EXCEPTION;
    EX_EOLICA EXCEPTION;
    EX_SOLAR EXCEPTION;
    EX_PK_NUL EXCEPTION;
    EX_FK_NUL EXCEPTION;
    EX_TIPUS_NOK EXCEPTION;
    PRAGMA EXCEPTION_INIT(EX_PK_NUL, -01400);
    PRAGMA EXCEPTION_INIT(EX_FK_NUL, -02291);
    PRAGMA EXCEPTION_INIT(EX_TIPUS_NOK, -02290);
    
    
/* ALTA CENTRAL DE PRODUCCIÓ */
    PROCEDURE ALTA_CENT_PROD (
          A_ID_CENT_PROD IN CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
          A_TIPUS_CENTRAL IN CENTRALS_PRODUCCIO.TIPUS_CENTRAL%TYPE,
          A_ENERGIA_MIN IN CENTRALS_PRODUCCIO.ENERGIA_MIN%TYPE,
          A_REBUIG IN CENTRALS_PRODUCCIO.REBUIG%TYPE,
          A_EMISSIONS IN CENTRALS_PRODUCCIO.EMISSIONS%TYPE,
          A_MOLINS IN CENTRALS_PRODUCCIO.MOLINS%TYPE,
          A_PANELLS IN CENTRALS_PRODUCCIO.PANELLS%TYPE,
          A_ENERGIA_MAX_PROD IN CENTRALS_PRODUCCIO.ENERGIA_MAX_PROD%TYPE,
          A_DATA_ULT_INSP_TECNICA IN CENTRALS_PRODUCCIO.DATA_ULT_INSP_TECNICA%TYPE,
          A_TELEFON IN CENTRALS_PRODUCCIO.TELEFON%TYPE,
          A_DOMICILI IN CENTRALS_PRODUCCIO.DOMICILI%TYPE,
          A_CODI_POSTAL IN CENTRALS_PRODUCCIO.CODI_POSTAL%TYPE,
          A_LOCALITAT IN CENTRALS_PRODUCCIO.LOCALITAT%TYPE,
          A_PROVINCIA IN CENTRALS_PRODUCCIO.PROVINCIA%TYPE,
          A_ID_PAIS IN CENTRALS_PRODUCCIO.ID_PAIS%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 

    
         BEGIN
         V_PROCEDIMENT := 'ALTA_CENT_PROD';	
         V_PARAM_ENTRADA := 'CENT_PROD: '||A_ID_CENT_PROD||' / TIPUS_CENTRAL: '||A_TIPUS_CENTRAL||' / ENERGIA_MIN: '||A_ENERGIA_MIN||
                            ' / REBUIG: '||A_REBUIG||' / EMISSIONS: '||A_EMISSIONS||' / MOLINS: '||A_MOLINS||
                            ' / PANELLS: '||A_PANELLS||' / ENERGIA_MAX_PROD: '||A_ENERGIA_MAX_PROD||
                            ' / DATA_ULT_INSP_TECNICA: '||A_DATA_ULT_INSP_TECNICA||
	                          ' / TELEFON: '||A_TELEFON||' / DOMICILI: '||A_DOMICILI||' / CODI_POSTAL: '||A_CODI_POSTAL||
                            ' / LOCALITAT: '||A_LOCALITAT||' / PROVINCIA: '||A_PROVINCIA||' / ID_PAIS: '||A_ID_PAIS;
         R_RSP := 'OK'; 
    
         CASE A_TIPUS_CENTRAL
         WHEN 'NUCLEAR' THEN
             -- VALIDEM SI LA CENTRAL ÉS 'NUCLEAR', S'HA DE INFORMAR EL CAMP ENERGIA_MIN I EL CAMPREBUIG (CONTÉ ELS KG DE REBUIG RADIOACTIU GENERAT)
             V_EMISSIONS := NULL;
             V_MOLINS := NULL;
             V_PANELLS := NULL;
             IF (A_ENERGIA_MIN IS NULL OR A_REBUIG IS NULL) 
             THEN 
                  RAISE EX_NUCLEAR;
             ELSE
                 V_ENERGIA_MIN := A_ENERGIA_MIN;
                 V_REBUIG := A_REBUIG;
             END IF;
         WHEN 'TÈRMICA' THEN
              -- VALIDEM SI LA CENTRAL ÉS TÈRMICA, S'HA DE INFORMAR EL CAMP EMISSIONS
              V_REBUIG := NULL;
              V_MOLINS := NULL;
              V_PANELLS := NULL;
              IF (A_EMISSIONS IS NULL)
              THEN
                    RAISE EX_TERMICA;
              ELSE
                    V_EMISSIONS := A_EMISSIONS;
              END IF;
         WHEN 'CARBÓ' THEN 
              -- VALIDEM SI LA CENTRAL ÉS CARBÓ, S'HA DE INFORMAR EL CAMP EMISSIONS
              V_REBUIG := NULL;
              V_MOLINS := NULL;
              V_PANELLS := NULL;
              IF (A_EMISSIONS IS NULL)
              THEN
                    RAISE EX_CARBO;
              ELSE
                    V_EMISSIONS := A_EMISSIONS;
              END IF;
         WHEN 'EÒLICA' THEN
              -- VALIDEM SI LA CENTRAL ÉS TÈRMICA, S'HA DE INFORMAR EL CAMP MOLINS
              V_REBUIG := NULL;
              V_EMISSIONS := NULL;
              V_PANELLS := NULL;
              IF (A_MOLINS IS NULL)
              THEN
                    RAISE EX_EOLICA;
              ELSE
                    V_MOLINS := A_MOLINS;
              END IF;
         WHEN 'SOLAR' THEN
              -- VALIDEM SI LA CENTRAL ÉS SOLAR, S'HA DE INFORMAR EL CAMP PANELLS
              V_REBUIG := NULL;
              V_MOLINS := NULL;
              V_EMISSIONS := NULL;
              IF (A_PANELLS IS NULL)
              THEN 
                    RAISE EX_SOLAR;
              ELSE
                    V_PANELLS := A_PANELLS;
              END IF;
         ELSE
            RAISE EX_TIPUS_NOK;
         END CASE;
        
            -- INSERTEM LA CENTRAL DE PRODUCCIÓ
          INSERT INTO CENTRALS_PRODUCCIO 
          VALUES (A_ID_CENT_PROD, A_TIPUS_CENTRAL, V_ENERGIA_MIN, V_REBUIG, V_EMISSIONS, V_MOLINS, V_PANELLS, A_ENERGIA_MAX_PROD, 
                  A_DATA_ULT_INSP_TECNICA, A_TELEFON, A_DOMICILI, A_CODI_POSTAL, A_LOCALITAT, A_PROVINCIA, A_ID_PAIS, NULL, NULL, NULL, NULL);
      

          -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
      
          EXCEPTION 
            WHEN EX_PK_NUL THEN
                 V_ERROR := -01001;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' . CENTRAL PRODUCCIÓ';
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_FK_NUL THEN
                 V_ERROR := -01002;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 IF (A_ID_PAIS IS NULL)
                 THEN 
                      R_RSP := V_ERROR|| ': '||V_DESC||' . PAIS: '||A_ID_PAIS;
                 END IF;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN DUP_VAL_ON_INDEX THEN
                 V_ERROR := -01003;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' . CENTRAL PRODUCCIÓ: '||A_ID_CENT_PROD;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_NUCLEAR THEN
                 V_ERROR := -02004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_TERMICA THEN
                 V_ERROR := -02005;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_CARBO THEN
                 V_ERROR := -02006;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_EOLICA THEN
                 V_ERROR := -02016;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_SOLAR THEN
                 V_ERROR := -02017;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_TIPUS_NOK THEN
                 V_ERROR := -01005;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END ALTA_CENT_PROD;
    
/* BAIXA CENTRAL DE PRODUCCIO */
    PROCEDURE BAIXA_CENT_PROD (
          B_ID_CENT_PROD IN CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
      
         BEGIN
         V_PROCEDIMENT := 'BAIXA_CENT_PROD';	
         V_PARAM_ENTRADA := 'CENT_PROD: '||B_ID_CENT_PROD;
         R_RSP := 'OK'; 
    
  
        -- VERIFIQUEM QUE LA CENTRAL DE PRODUCCIÓ A MODIFICAR EXISTEIXI A LA TAULA
          SELECT ID_CENT_PROD
          INTO   V_ID_CENT_PROD
          FROM   CENTRALS_PRODUCCIO
          WHERE  ID_CENT_PROD = B_ID_CENT_PROD;
      
       -- VERIFIQUEM QUE LA CENTRAL DE PRODUCCIÓ NO TINGUI CENTRALETES DE DISTRIBUCIÓ ASSOCIADES A COMPTADORS ACTIUS (CONTADORS AMB CONTRACTES ASSOCIATS) 
          SELECT COUNT(CO.ID_COMPTADOR)
          INTO V_TOTAL
          FROM CONTRACTES CO
          WHERE CO.ID_COMPTADOR IN (
                  SELECT ID_COMPTADOR
                  FROM CON_DISTRIBUCIO_COMPTADOR REL 
                  WHERE REL.ID_CENTL_DIST IN (
                        SELECT ID_CENTL_DIST
                        FROM CON_PRODUCCIO_DISTRIBUCIO LC
                        WHERE LC.ID_CENT_PROD = B_ID_CENT_PROD));
        
          IF (V_TOTAL > 0)
          THEN
               RAISE EX_CENT_PROD_ACTIU;
          END IF;
          
          -- ESBORREM LES DADES DE LA CENTRAL DE PRODUCCIÓ 
          -- 1. TAULA HIST_CENTRALS_PRODUCCIO
          DELETE FROM HIST_CENTRALS_PRODUCCIO WHERE ID_CENT_PROD = B_ID_CENT_PROD;    
          -- 2. TAULA CON_PRODUCCIO_DISTRIBUCIO
          DELETE FROM CON_PRODUCCIO_DISTRIBUCIO WHERE ID_CENT_PROD = B_ID_CENT_PROD;    
          -- 3. TAULA CENTRALS_PRODUCCIO
          DELETE FROM CENTRALS_PRODUCCIO WHERE ID_CENT_PROD = B_ID_CENT_PROD;   
       
          -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
          
          -- ACTUALITZEM ESTADISTIQUES
          PKG_MODUL_ESTADISTIC.ESTADISTIQUES(R_RSP);
    
          
         EXCEPTION 
            WHEN EX_PK_NUL THEN
                 V_ERROR := -01001;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRAL PRODUCCIO';
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRAL PRODUCCIO: '||B_ID_CENT_PROD;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_CENT_PROD_ACTIU THEN
                 V_ERROR := -02003;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRAL PRODUCCIO: '||B_ID_CENT_PROD;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END BAIXA_CENT_PROD;
    
/* MODIFICACIÓ DE LES PROPIETATS D'UNA CENTRAL DE PRODUCCIÓ */
    PROCEDURE MODIF_CENT_PROD_PROPIETATS (
          M_ID_CENT_PROD IN CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
          M_TIPUS_CENTRAL IN CENTRALS_PRODUCCIO.TIPUS_CENTRAL%TYPE,
          M_ENERGIA_MIN IN CENTRALS_PRODUCCIO.ENERGIA_MIN%TYPE,
          M_REBUIG IN CENTRALS_PRODUCCIO.REBUIG%TYPE,
          M_EMISSIONS IN CENTRALS_PRODUCCIO.EMISSIONS%TYPE,
          M_MOLINS IN CENTRALS_PRODUCCIO.MOLINS%TYPE,
          M_PANELLS IN CENTRALS_PRODUCCIO.PANELLS%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 
  
  
         BEGIN
         V_PROCEDIMENT := 'MODIF_CENT_PROD_PROPIETATS';	
         V_PARAM_ENTRADA := 'CENT_PROD: '||M_ID_CENT_PROD||' / TIPUS_CENTRAL: '||M_TIPUS_CENTRAL||' / ENERGIA_MIN: '||M_ENERGIA_MIN||
		                        ' / REBUIG: '||M_REBUIG||' / EMISSIONS: '||M_EMISSIONS||' / MOLINS: '||M_MOLINS||
                            ' / M_PANELLS: '||M_PANELLS;
         R_RSP := 'OK'; 
    
  
        -- VERIFIQUEM QUE LA CENTRAL DE PRODUCCIÓ A MODIFICAR EXISTEIXI A LA TAULA
          SELECT ID_CENT_PROD
          INTO   V_ID_CENT_PROD
          FROM   CENTRALS_PRODUCCIO
          WHERE  ID_CENT_PROD = M_ID_CENT_PROD;
      
          CASE M_TIPUS_CENTRAL
          WHEN 'NUCLEAR' THEN
               -- VALIDEM SI LA CENTRAL ÉS 'NUCLEAR', S'HA DE INFORMAR EL CAMP ENERGIA_MIN I EL CAMPREBUIG (CONTÉ ELS KG DE REBUIG RADIOACTIU GENERAT)
               V_EMISSIONS := NULL;
               V_MOLINS := NULL;
               V_PANELLS := NULL;
               IF (M_ENERGIA_MIN IS NULL OR M_REBUIG IS NULL) 
               THEN 
                    RAISE EX_NUCLEAR;
               ELSE
                   V_ENERGIA_MIN := M_ENERGIA_MIN;
                   V_REBUIG := M_REBUIG;
               END IF;
         WHEN 'TÈRMICA' THEN
              -- VALIDEM SI LA CENTRAL ÉS TÈRMICA, S'HA DE INFORMAR EL CAMP EMISSIONS
              V_REBUIG := NULL;
              V_MOLINS := NULL;
              V_PANELLS := NULL;
              IF (M_EMISSIONS IS NULL)
              THEN
                    RAISE EX_TERMICA;
              ELSE
                    V_EMISSIONS := M_EMISSIONS;
              END IF;
         WHEN 'CARBÓ' THEN 
              -- VALIDEM SI LA CENTRAL ÉS CARBÓ, S'HA DE INFORMAR EL CAMP EMISSIONS
              V_REBUIG := NULL;
              V_MOLINS := NULL;
              V_PANELLS := NULL;
              IF (M_EMISSIONS IS NULL)
              THEN
                    RAISE EX_CARBO;
              ELSE
                    V_EMISSIONS := M_EMISSIONS;
              END IF;
         WHEN 'EÒLICA' THEN
              -- VALIDEM SI LA CENTRAL ÉS TÈRMICA, S'HA DE INFORMAR EL CAMP MOLINS
              V_REBUIG := NULL;
              V_EMISSIONS := NULL;
              V_PANELLS := NULL;
              IF (M_MOLINS IS NULL)
              THEN
                    RAISE EX_EOLICA;
              ELSE
                    V_MOLINS := M_MOLINS;
              END IF;
         WHEN 'SOLAR' THEN
              -- VALIDEM SI LA CENTRAL ÉS SOLAR, S'HA DE INFORMAR EL CAMP PANELLS
              V_REBUIG := NULL;
              V_MOLINS := NULL;
              V_EMISSIONS := NULL;
              IF (M_PANELLS IS NULL)
              THEN 
                    RAISE EX_SOLAR;
              ELSE
                    V_PANELLS := M_PANELLS;
              END IF;
         ELSE
            RAISE EX_TIPUS_NOK;
         END CASE;
         
          -- MODIFIQUEM LES DADES DE LA CENTRAL DE PRODUCCIÓ
          UPDATE CENTRALS_PRODUCCIO
          SET    ENERGIA_MIN = V_ENERGIA_MIN,
                 REBUIG = V_REBUIG,
                 EMISSIONS = V_EMISSIONS,
                 MOLINS = V_MOLINS,
                 PANELLS = V_PANELLS
          WHERE ID_CENT_PROD = M_ID_CENT_PROD;
      
          -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
          
         EXCEPTION 
            WHEN EX_PK_NUL THEN
                 V_ERROR := -01001;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRAL PRODUCCIÓ';
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRAL PRODUCCIÓ: '||M_ID_CENT_PROD;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_NUCLEAR THEN
                 V_ERROR := -02004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_TERMICA THEN
                 V_ERROR := -02005;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_CARBO THEN
                 V_ERROR := -02006;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_EOLICA THEN
                 V_ERROR := -02016;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_SOLAR THEN
                 V_ERROR := -02017;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_TIPUS_NOK THEN
                 V_ERROR := -01005;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
              WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END MODIF_CENT_PROD_PROPIETATS;

/* MODIFICACIÓ DE LES PROPIETATS D'UNA CENTRAL DE PRODUCCIÓ */
    PROCEDURE MODIF_CENT_PROD_LOCALITZACIO (
          M_ID_CENT_PROD IN CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
          M_TELEFON IN CENTRALS_PRODUCCIO.TELEFON%TYPE,
          M_DOMICILI IN CENTRALS_PRODUCCIO.DOMICILI%TYPE,
          M_CODI_POSTAL IN CENTRALS_PRODUCCIO.CODI_POSTAL%TYPE,
          M_LOCALITAT IN CENTRALS_PRODUCCIO.LOCALITAT%TYPE,
          M_PROVINCIA IN CENTRALS_PRODUCCIO.PROVINCIA%TYPE,
          M_ID_PAIS IN CENTRALS_PRODUCCIO.ID_PAIS%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS
	  
         BEGIN
         V_PROCEDIMENT := 'MODIFICACIO_CENT_PROD_LOCALITZACIO';	
         V_PARAM_ENTRADA := 'CENT_PROD: '||M_ID_CENT_PROD||' / TELEFON: '||M_TELEFON||' / DOMICILI: '||M_DOMICILI||
	                          ' / CODI POSTAL: '||M_CODI_POSTAL||'/ LOCALITAT : '|| M_LOCALITAT||' / PROVINCIA: '||M_PROVINCIA||
                            ' / PAIS: '||M_ID_PAIS;
         R_RSP := 'OK'; 
    
  
         -- VERIFIQUEM QUE LA CENTRAL DE PRODUCCIÓ A MODIFICAR EXISTEIXI A LA TAULA
          SELECT ID_CENT_PROD
          INTO   V_ID_CENT_PROD
          FROM   CENTRALS_PRODUCCIO
          WHERE  ID_CENT_PROD = M_ID_CENT_PROD;
      
          -- MODIFIQUEM LES DADES DE LA CENTRAL
          UPDATE CENTRALS_PRODUCCIO
          SET    TELEFON = M_TELEFON,
                 DOMICILI = M_DOMICILI,
                 CODI_POSTAL = M_CODI_POSTAL,
                 LOCALITAT = M_LOCALITAT,
                 PROVINCIA = M_PROVINCIA,
                 ID_PAIS = M_ID_PAIS
          WHERE ID_CENT_PROD = M_ID_CENT_PROD;
      
          -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
          
         EXCEPTION 
            WHEN EX_PK_NUL THEN
                 V_ERROR := -01001;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRAL PRODUCCIÓ';
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN EX_FK_NUL THEN
                 V_ERROR := -01002;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' .PAÍS';
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRAL PRODUCCIÓ: '||M_ID_CENT_PROD;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END MODIF_CENT_PROD_LOCALITZACIO;

/* MODIFICACIÓ L'ENERGÍA MÀXIMA PRODUÏDA PER UNA CENTRAL*/
    PROCEDURE MODIF_CENT_PROD_ENERG_MAX_PROD (
          M_ID_CENT_PROD IN CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
          M_ENERGIA_MAX_PROD IN CENTRALS_PRODUCCIO.ENERGIA_MAX_PROD%TYPE,
          R_RSP OUT NOCOPY LOGS.RSP%TYPE) IS 
  
    	  
        BEGIN
        V_PROCEDIMENT := 'MODIF_CENT_PROD_ENERG_PROD';	
        V_PARAM_ENTRADA := 'CENT_PROD: '||M_ID_CENT_PROD||' / ENERGIA_MAX_PROD: '||M_ENERGIA_MAX_PROD;
        R_RSP := 'OK'; 
    
  
          -- VERIFIQUEM QUE LA CENTRAL DE PRODUCCIÓ A MODIFICAR EXISTEIXI A LA TAULA
          SELECT ID_CENT_PROD
          INTO   V_ID_CENT_PROD
          FROM   CENTRALS_PRODUCCIO
          WHERE  ID_CENT_PROD = M_ID_CENT_PROD;
      
           -- MODIFIQUEM L'ENERGIA MÀXIMA PRODUÏDA PER LA CENTRAL DE PRODUCCIÓ
          UPDATE CENTRALS_PRODUCCIO
          SET    ENERGIA_MAX_PROD = M_ENERGIA_MAX_PROD
          WHERE  ID_CENT_PROD = M_ID_CENT_PROD;
      
          -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
          
         EXCEPTION 
            WHEN EX_PK_NUL THEN
                 V_ERROR := -01001;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRAL PRODUCCIÓ';
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRAL PRODUCCIÓ: '||M_ID_CENT_PROD;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
            WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
    END MODIF_CENT_PROD_ENERG_MAX_PROD;   
    
/* ALTA HISTÒRIC PRODUCCIO */
    PROCEDURE ALTA_HIST_CENT_PROD (
          A_ID_CENT_PROD IN HIST_CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
          A_DATA_MENSUAL IN HIST_CENTRALS_PRODUCCIO.DATA_MENSUAL%TYPE,
          A_ENERGIA_PROD IN HIST_CENTRALS_PRODUCCIO.ENERGIA_PROD%TYPE,
          R_RSP          OUT NOCOPY LOGS.RSP%TYPE) IS
          
          BEGIN
          V_PROCEDIMENT := 'ALTA_HIST_CENT_PROD';	
          V_PARAM_ENTRADA := 'CENT_PROD: '||A_ID_CENT_PROD||' DATA_MENSUAL: '||A_DATA_MENSUAL||
                            '/ ENERGIA_PROD: '||A_ENERGIA_PROD;
          R_RSP := 'OK'; 
          
          --VERIFIQUEM QUE EL REGISTRE NO EXISTEIXI
          SELECT COUNT(1)
          INTO   V_TOTAL
          FROM   HIST_CENTRALS_PRODUCCIO
          WHERE  ID_CENT_PROD = A_ID_CENT_PROD AND TO_CHAR(DATA_MENSUAL,'DD/MM/YYYY') = TO_CHAR(A_DATA_MENSUAL,'DD/MM/YYYY');
           
          --INSERTEM EL REGISTRE
          INSERT INTO HIST_CENTRALS_PRODUCCIO
          VALUES(A_ID_CENT_PROD, A_DATA_MENSUAL, A_ENERGIA_PROD, NULL, NULL, NULL, NULL);
          
          -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
          
          -- ACTUALITZEM LES ESTADÍSTIQUES
          PKG_MODUL_ESTADISTIC.MODUL_ESTADISTIC_01(R_RSP);
          PKG_MODUL_ESTADISTIC.MODUL_ESTADISTIC_05(R_RSP);
          
          EXCEPTION
          WHEN EX_PK_NUL THEN
                 V_ERROR := -01001;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRAL PRODUCCIÓ';
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN DUP_VAL_ON_INDEX THEN
                 V_ERROR := -01003;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' . CENTRAL PRODUCCIÓ: '||A_ID_CENT_PROD||' / DATA_MENSUAL: '||A_DATA_MENSUAL;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
         END ALTA_HIST_CENT_PROD;
        
        
/* BAIXA HISTÒRIC PRODUCCIO */
    PROCEDURE BAIXA_HIST_CENT_PROD (
          B_ID_CENT_PROD IN HIST_CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
          B_DATA_MENSUAL IN HIST_CENTRALS_PRODUCCIO.DATA_MENSUAL%TYPE,
          R_RSP          OUT NOCOPY LOGS.RSP%TYPE) IS
          
          BEGIN
          V_PROCEDIMENT := 'BAIXA_HIST_CENT_PROD';	
          V_PARAM_ENTRADA := 'CENT_PROD: '||B_ID_CENT_PROD||' DATA_MENSUAL: '||B_DATA_MENSUAL;                       
          R_RSP := 'OK'; 
          
          --VERIFIQUEM QUE EL REGISTRE EXISTEIXI
          SELECT COUNT(1)
          INTO   V_TOTAL
          FROM   HIST_CENTRALS_PRODUCCIO
          WHERE  ID_CENT_PROD = B_ID_CENT_PROD AND TO_CHAR(DATA_MENSUAL,'DD/MM/YYYY') = TO_CHAR(B_DATA_MENSUAL,'DD/MM/YYYY');
                     
          IF (V_TOTAL = 0)
          THEN
              RAISE NO_DATA_FOUND;
          END IF;
          
          --ESBORREM EL REGISTRE
          DELETE FROM HIST_CENTRALS_PRODUCCIO
          WHERE  ID_CENT_PROD = B_ID_CENT_PROD AND TO_CHAR(DATA_MENSUAL,'DD/MM/YYYY') = TO_CHAR(B_DATA_MENSUAL,'DD/MM/YYYY');
    
                 -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
          
          -- ACTUALITZEM LES ESTADÍSTIQUES
          PKG_MODUL_ESTADISTIC.MODUL_ESTADISTIC_01(R_RSP);
          PKG_MODUL_ESTADISTIC.MODUL_ESTADISTIC_05(R_RSP);
          
          EXCEPTION
          WHEN EX_PK_NUL THEN
                 V_ERROR := -01001;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRAL PRODUCCIÓ';
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' . CENTRAL PRODUCCIÓ: '||B_ID_CENT_PROD;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
         END BAIXA_HIST_CENT_PROD;         
    
/* MODIFICACIÓ HISTÒRIC PRODUCCIO */
    PROCEDURE MODIF_HIST_CENT_PROD (
          M_ID_CENT_PROD IN HIST_CENTRALS_PRODUCCIO.ID_CENT_PROD%TYPE,
          M_DATA_MENSUAL IN HIST_CENTRALS_PRODUCCIO.DATA_MENSUAL%TYPE,
          M_ENERGIA_PROD IN HIST_CENTRALS_PRODUCCIO.ENERGIA_PROD%TYPE,
          R_RSP          OUT NOCOPY LOGS.RSP%TYPE) IS
          
          BEGIN
          V_PROCEDIMENT := 'MODIF_HIST_CENT_PROD';	
          V_PARAM_ENTRADA := 'CENT_PROD: '||M_ID_CENT_PROD||' DATA_MENSUAL: '||M_DATA_MENSUAL||
                            '/ ENERGIA_PROD: '||M_ENERGIA_PROD;
          R_RSP := 'OK'; 
          
          --VERIFIQUEM QUE EL REGISTRE EXISTEIXI
          SELECT COUNT(1)
          INTO   V_TOTAL
          FROM   HIST_CENTRALS_PRODUCCIO
          WHERE  ID_CENT_PROD = M_ID_CENT_PROD AND TO_CHAR(DATA_MENSUAL,'DD/MM/YYYY') = TO_CHAR(M_DATA_MENSUAL,'DD/MM/YYYY');
          
          IF (V_TOTAL = 0)
          THEN
              RAISE NO_DATA_FOUND;
          END IF;
          
          --MODIFIQUEM EL REGISTRE
          UPDATE HIST_CENTRALS_PRODUCCIO
          SET    ENERGIA_PROD = M_ENERGIA_PROD
          WHERE  ID_CENT_PROD = M_ID_CENT_PROD AND TO_CHAR(DATA_MENSUAL,'DD/MM/YYYY') = TO_CHAR(M_DATA_MENSUAL,'DD/MM/YYYY');
    
           -- INSERTEM LA CRIDA REALITZADA A LA TAULA DE LOGS
          INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
          COMMIT;
          
          -- ACTUALITZEM LES ESTADÍSTIQUES
          PKG_MODUL_ESTADISTIC.MODUL_ESTADISTIC_01(R_RSP);
          PKG_MODUL_ESTADISTIC.MODUL_ESTADISTIC_05(R_RSP);
          
          EXCEPTION
          WHEN EX_PK_NUL THEN
                 V_ERROR := -01001;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' .CENTRAL PRODUCCIÓ';
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN NO_DATA_FOUND THEN
                 V_ERROR := -01004;
                 -- OBTENIM EL MISSATGE D'ERROR MITJANÇANT LA CRIDA A LA FUNCIÓ MISSATGE_ERROR
                 V_DESC := MISSATGE_ERROR(V_ERROR);
                 R_RSP := V_ERROR|| ': '||V_DESC||' . CENTRAL PRODUCCIÓ: '||M_ID_CENT_PROD;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
          WHEN OTHERS THEN
                 V_ERROR := SQLCODE;
                 V_DESC := SUBSTR(SQLERRM, 1, 256);
                 R_RSP := V_ERROR|| ': '||V_DESC;
                 -- INSERTEM L'ERROR A LA TAULA DE LOGS
                 INSERT INTO LOGS VALUES (NULL, V_PROCEDIMENT, V_PARAM_ENTRADA, NULL, NULL, NULL, R_RSP);
                 DBMS_OUTPUT.PUT_LINE(V_PROCEDIMENT|| ': '||R_RSP);
         END MODIF_HIST_CENT_PROD;   
         
END PKG_ABM_PRODUCCIO;
/
/**************************************************************************************************************************************************
**************************************************************************************************************************************************/        
COMMIT;

